﻿//*************************************************}
//                                                 }
//       Class: Tectus LF-Reader                   }
//       Programmed with Visual Studio 2008        }
//       Tectus Transponder Technology GmbH        }
//       Sascha Haveloh, s.haveloh@tec-tus.de      }
//                                                 }
//*************************************************}
// Pocker PC DLL...für TLM-30 Reader
// 09.01.2009, 21.01.2009
// aktuelle Firmware Versionen
// - btrw-hdx.v1.40.frm fuer alles TIRIS, UNIQUE32/64, ZOODIAC
// - btrw-rw.v1.40.frm fuer Q5, HITAG-S(1), HITAG-II, EM4305
// - btrw-ti.v1.40.frm fuer TITAN, UNIQUE32/64, ZOODIAC
// - bt-psk.v1.72  04.08.2011 neue firmwareversion (kontrolle auf bt-psk) TAG-TYP 8 = PSK 
//  04.11.2009 GetVersion beim 1 x Starten des Reades kann es vorkommen das ein "LF" am Anfang steht

// 26.11.2012..umbau auf C#...unterstützt wird jetzt LF...mit einer Firmwareversion
// 28.11.2012...umbau auf PDA

using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Forms;
using System.Runtime;
using System.Runtime.InteropServices;
using System.Threading;
using System.IO;


namespace BT_LF
{
    public class BT_LF : Form
    {

        public BT_LF() 
        {
            Init();
        }


        ~BT_LF() 
        {
            DeInit();
        }

        public sealed class ControlChars
        {
            public const char Back = '\b';
            public const char Cr = '\r';
            public const string CrLf = "\r\n";
            public const char FormFeed = '\f';
            public const char Lf = '\n';
            public const string NewLine = "\r\n";
            public const char NullChar = '\0';
            public const char Quote = '"';
            public const char Tab = '\t';
            public const char VerticalTab = '\v';
        }
        #region "Variablen"
        private string _DLLVersion = "BT_LF V1.1";
        private int TLMCommand = -1;
        private string fTagString = "";
        private string scanline = "";
        private int fTagType = -1;
        private bool fIsTagSelect = false;
        private string fTitanPassword = "00000000";
        private string fHitag2Password = "4D494B52";
        private string fHDXCRC = "";
        private string HitagUID = "";
        private string Hitag2UID = "";
        private string Hitag2UID1 = "";
        private string fLastException = "";
        private string fErrorReceived = "";
        private string fPinInfo = "";

        private System.IO.Ports.SerialPort SerialPort1 = new System.IO.Ports.SerialPort();
        private System.Windows.Forms.Timer DelayTimer = new System.Windows.Forms.Timer();
        #endregion

        #region "Propertys"

        public string TitanPassword
        {
            get { return fTitanPassword; }
            set { fTitanPassword = value; }
        }

        public string Hitag2Password
        {
            get { return fHitag2Password; }
            set { fHitag2Password = value; }
        }

        public string HDXCRC
        {
            get { return fHDXCRC; }
            set { fHDXCRC = value; }
        }

        public int GetLastTransponderTyp
        {
            get { return fTagType; }

        }

        public string GetLastTransponderNr
        {
            get { return fTagString; }
        }

        public bool IsTagSelect
        {
            get { return fIsTagSelect; }
        }

        public int TimeOut
        {
            get { return DelayTimer.Interval; }
            set { DelayTimer.Interval = value; }
        }

        public string DLLVersion
        {
            get { return _DLLVersion; }
        }

        public string LastException
        {
            get { return fLastException; }
        }


        public bool CTSInfo
        {
            get { return SerialPort1.CtsHolding; }
        }

        public bool CDInfo
        {
            get { return SerialPort1.CDHolding; }
        }

        public bool DSRInfo
        {
            get { return SerialPort1.DsrHolding; }
        }

        public bool DTR
        {
            get { return SerialPort1.DtrEnable; }
            set { SerialPort1.DtrEnable = value; }
        }

        public bool RTS
        {
            get { return SerialPort1.RtsEnable; }
            set { SerialPort1.RtsEnable = value; }
        }

        public System.IO.Ports.StopBits StopBits
        {
            get { return SerialPort1.StopBits; }
            set { SerialPort1.StopBits = value; }
        }

        public int DataBits
        {
            get { return SerialPort1.DataBits; }
            set { SerialPort1.DataBits = value; }
        }

        public System.IO.Ports.Handshake Handshake
        {
            get { return SerialPort1.Handshake; }
            set { SerialPort1.Handshake = value; }
        }

        public System.IO.Ports.Parity Parity
        {
            get { return SerialPort1.Parity; }
            set { SerialPort1.Parity = value; }
        }

        public int BaudRate
        {
            get { return SerialPort1.BaudRate; }
            set { SerialPort1.BaudRate = value; }
        }

        public string Comport
        {
            get { return SerialPort1.PortName; }
            set { SerialPort1.PortName = value; }
        }

        public int WriteTimeOut
        {
            get { return SerialPort1.WriteTimeout; }
            set { SerialPort1.WriteTimeout = value; }
        }

        public int ReadTimeOut
        {
            get { return SerialPort1.ReadTimeout; }
            set { SerialPort1.ReadTimeout = value; }
        }

        public bool Active
        {
            get { return SerialPort1.IsOpen; }
            set
            {
                if (value)
                {
                    try
                    {
                        SerialPort1.ReadTimeout = 1000;
                        SerialPort1.WriteTimeout = 1000;
                        SerialPort1.Open();
                        SerialPort1.DtrEnable = true;
                        SerialPort1.RtsEnable = true;
                    }
                    catch (Exception ex)
                    {
                        fLastException = ex.Message.ToString();
                    }
                }
                else
                {
                    try
                    {
                        SerialPort1.DtrEnable = false;
                        SerialPort1.RtsEnable = false;
                        SerialPort1.Close();
                    }
                    catch (Exception ex)
                    {
                        fLastException = ex.Message.ToString();
                    }
                }
            }
        }

        #endregion

        #region "Events"

        public event GetTLMDataEventHandler TLMData;
        public delegate void GetTLMDataEventHandler(object sender, string Direction, string Data);

        public event GetTLMTransponderEventHandler TLMTransponder;
        public delegate void GetTLMTransponderEventHandler(object sender, int TagTyp, string TagNr);


        public event GetPortErrorReceivedEventHandler PortErrorReceived;
        public delegate void GetPortErrorReceivedEventHandler(object sender, string Error);

        public event GetPortPinChangedEventHandler PortPinChanged;
        public delegate void GetPortPinChangedEventHandler(object sender, string PinInfo);

        #endregion

        #region "AddEvents"
        private void AddEvents()
        {
            this.SerialPort1.DataReceived += new System.IO.Ports.SerialDataReceivedEventHandler(this.SerialPort1_DataReceived);
            this.SerialPort1.PinChanged += new System.IO.Ports.SerialPinChangedEventHandler(this.SerialPort1_PinChanged);
            this.SerialPort1.ErrorReceived += new System.IO.Ports.SerialErrorReceivedEventHandler(this.SerialPort1_ErrorReceived);
            this.DelayTimer.Tick += new System.EventHandler(this.DelayTimer_Tick);
        }
        #endregion

        #region "Function Private"
        /*private bool CheckTag(int TagType)
        {
            bool Result = false;
            switch (TagType)
            {
                case 1:
                    Result = (fTagType == 1) & IsTagSelect;
                    if (!Result)
                    {
                        scanline = "";
                        SerialPort1.WriteLine("@hv" + ControlChars.Cr);
                        scanline = "";
                        SerialPort1.WriteLine("@hS" + ControlChars.Cr);
                        if (scanline.IndexOf("+ ") >= 0)
                        {
                            fIsTagSelect = true;
                            fTagType = 1;
                            Result = true;
                        }
                    }
                    break;
                case 2:
                    break;
            }
            return Result;
        }*/

        private void SetTransponderInfo(int TagTyp, string TagNr)
        {
            if (TLMTransponder != null) { TLMTransponder(this, TagTyp, TagNr); }
        }

        private void SetReaderText(string Direction, string data)
        {
            if (TLMData != null) { TLMData(this, Direction, data); }
        }


        private void MyDataReceivedAll(object sender, System.EventArgs e)
        {
            SetReaderText("<", scanline);
            ComRecNeu(scanline);
        }


        private void ComRecNeu(string RecBuff)
        {
            int pos = 0;
            string tmpUID = string.Empty;
            if (RecBuff.Length > 0)
            {
                if (TLMCommand == 99)
                {
                    DelayTimer.Enabled = false;
                    fTagString = RecBuff;
                    return;
                }

                if (RecBuff.IndexOf("Error: 54") >= 0) // ERROR: Unknown binary operator Like
                {
                    DelayTimer.Enabled = false;
                    fTagString = RecBuff.Trim();
                    return;
                }

                if ((RecBuff.IndexOf("- 1") >= 0) | (RecBuff.IndexOf("- 2") >= 0) | (RecBuff.IndexOf("- 3") >= 0) | (RecBuff.IndexOf("- 4") >= 0) | (RecBuff.IndexOf("- 5") >= 0) | (RecBuff.IndexOf("- 6") >= 0))
                {
                    DelayTimer.Enabled = false;
                    fTagString = RecBuff;
                    return;
                }
                //Command sv
                if (TLMCommand == 10)
                {
                    DelayTimer.Enabled = false;
                    fTagString = RecBuff;
                    return;                
                }
                else if (TLMCommand == 0) //0 - EM 41xx (UNIQUE) - RO
                {
                    if (RecBuff.IndexOf("+ ") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        pos = RecBuff.IndexOf(" ") + 1;
                        fTagString = RecBuff.Substring(pos, 10);
                        fIsTagSelect = true;
                        SetTransponderInfo(fTagType, fTagString);
                        return;
                    }
                }
                else if (TLMCommand == 1) //1 - HITAG S - RW
                {
                    if (RecBuff.IndexOf("+ ") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        pos = RecBuff.IndexOf(" ") + 1;
                        if ((HitagUID.Length >= 8))
                        {
                            tmpUID = HitagUID.Substring(6, 2) + HitagUID.Substring(4, 2) + HitagUID.Substring(2, 2) + HitagUID.Substring(0, 2);
                            fTagString = tmpUID;
                            fIsTagSelect = true;
                            SetTransponderInfo(fTagType, fTagString);
                            return;
                        }
                    }                   
                }
                else if (TLMCommand == 2) //2 - FDX-B ISO 11784/5-RW
                {
                    if (RecBuff.IndexOf("+ ") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        pos = RecBuff.IndexOf(" ") + 1;
                        fTagString = RecBuff.Substring(pos, 16);
                        fIsTagSelect = true;
                        SetTransponderInfo(fTagType, fTagString);
                        return;
                    }                  
                }
                else if (TLMCommand == 3) //3 - Q5 / 5557 -- RW
                {
                    if (RecBuff.IndexOf("+ ") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        pos = RecBuff.IndexOf(" ") + 1;
                        fTagString = RecBuff.Substring(pos, 8);
                        fIsTagSelect = true;
                        SetTransponderInfo(fTagType, fTagString);
                        return;
                    }                    
                }
                else if ((TLMCommand == 4)) //4 - HDX Transponder (TI)-RW
                {
                    //+ r 000000000617DED6 FFD0 + 7E
                    //+ m 0000000000000000 0000 + 06
                    if ((RecBuff.IndexOf("+ r") >= 0) | (RecBuff.IndexOf("+ m") >= 0) | (RecBuff.IndexOf("+ w") >= 0))  // ERROR: Unknown binary operator Like
                    {
                        DelayTimer.Enabled = false;
                        fTagString = RecBuff.Substring(1, 1) + RecBuff.Substring(3, 16);
                        HDXCRC = RecBuff.Substring(20, 4);
                        fIsTagSelect = true;
                        SetTransponderInfo(fTagType, fTagString);
                        return;
                    }
                }
                else if (TLMCommand == 5) //5 - HITAG 2 - RW 
                {
                    if (RecBuff.IndexOf("+ ") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        pos = RecBuff.IndexOf(" ") + 1;
                        if (RecBuff.Length >= 10)
                        {
                            tmpUID = RecBuff.Substring(1, 8);
                            if ((tmpUID == Hitag2UID) | (tmpUID == Hitag2UID1))
                            {
                                fTagString = tmpUID;
                                SetTransponderInfo(fTagType, fTagString);
                                fIsTagSelect = true;
                                return;
                            }
                            else
                            {
                                fTagString = "- 1";
                            }
                        }
                    }
                }
                else if (TLMCommand == 6) //6 - EM 4305 - RW 
                {
                    if (RecBuff.IndexOf("+ ") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        pos = RecBuff.IndexOf(" ") + 1;
                        fTagString = RecBuff.Substring(pos, 8);
                        fIsTagSelect = true;
                        SetTransponderInfo(fTagType, fTagString);
                        return;
                    }                    
                }
                else if (TLMCommand == 7) //7 - EM4450/4550 (TITAN)-RW
                {
                    if (RecBuff.IndexOf("+ ") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        pos = RecBuff.IndexOf(" ") + 1;
                        fTagString = RecBuff.Substring(pos, 8);
                        fIsTagSelect = true;
                        SetTransponderInfo(fTagType, fTagString);
                        return;
                    }
                    if (RecBuff.IndexOf("- 3") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        fTagString = "- 3";
                    }
                }
                else if (TLMCommand == 11) //1 - HITAG S - Data Read
                {
                    if (RecBuff.IndexOf("+ ") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        pos = RecBuff.IndexOf(" ") + 1;
                        fTagString = RecBuff.Substring(pos, 8);
                        return;
                    }
                }
                else if (TLMCommand == 31) //3 - Q5 / 5557 - Data Read 
                {
                    if (RecBuff.IndexOf("+ ") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        pos = RecBuff.IndexOf(" ") + 1;
                        fTagString = RecBuff.Substring(pos, 8);
                        return;
                    }                    
                }
                else if (TLMCommand == 41) //4 - HDX Transponder (TI)- Data read
                {
                    //+ r 000000000617DED6 FFD0 + 7E
                    //+ m 0000000000000000 0000 + 06
                    if ((RecBuff.IndexOf("+ r") >= 0) | (RecBuff.IndexOf("+ m") >= 0))
                    {
                        DelayTimer.Enabled = false;
                        fTagString = RecBuff.Substring(3, 16);
                        HDXCRC = RecBuff.Substring(20, 4);
                        return;
                    }                    
                }
                else if (TLMCommand == 51) //5 - HITAG 2 - Data Read
                {
                    if (RecBuff.IndexOf("+ ") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        pos = RecBuff.IndexOf(" ") + 1;
                        fTagString = RecBuff.Substring(pos, 8);
                        return;
                    }                    
                }
                else if (TLMCommand == 61) //6 - EM 4305 - Data Read
                {
                    if (RecBuff.IndexOf("+ ") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        pos = RecBuff.IndexOf(" ") + 1;
                        fTagString = RecBuff.Substring(pos, 8);
                        return;
                    }                    
                }
                else if (TLMCommand == 71) //7 - EM4450/4550 (TITAN)- Data Read
                {
                    if (RecBuff.IndexOf("+ ") >= 0)
                    {
                        DelayTimer.Enabled = false;
                        pos = RecBuff.IndexOf(" ") + 1;
                        fTagString = RecBuff.Substring(pos, 8);
                        return;
                    }                    
                }
                else if (TLMCommand == 12) //1 - HITAG S - Data Write
                {
                    DelayTimer.Enabled = false;
                    fTagString = RecBuff;                    
                }
                else if (TLMCommand == 32) //3 - Q5 / 5557 - Data Write
                {
                    DelayTimer.Enabled = false;
                    fTagString = RecBuff;                    
                }
                else if (TLMCommand == 42) //'4 - HDX Transponder (TI)-RW Data Write
                {
                    DelayTimer.Enabled = false;
                    if (RecBuff.IndexOf("+") >= 0)
                    {
                        fTagString = "+";
                    }
                    fTagString = RecBuff;
                }
                else if (TLMCommand == 52) //5 - HITAG 2 - Data Write
                {
                    DelayTimer.Enabled = false;
                    fTagString = RecBuff;                    
                }
                else if (TLMCommand == 62) //6 - EM 4305 - Data Write
                {
                    DelayTimer.Enabled = false;
                    fTagString = RecBuff;                    
                }
                else if (TLMCommand == 72) //7 - EM4450/4550 (TITAN)- Data Write
                {
                    DelayTimer.Enabled = false;
                    fTagString = RecBuff;
                }
            }
        }

        private void HitagS()
        {
            //hv; hS;
            try
            {
                fTagString = "";
                HitagUID = "";
                TLMCommand = 99;
                scanline = "";
                SerialPort1.WriteLine("@hv" + ControlChars.Cr);
                System.Windows.Forms.Application.DoEvents();
                System.Threading.Thread.Sleep(200);
                System.Windows.Forms.Application.DoEvents();
                fTagString = scanline;
                if (!string.IsNullOrEmpty(fTagString))
                {
                    HitagUID = fTagString.Trim();
                }
                TLMCommand = 3;
                scanline = "";
                SerialPort1.WriteLine("@hS" + ControlChars.Cr);
                DelayTimer.Enabled = true;
            }
            catch (Exception ex)
            {
                fLastException = ex.Message.ToString();
            }
        }

        private void SendFData_COM(char[] RData, int len)
        {
            // for (int i = 0; i <= len; i++) {SerialPort1.Write(RData, i, 1);}
            SerialPort1.Write(RData, 0, RData.Length);
        }


        private bool SendTagString(string RData)
        {
            bool Result = false;
            SetReaderText(">", RData);
            fTagString = "";
            scanline = "";
            RData = "@" + RData + ControlChars.Cr;
            SendFData_COM(RData.ToCharArray(), RData.Length - 1);
            //SerialPort1.WriteLine(RData);
            DelayTimer.Enabled = true;
            while (DelayTimer.Enabled)
            {
                System.Windows.Forms.Application.DoEvents();
            }
            Result = !string.IsNullOrEmpty(fTagString);
            return Result;
        }

        private string GetHDXCRC(string HDXData)
        {
            string tmp1 = null;
            string tmp2 = null;
            long hi32 = 0;
            long lo32 = 0;
            long crc = 0;
            string CRCHex = "";
            if (HDXData.Length != 16)
            {
                return CRCHex;
            }
            tmp1 = HDXData.Substring(0, 8);
            tmp2 = HDXData.Substring(8, 8);
            hi32 = Convert.ToInt32(tmp1, 16);
            lo32 = Convert.ToInt32(tmp2, 16);
            crc = compute_crc(hi32, lo32);
            CRCHex = crc.ToString("X4");
            return CRCHex;
        }

        private long compute_crc(long hi32, long lo32)
        {
            const long CRC_POLYNOME = 0x8408;
            long crc = 0;
            int i = 0;
            for (i = 0; i <= 31; i++)
            {
                // es geht mit LSB vorne

                if (((lo32 ^ crc) & 1) == 1)
                {
                    crc = crc >> 1;
                    // Verschiebung nach rechts
                    crc = crc ^ CRC_POLYNOME;
                }
                else
                {
                    crc = crc >> 1;
                }

                lo32 = lo32 >> 1;
            }

            for (i = 0; i <= 31; i++)
            {
                // genau wie vorhin
                if (((hi32 ^ crc) & 1) == 1)
                {
                    crc = crc >> 1;
                    crc = crc ^ CRC_POLYNOME;
                }
                else
                {
                    crc = crc >> 1;
                }

                hi32 = hi32 >> 1;
            }
            return crc;
        }

        private string DeleteChars(string Value)
        {
            int pos = 0;
            string tmp = null;

            pos = Value.IndexOf(ControlChars.Cr);
            if (pos != -1)
            {
                Value = Value.Remove(pos, 1);
            }

            pos = Value.IndexOf(ControlChars.Lf);
            if (pos != -1)
            {
                Value = Value.Remove(pos, 1);
            }
            pos = Value.IndexOf(ControlChars.CrLf);
            if (pos != -1)
            {
                Value = Value.Remove(pos, 2);
            }
            tmp = Value;
            return tmp;
        }


        #endregion

        #region "Function Public"

        public string GetComports()
        {
            string result = "";
            string[] myComports;
            int i;
            myComports = System.IO.Ports.SerialPort.GetPortNames();
            for (i = 0; i < myComports.Length; i++)
            {
                result = result + myComports[i] + ControlChars.Cr;
            }
            return result;
        }

        public string GetTransponderTyp(int TagTyp)
        {
            //'0 - EM 41xx (UNIQUE) - RO
            //'1 - HITAG S - RW
            //'2 - FDX-B ISO 11784/5-RW
            //'3 - Q5 / 5557 -- RW
            //'4 - HDX Transponder (TI)-RW
            //'5 - HITAG 2 - RW
            //'6 - EM 4305 - RW
            //'7 - EM4450/4550 (TITAN)-RW
            //'8 - PSK - RO

            string result = "";
            switch (TagTyp)
            {
                case -1:
                    result = "none";
                    break;
                case 0:
                    result = "EM 41xx (UNIQUE) - RO";
                    break;
                case 1:
                    result = "HITAG S - RW";
                    break;
                case 2:
                    result = "FDX-B ISO 11784/5-RW";
                    break;
                case 3:
                    result = "Q5 / 5557 -- RW";
                    break;
                case 4:
                    result = "HDX Transponder (TI)-RW";
                    break;
                case 5:
                    result = "HITAG 2 - RW";
                    break;
                case 6:
                    result = "EM 4305 - RW";
                    break;
                case 7:
                    result = "EM4450/4550 (TITAN)-RW";
                    break;
                case 8:
                    result = "PSK - RO";
                    break;
            }
            return result;
        }


        public string WriteTagData(int TagType, string Block, string Data)
        {
            string Result = "";
            switch (TagType)
            {
                case -1:
                    break;
                case 0:
                    //0 - EM 41xx (UNIQUE) - RO
                    Result = "- 9";
                    break;
                case 1:
                    //1 - HITAG S - RW
                    TLMCommand = 12;
                    if (IsTagSelect)
                    {
                        if (Data.Length != 8)
                        {
                            Result = "- 3";
                            return Result;
                        }
                        if (SendTagString("hw " + Block + " 0x" + Data))
                        {
                            Result = fTagString;
                        }
                        else
                        {
                            Result = "- 2";
                        }
                    }
                    else
                    {
                        Result = "- 1";
                    }
                    break;
                case 2:
                    //2 - FDX-B ISO 11784/5-RW
                    Result = "- 9";
                    break;
                case 3:
                    //3 - Q5 / 5557 -- RW
                    TLMCommand = 32;
                    if (IsTagSelect)
                    {
                        if (Data.Length != 8)
                        {
                            Result = "- 3";
                            return Result;
                        }
                        if (SendTagString("qw " + Block + " 0x" + Data))
                        {
                            Result = fTagString;
                        }
                        else
                        {
                            Result = "- 2";
                        }
                    }
                    else
                    {
                        Result = "- 1";
                    }
                    break;
                case 4:
                    //4 - HDX Transponder (TI)-RW
                    TLMCommand = 42;
                    //xmp 0x10 0x1111111111111111 0x8456  // write Page 10 mit Checksumme
                    if (IsTagSelect)
                    {
                        if (Data.Length != 16)
                        {
                            Result = "- 3";
                            return Result;
                        }
                        HDXCRC = GetHDXCRC(Data);
                        if (SendTagString("xmp " + Block + " 0x" + Data + " 0x" + HDXCRC))
                        {
                            Result = fTagString;
                        }
                        else
                        {
                            Result = "- 2";
                        }
                    }
                    else
                    {
                        Result = "- 1";
                    }
                    break;
                case 5:
                    //5 - HITAG 2 - RW
                    TLMCommand = 52;
                    if (IsTagSelect)
                    {
                        if (Data.Length != 8)
                        {
                            Result = "- 3";
                            return Result;
                        }
                        if (SendTagString("kw " + Block + " 0x" + Data))
                        {
                            Result = fTagString;
                        }
                        else
                        {
                            Result = "- 2";
                        }
                    }
                    else
                    {
                        Result = "- 1";
                    }
                    break;
                case 6:
                    //6 - EM 4305 - RW
                    TLMCommand = 62;
                    if (IsTagSelect)
                    {
                        if (Data.Length != 8)
                        {
                            Result = "- 3";
                            return Result;
                        }
                        if (SendTagString("vw " + Block + " 0x" + Data))
                        {
                            Result = fTagString;
                        }
                        else
                        {
                            Result = "- 2";
                        }
                    }
                    else
                    {
                        Result = "- 1";
                    }
                    break;
                case 7:
                    //7 - EM4450/4550 (TITAN)-RW
                    TLMCommand = 72;
                    if (IsTagSelect)
                    {
                        if (Data.Length != 8)
                        {
                            Result = "- 3";
                            return Result;
                        }
                        if (SendTagString("tw " + Block + " 0x" + Data))
                        {
                            Result = fTagString;
                        }
                        else
                        {
                            Result = "- 2";
                        }
                    }
                    else
                    {
                        Result = "- 1";
                    }
                    break;
            }
            return Result;
        }

        public string ReadTagData(int TagType, string Block)
        {
            string Result = "";
            switch (TagType)
            {
                case -1:
                    break;
                case 0:
                    //0 - EM 41xx (UNIQUE) - RO
                    break;
                case 1:
                    //1 - HITAG S - RW
                    TLMCommand = 11;
                    if (IsTagSelect)
                    {
                        if (SendTagString("hr " + Block))
                        {
                            Result = fTagString;
                        }
                        else
                        {
                            Result = "- 2";
                        }
                    }
                    else
                    {
                        Result = "- 1";
                    }
                    break;
                case 2:
                    //2 - FDX-B ISO 11784/5-RW
                    break;
                case 3:
                    //3 - Q5 / 5557 -- RW
                    TLMCommand = 31;
                    if (IsTagSelect)
                    {
                        if (SendTagString("qr " + Block))
                        {
                            Result = fTagString;
                        }
                        else
                        {
                            Result = "- 2";
                        }
                    }
                    else
                    {
                        Result = "- 1";
                    }
                    break;
                case 4:
                    //4 - HDX Transponder (TIRIS)-RW
                    TLMCommand = 41;
                    if (SendTagString("xmg " + Block))
                    {
                        Result = fTagString;
                    }
                    else
                    {
                        Result = "- 1";
                    }
                    break;
                case 5:
                    //5 - HITAG 2 - RW
                    TLMCommand = 51;
                    if (IsTagSelect)
                    {
                        if (SendTagString("kr " + Block))
                        {
                            Result = fTagString;
                        }
                        else
                        {
                            Result = "- 2";
                        }
                    }
                    else
                    {
                        Result = "- 1";
                    }
                    break;
                case 6:
                    //6 - EM 4305 - RW
                    TLMCommand = 61;
                    if (IsTagSelect)
                    {
                        if (SendTagString("vr " + Block))
                        {
                            Result = fTagString;
                        }
                        else
                        {
                            Result = "- 2";
                        }
                    }
                    else
                    {
                        Result = "- 1";
                    }
                    break;
                case 7:
                    //7 - EM4450/4550 (TITAN)-RW
                    TLMCommand = 71;
                    if (IsTagSelect)
                    {
                        if (SendTagString("tr " + Block))
                        {
                            Result = fTagString;
                        }
                        else
                        {
                            Result = "- 2";
                        }
                    }
                    else
                    {
                        Result = "- 1";
                    }
                    break;
            }
            return Result;
        }


        //0 - EM 41xx (UNIQUE) - RO
        //1 - HITAG S - RW
        //2 - FDX-B ISO 11784/5-RW
        //3 - Q5 / 5557 -- RW
        //4 - HDX Transponder (TI)-RW
        //5 - HITAG 2 - RW
        //6 - EM 4305 - RW
        //7 - EM4450/4550 (TITAN)-RW
        //8 - PSK - RO


        public bool CheckTransponderTyp()
        {
            bool result = false;
            for (int i = 0; i <= 8; i++)
            {
                SelectTag(i);
                result = fIsTagSelect;
                if (result) { return result; }
            }
            return result;
        }


        public string SelectTag(int TagType)
        {
            string Result = "";
            Result = "- 99";
            fIsTagSelect = false;
            fTagType = TagType;
            if (!this.Active)
            {
                return Result;
            }
            switch (fTagType)
            {
                case -1:
                    break;
                case 8:
                    //8 - PSK - RO
                    TLMCommand = 0;
                    fTagString = "";
                    if (SendTagString("rt"))
                    {
                        Result = fTagString;
                        return Result;
                    }
                    break;
                case 0:
                    //0 - EM 41xx (UNIQUE) -- RO
                    TLMCommand = 0;
                    fTagString = "";
                    if (SendTagString("ru"))
                    {
                        Result = fTagString;
                        return Result;
                    }
                    break;
                case 1:
                    //1 - HITAG S -- RW
                    fTagString = "";
                    HitagUID = "";
                    TLMCommand = 99;
                    scanline = "";
                    if (SendTagString("hv"))
                    {
                        fTagString = scanline;
                        if (!string.IsNullOrEmpty(fTagString))
                        {
                            HitagUID = fTagString.Trim();
                        }
                        else
                        {
                            Result = "- 1";
                            return Result;
                        }
                        TLMCommand = 1;
                        if (SendTagString("hS"))
                        {
                            Result = fTagString;
                            return Result;
                        }
                    }
                    break;
                case 2:
                    //2 - FDX-B ISO 11784/5--RW
                    TLMCommand = 2;
                    if (SendTagString("rz"))
                    {
                        Result = fTagString;
                        return Result;
                    }
                    break;
                case 3:
                    //3 - Q5 / 5557 = RW
                    TLMCommand = 3;
                    if (SendTagString("qr0"))
                    {
                        Result = fTagString;
                        return Result;
                    }
                    break;
                case 4:
                    //4 - HDX Transponder (TIRIS)--RW
                    TLMCommand = 4;
                    HDXCRC = "";
                    if (SendTagString("xmr"))
                    {
                        Result = fTagString;
                        return Result;
                    }
                    break;
                case 5:
                    //5 - HITAG 2 - RW
                    //TLMCommand = 5
                    Hitag2UID = "";
                    fTagString = "";
                    TLMCommand = 99;
                    scanline = "";
                    if (SendTagString("kap 0x" + Hitag2Password + " 0"))
                    {
                        fTagString = scanline;
                        if ((!string.IsNullOrEmpty(fTagString)) & (fTagString.IndexOf("+") > 0))
                        {
                            if (fTagString.Length >= 10)
                            {
                                Hitag2UID = fTagString.Substring(2, 8);
                                Hitag2UID1 = fTagString.Substring(10, 8);
                            }
                            TLMCommand = 5;
                            if (SendTagString("kr 0 "))
                            {
                                Result = fTagString;
                                return Result;
                            }
                        }
                        else
                        {
                            Result = "- 1";
                            if (fTagString.IndexOf("Error: 54") >= 0)
                            {
                                Result = fTagString;
                            }
                            return Result;
                        }
                    }
                    break;
                case 6:
                    //6 - EM 4305 - RW
                    TLMCommand = 6;
                    if (SendTagString("vr0"))
                    {
                        Result = fTagString;
                        return Result;
                    }
                    break;
                case 7:
                    //7 - EM4450/4550 (TITAN) - RW
                    fTagString = "";
                    TLMCommand = 99;
                    scanline = "";
                    if (SendTagString("tl 0x" + TitanPassword))
                    {
                        fTagString = scanline;
                        if (fTagString.IndexOf("+") >= 0)
                        {
                            TLMCommand = 7;
                            if (SendTagString("tr 32 "))
                            {
                                Result = fTagString;
                                return Result;
                            }
                        }
                        else
                        {
                            Result = fTagString;
                            return Result;
                        }
                    }
                    break;
            }
            return Result;
        }

        public string ChangeHitag2Password(string OldPassword, string NewPassword)
        {
            string Result = null;
            Result = "";
            if (OldPassword.Length != 8)
            {
                return Result;
            }
            if (NewPassword.Length != 8)
            {
                return Result;
            }
            Hitag2Password = NewPassword;
            Result = SelectTag(5);
            if (Result.IndexOf("-") >= 0)  
            {
                return Result;
            }
            else
            {
                Result = WriteTagData(5, "1", NewPassword);
            }
            return Result;
        }

        public string ChangeTitanPassword(string OldPassword, string NewPassword)
        {
            string Result = null;
            string tmp1 = null;
            string tmp2 = null;
            Result = "";
            if (OldPassword.Length != 8)
            {
                return Result;
            }
            if (NewPassword.Length != 8)
            {
                return Result;
            }
            tmp1 = "0x" + OldPassword;
            tmp2 = "0x" + NewPassword;
            Result = SendDataToReader("ts " + tmp1 + " " + tmp2);
            return Result;
        }

        #region "utility"


        private byte[] ASCIIToByteArray(string str)
        {
            byte[] Result;
            ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            Result = enc.GetBytes(str);
            return Result;
        }

        public string ByteArrayToASCII(byte[] arr)
        {
            string Result = string.Empty;
            System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            Result = enc.GetString(arr, 0, arr.Length);
            return Result;
        }


        public void ClearStringArray(ref string[] src)
        {
            for (int i = src.GetLowerBound(0); i <= src.GetUpperBound(0); i++)
            {
                src[i] = "";
            }
        }

        public void ClearCharArray(ref char[] src)
        {
            for (int i = src.GetLowerBound(0); i <= src.GetUpperBound(0); i++)
            {
                src[i] = '\0';
            }
        }

        public void ClearByteArray(ref byte[] src)
        {
            for (int i = src.GetLowerBound(0); i <= src.GetUpperBound(0); i++)
            {
                src[i] = 0;
            }
            // Array.Clear(src, 0, src.Length);  another solution
            // for (int i; i = 0; i <= src.length) { src[i] = 0; }
        }

        public string GetInverted(string Data)
        {
            string Result = string.Empty;
            for (int i = Data.Length - 1; i >= 0; i += -1)
            {
                Result = Result + Data[i];
            }
            return Result;
        }


        public string GetByteDataBufferToHex(byte[] Src, int BStart, int BStop)
        {
            string Result = string.Empty;
            for (int i = BStart; i <= BStop; i++)
            {
                Result += Src[i].ToString("X2");
            }
            return Result;
        }

        public string GetByteDataBufferToHexInverted(byte[] Src, int BStart, int BStop)
        {
            string Result = string.Empty;
            for (int i = BStop; i >= BStart; i += -1)
            {
                Result += Src[i].ToString("X2");
            }
            return Result;
        }

        public string GetByteBufferToHex(byte[] Src, int DataLen)
        {
            string Result = string.Empty;
            for (int i = 0; i <= DataLen - 1; i++)
            {
                Result += Src[i].ToString("X2");
            }
            return Result;
        }

        public string ASCIIToHex(string Input)
        {
            string Result = string.Empty;
            System.Text.StringBuilder sb = new System.Text.StringBuilder();
            foreach (char c in Input)
            {
                sb.Append(Convert.ToInt32(c).ToString("X2"));
            }
            Result = sb.ToString();
            return Result;
        }

        public string HexToASCII(string Input)
        {
            string Result = string.Empty;
            System.Text.StringBuilder sb = new System.Text.StringBuilder();
            for (int i = 0; i <= Input.Length - 2; i += 2)
            {
                sb.Append(Convert.ToChar(Convert.ToByte(Input.Substring(i, 2), 16))).ToString();
            }
            Result = sb.ToString();
            return Result;
        }

        public int HexToInt(string Input)
        {
            return Convert.ToInt32(Input, 16);
        }


        public byte[] HexToByteArray(string input)
        {
            byte[] result;
            int c = 0;
            result = new byte[input.Length / 2];
            for (int i = 0; i <= input.Length - 1; i += 2)
            {
                result[c] = Convert.ToByte(input.Substring(i, 2).ToString(), 16);
                c += 1;
            }
            return result;
        }

        public string ByteToHex(byte[] src)
        {
            string Result = string.Empty;
            for (int i = src.GetLowerBound(0); i <= src.GetUpperBound(0); i++)
            {
                Result += src[i].ToString("X2");
            }
            return Result;
        }

        public byte[] GetASCIIByte(string src)
        {
            byte[] Result;
            ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            Result = enc.GetBytes(src);
            return Result;
        }

        public byte[] GetASCIIByteFix(string src)
        {
            byte[] Result = new byte[256];
            for (int i = 0; i <= src.Length - 1; i++)
            {
                Result[i] = (byte)Convert.ToChar(src.Substring(i, 1));
            }
            return Result;
        }

        public char[] GetASCIIChar(string src)
        {
            char[] Result = new char[src.Length - 1];
            Result = src.ToCharArray();
            return Result;
        }

        #endregion

        public string GetVersion()
        {
            string Result = "";
            if (this.Active)
            {
                TLMCommand = 10;
                if (SendTagString("sv"))
                {
                    Result = fTagString;
                    if (Result.IndexOf(ControlChars.Lf) >= 0)
                    {
                        Result = Result.Replace(ControlChars.Lf, '\0');
                    }
                    if (Result.IndexOf(ControlChars.Cr) > 0)
                    {
                        Result = Result.Replace(ControlChars.Cr, '\0');
                        Result = Result.Trim();
                    }
                }
            }
            return Result;
        }

        public string SendDataToReader(string Command)
        {
            string Result = "";
            TLMCommand = 99;
            if (SendTagString(Command))
            {
                Result = fTagString;
            }
            return Result;
        }
        #endregion

        #region "Sub Private"
        private void SerialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            scanline += SerialPort1.ReadExisting();
            if (scanline.IndexOf(ControlChars.Cr) >= 0)
            {
                scanline = scanline.Replace(ControlChars.Cr, '\0');
                scanline = scanline.TrimEnd('\0');
                if (scanline.Length > 0)
                {
                    this.Invoke(new EventHandler(MyDataReceivedAll));
                }
            }
            if ((scanline.IndexOf("+") >= 0) & (scanline.Length == 1))
            {
                //this.Invoke(new EventHandler(MyDataReceivedAll));
            }
        }

        private void SerialPort1_ErrorReceived(object sender, System.IO.Ports.SerialErrorReceivedEventArgs e)
        {
            fErrorReceived = "No";
            switch (e.EventType)
            {
                case System.IO.Ports.SerialError.Frame:
                    fErrorReceived = "Frame";
                    break;
                case System.IO.Ports.SerialError.Overrun:
                    fErrorReceived = "Overun";
                    break;
                case System.IO.Ports.SerialError.RXOver:
                    fErrorReceived = "RXOver";
                    break;
                case System.IO.Ports.SerialError.TXFull:
                    fErrorReceived = "TXFull";
                    break;
            }
            this.Invoke(new EventHandler(ErrorChange));
        }

        private void SerialPort1_PinChanged(object sender, System.IO.Ports.SerialPinChangedEventArgs e)
        {
            //If e.EventType = IO.Ports.SerialPinChange.CtsChanged Then
            //        Me.Invoke(PegelWechsel)
            //       End If
            switch (e.EventType)
            {
                case System.IO.Ports.SerialPinChange.Break:
                    fPinInfo = "Break..";
                    break;
                // Bei der Eingabe wurde ein "break" erkannt.

                case System.IO.Ports.SerialPinChange.CDChanged:
                    fPinInfo = "CD Changed (Carrier Detect)";
                    break;
                // Der Zustand des CD-Signals (Carrier Detect) hat sich geändert.
                // Mit diesem Signal wird angezeigt, ob ein Modem mit einer 
                //  Telefonleitung verbunden ist und ein Datenträgersignal 
                //  erkannt wurde.

                case System.IO.Ports.SerialPinChange.CtsChanged:
                    fPinInfo = "CTS Changed (Clear to Send)";
                    break;
                // Der Zustand des CTS-Signals (Clear to Send) hat sich geändert.
                //  Mit diesem Signal wird angegeben, ob Daten über den seriellen
                //  Anschluss gesendet werden können. 

                case System.IO.Ports.SerialPinChange.DsrChanged:
                    fPinInfo = "DSR Changed (Data Set Ready)";
                    break;
                // Zustand des DSR-Signals (Data Set Ready) hat sich geändert.
                //  Mit diesem Signal wird angezeigt, ob das Gerät am seriellen
                //  Anschluss betriebsbereit ist.

                case System.IO.Ports.SerialPinChange.Ring:
                    fPinInfo = "Ring..";
                    break;
                //Ein Ringindikator wurde erkannt. 
            }
        }

        private void ErrorChange(object sender, System.EventArgs e) // Extern void Info
        {
            if (PortErrorReceived != null) { PortErrorReceived(this, fErrorReceived); }
        }

        private void PinChanged(object sender, System.EventArgs e)
        {
            if (PortPinChanged != null) { PortPinChanged(this, fPinInfo); }
        }

        private void DelayTimer_Tick(object sender, System.EventArgs e)
        {
            DelayTimer.Enabled = false;
        }


        private void Init()
        {
            //this.CreateHandle(); // .. Muss gestartet werden, sonst handle fehler
            AddEvents();
            SerialPort1.BaudRate = 9600;
            SerialPort1.DataBits = 8;
            SerialPort1.Parity = System.IO.Ports.Parity.None;
            SerialPort1.StopBits = System.IO.Ports.StopBits.One;
            SerialPort1.DtrEnable = false;
            SerialPort1.RtsEnable = false;
            SerialPort1.PortName = "COM1";
            DelayTimer.Interval = 2000;
            TLMCommand = 99;
        }

        private void DeInit()
        {
            DelayTimer.Enabled = false;
            this.Active = false;

            SerialPort1.Dispose();
            DelayTimer.Dispose();
            SerialPort1 = null;
            DelayTimer = null;
        }

        #endregion

    }
}

