﻿//*************************************************}
//                                                 }
//       Class: RFIDFeig/UHF                       }
//       Programmed with Visual Studio 2008        }
//       Tectus Transponder Technology GmbH        }
//       Sascha Haveloh, s.haveloh@tec-tus.de      }
//                                                 }
//*************************************************}
// Version 1.0 - 29.10.2012  
// Testing UHF Reader
// Import Class..RFIDFeigBartec.cs
// 05.11.2012 .. feisc.h constante aktualisiert.
// ISO Mode neu abfrage.
// 08.11.2012 Utility wird aktualisiert
// COM is only in the older version for fecomce_mc9090.dll not use in MC95
// Read/WriteTimer

using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Runtime;
using System.Runtime.InteropServices;
using System.Threading;
using System.IO;

public class RFIDFeigUHF
   
{

    public RFIDFeigUHF()  // public override void New() ... delphi..info..contructor create...
    {
        New_Init();
    }


    ~RFIDFeigUHF() // protected override void Finalize() ... delphi info..destroy...
    {
        ReaderUnload();
    }


    public sealed class ControlChars
    {
        public const char Back = '\b';
        public const char Cr = '\r';
        public const string CrLf = "\r\n";
        public const char FormFeed = '\f';
        public const char Lf = '\n';
        public const string NewLine = "\r\n";
        public const char NullChar = '\0';
        public const char Quote = '"';
        public const char Tab = '\t';
        public const char VerticalTab = '\v';
    }



	#region "FEIG constants WM_USER"

	public const int WM_USER = 0x400;
	public const int WM_USER_CTS = WM_USER + 5;
	public const int WM_USER_RTS = WM_USER + 6;
	public const int WM_USER_DEVICE_DISCONNECT = WM_USER + 7;

	public const int WM_USER_DEVICE_CONNECT = WM_USER + 8;
	#endregion

	#region "FEUSB constans"

	// FEUSB error codes

	// common errors
	public const int FEUSB_ERR_EMPTY_DEVICELIST = -1100;
	public const int FEUSB_ERR_EMPTY_SCANLIST = -1101;
	public const int FEUSB_ERR_POINTER_IS_NULL = -1102;
	public const int FEUSB_ERR_NO_MORE_MEM = -1103;
	public const int FEUSB_ERR_SET_CONFIGURATION = -1104;
	public const int FEUSB_ERR_KERNEL = -1105;
	public const int FEUSB_ERR_UNSUPPORTED_OPTION = -1106;

	public const int FEUSB_ERR_UNSUPPORTED_FUNCTION = -1107;
	// error while scanning
	public const int FEUSB_ERR_NO_FEIG_DEVICE = -1110;
	public const int FEUSB_ERR_SEARCH_MISMATCH = -1111;
	public const int FEUSB_ERR_NO_DEVICE_FOUND = -1112;
	public const int FEUSB_ERR_DEVICE_IS_SCANNED = -1113;

	public const int FEUSB_ERR_SCANLIST_OVERFLOW = -1114;
	// handle errors
	public const int FEUSB_ERR_UNKNOWN_HND = -1120;
	public const int FEUSB_ERR_HND_IS_NULL = -1121;
	public const int FEUSB_ERR_HND_IS_NEGATIVE = -1122;

	public const int FEUSB_ERR_NO_HND_FOUND = -1123;
	// communication errors
	public const int FEUSB_ERR_TIMEOUT = -1130;
	public const int FEUSB_ERR_NO_SENDDATA = -1131;
	public const int FEUSB_ERR_UNKNOWN_INTERFACE = -1132;
	public const int FEUSB_ERR_UNKNOWN_DIRECTION = -1133;
	public const int FEUSB_ERR_RECBUF_TOO_SMALL = -1134;
	public const int FEUSB_ERR_SENDDATA_LEN = -1135;
	public const int FEUSB_ERR_UNKNOWN_DESCRIPTOR_TYPE = -1136;
	public const int FEUSB_ERR_DEVICE_NOT_PRESENT = -1137;

	public const int FEUSB_ERR_TRANSMIT_PROCESS = -1138;
	// open/close device errors
	public const int FEUSB_ERR_DEVICE_NOT_SCANNED = -1140;
	public const int FEUSB_ERR_DEVHND_NOT_IN_SCANLIST = -1141;

	public const int FEUSB_ERR_DRIVERLIST = -1142;
	// parameter errors
	public const int FEUSB_ERR_UNKNOWN_PARAMETER = -1150;
	public const int FEUSB_ERR_PARAMETER_OUT_OF_RANGE = -1151;
	public const int FEUSB_ERR_ODD_PARAMETERSTRING = -1152;
	public const int FEUSB_ERR_INDEX_OUT_OF_RANGE = -1153;
	public const int FEUSB_ERR_UNKNOWN_SCANOPTION = -1154;

	public const int FEUSB_ERR_UNKNOWN_ERRORCODE = -1155;
	// identification errors
	public const int FEUSB_ERR_DEV_DESC_LENGTH = -1160;
	public const int FEUSB_ERR_CFG_DESC_LENGTH = -1161;
	public const int FEUSB_ERR_INTF_DESC_LENGTH = -1162;
	public const int FEUSB_ERR_ENDP_DESC_LENGTH = -1163;
	public const int FEUSB_ERR_HID_DESC_LENGTH = -1164;
	public const int FEUSB_ERR_STRG_DESC_LENGTH = -1165;
	public const int FEUSB_ERR_READ_DEV_DESCRIPTOR = -1166;
	public const int FEUSB_ERR_READ_CFG_DESCRIPTOR = -1167;

	public const int FEUSB_ERR_READ_STRG_DESCRIPTOR = -1168;
	public const int FEUSB_ERR_MAX_INTERFACES = -1170;
	public const int FEUSB_ERR_MAX_ENDPOINTS = -1171;

	public const int FEUSB_ERR_MAX_STRINGS = -1172;

	// constants for dynamic load of library

	// miscellaneous functions
	public const int FEUSB_GET_DLL_VERSION = 1100;
	public const int FEUSB_GET_DRV_VERSION = 1101;
	public const int FEUSB_GET_ERROR_TEXT = 1102;

	public const int FEUSB_GET_LAST_ERROR = 1103;
	// function for scan-list					
	public const int FEUSB_SCAN_ = 1110;
	public const int FEUSB_SCAN_AND_OPEN_ = 1111;
	public const int FEUSB_GET_SCAN_LIST_PARA = 1112;
	public const int FEUSB_GET_SCAN_LIST_SIZE = 1113;

	public const int FEUSB_CLEAR_SCAN_LIST = 1114;
	// functions for event notification			
	public const int FEUSB_ADD_EVENT_HANDLER = 1120;

	public const int FEUSB_DEL_EVENT_HANDLER = 1121;
	// device functions							
	public const int FEUSB_OPEN_DEVICE = 1130;
	public const int FEUSB_CLOSE_DEVICE = 1131;
	public const int FEUSB_IS_DEVICE_PRESENT = 1132;
	public const int FEUSB_GET_DEVICE_LIST = 1133;
	public const int FEUSB_GET_DEVICE_PARA = 1134;
	public const int FEUSB_SET_DEVICE_PARA = 1135;

	public const int FEUSB_GET_DEVICE_HND = 1136;
	// communication function					
	public const int FEUSB_TRANSCEIVE_ = 1140;
	public const int FEUSB_TRANSMIT_ = 1141;

	public const int FEUSB_RECEIVE_ = 1142;

	// constants for uiFlag in FEUSB_MSG_INIT
		//not useable in Visual Basic
	public const int FEUSB_THREAD_ID = 1;
		//not useable in Visual Basic
	public const int FEUSB_WND_HWND = 2;

	public const int FEUSB_CALLBACK = 3;
	// defines for uiUse in FEUSB_EVENT_INIT
	public const int FEUSB_DEV_CONNECT_EVENT = 1;

	public const int FEUSB_DEV_DISCONNECT_EVENT = 2;
	// interface constants
	// scan options
	public const int FEUSB_SCAN_FIRST = 0x1;
	public const int FEUSB_SCAN_NEXT = 0x2;
	public const int FEUSB_SCAN_NEW = 0x3;
	public const int FEUSB_SCAN_ALL = 0xf;
	public const int FEUSB_SCAN_SEARCH = 0x10000;

	public const int FEUSB_SCAN_PACK = 0x20000;
	// search options
	public const int FEUSB_SEARCH_FAMILY = 0x1;
	public const int FEUSB_SEARCH_PRODUCT = 0x2;

	public const int FEUSB_SEARCH_DEVICEID = 0x4;
	// communication directions
	public const int FEUSB_DIR_IN = 1;

	public const int FEUSB_DIR_OUT = 2;
	// Interface names
	public const string FEUSB_INTERFACE_OBID_RCI = "OBID-RCI";

	public const string FEUSB_INTERFACE_OBID_RCI_V2 = "OBID-RCI2";


	// structure for transfering callback functions
	public struct FEUSB_EVENT_INIT
	{
			// defines the event (e.g. FEUSB_DEV_CONNECT_EVENT)
		public int uiUse;
			// set always to 0 - in Visual Basic not used !!
		public int uiMsg;
			// specifies the use of the Variant (e.g. FEUSB_CALLBACK)
		public int uiFlag;
			// for callback-function
		public int Method;
	}

	//type  FEUSB_SCANSEARCH = record
	// iMask:                          Integer;
	//cFamily,cProduct,cInterface:    array[0..24] of Char;
	//cDeviceID:                      array[0..8] of Char;
	//end;

	// new datatype
	//<StructLayout(LayoutKind.Sequential)> _

	//typedef struct _FEUSB_SCANSEARCH
	//{
	//	int		iMask;
	//	char	cFamilyName[25];
	//	char	cDeviceName[25];
	//	char	cInterface[25];
	//	char	cDeviceID[16];

	//} FEUSB_SCANSEARCH;

	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
	public struct FEUSB_SCANSEARCH
	{
		public short iMask;
        [MarshalAs(UnmanagedType.ByValTStr,SizeConst = 25)]
        //[MarshalAs(UnmanagedType.ByValTStr, sizeconst = 25)]
			// 25
		public string cFamily;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 25)]
			// 25
		public string cProduct;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 25)]
			// 25
		public string cInterface;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 9)]
			// 9
		public string cDeviceID;
	}


	[System.Runtime.InteropServices.StructLayout(System.Runtime.InteropServices.LayoutKind.Sequential)]
	public struct FEUSB_SCANSEARCH1
	{
		public int iMask;
		public string cFamily;
		public string cProduct;
		public string cInterface;
		public string cDeviceID;
	}

	[System.Runtime.InteropServices.StructLayout(System.Runtime.InteropServices.LayoutKind.Sequential)]
	public struct FEUSB_SCANSEARCH_BYTE
	{
		public short iMask;
		public byte[] cFamily;
		public byte[] cProduct;
		public byte[] cInterface;
		public byte[] cDeviceID;
	}

	#endregion

	#region "FEUSB DLL"

	// #####################################################
	// FEUSB DLL 
	// #####################################################

	// common functions
	[DllImport(dllName_usb, EntryPoint = "FEUSB_GetDLLVersion", SetLastError = true)]
	private static extern void FEUSB_GetDLLVersion(string cVersion);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_GetErrorText", SetLastError = true)]
	private static extern int FEUSB_GetErrorText(int iError, byte[] cErrorText);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_GetLastError", SetLastError = true)]
	private static extern int FEUSB_GetLastError(int iDevHnd, int iErrorCode, string cErrorText);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_Scan", SetLastError = true)]
	private static extern int FEUSB_Scan(int iScanOpt, ref FEUSB_SCANSEARCH_BYTE SearchOpt);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_ScanAndOpen", SetLastError = true)]
	private static extern int FEUSB_ScanAndOpen(int iScanOpt, ref FEUSB_SCANSEARCH pSearchOpt);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_GetScanListPara", SetLastError = true)]
	private static extern int FEUSB_GetScanListPara(int iIndex, string cPara, byte[] cValue);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_GetScanListSize", SetLastError = true)]
	private static extern int FEUSB_GetScanListSize();

    [DllImport(dllName_usb, EntryPoint = "FEUSB_ClearScanList", SetLastError = true)]
	private static extern void FEUSB_ClearScanList();


	// functions for event notification
	[DllImport(dllName_usb, EntryPoint = "FEUSB_AddEventHandler", SetLastError = true)]
	private static extern int FEUSB_AddEventHandler(int iPortHnd, FEUSB_EVENT_INIT pInit);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_DelEventHandler", SetLastError = true)]
	private static extern int FEUSB_DelEventHandler(int iPortHnd, FEUSB_EVENT_INIT pInit);
	// device functions
	[DllImport(dllName_usb, EntryPoint = "FEUSB_OpenDevice", SetLastError = true)]
	private static extern int FEUSB_OpenDevice(long dwDeviceID);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_CloseDevice", SetLastError = true)]
	private static extern int FEUSB_CloseDevice(int iDevHnd);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_IsDevicePresent", SetLastError = true)]
	private static extern int FEUSB_IsDevicePresent(int iDevHnd);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_GetDeviceList", SetLastError = true)]
	private static extern int FEUSB_GetDeviceList(int iNext);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_GetDeviceHnd", SetLastError = true)]
	private static extern int FEUSB_GetDeviceHnd(int dwDeviceID);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_GetDevicePara", SetLastError = true)]
    private static extern int FEUSB_GetDevicePara(int iDeviceHnd, string cParaID, string cValue);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_SetDevicePara", SetLastError = true)]
	private static extern int FEUSB_SetDevicePara(int iDeviceHnd, string cParaID, string cValue);
	// communication function
	[DllImport(dllName_usb, EntryPoint = "FEUSB_Transceive", SetLastError = true)]
	private static extern int FEUSB_Transceive(int iDevHnd, string cInterface, int iDir, string cSendData, int iSendLen, string cRecData, int iRecLen);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_Transmit", SetLastError = true)]
	private static extern int FEUSB_Transmit(int iDevHnd, string cInterface, string cSendData, int iSendLen);
	[DllImport(dllName_usb, EntryPoint = "FEUSB_Receive", SetLastError = true)]
	private static extern int FEUSB_Receive(int iDevHnd, string cInterface, string cRecData, int iRecLen);

	#endregion


	#region "FECOM constants"
	// #####################################################
	// FECOM constants
	// #####################################################

    // FECOM error codes

	public const int FECOM_ERR_NEWPORT_FAILURE = -1000;
	public const int FECOM_ERR_EMPTY_LIST = -1001;
	public const int FECOM_ERR_POINTER_IS_NULL = -1002;
	public const int FECOM_ERR_NO_MEMORY = -1003;
	public const int FECOM_ERR_UNSUPPORTED_HARDWARE = -1004;
	public const int FECOM_ERR_PORT_NOT_FOUND = -1005;

	public const int FECOM_ERR_UNSUPPORTED = -1006;
	// error while open the port
	public const int FECOM_ERR_NO_PORT = -1010;
	public const int FECOM_ERR_NO_CONNECT = -1011;
	public const int FECOM_ERR_LINK_ID = -1012;

	public const int FECOM_ERR_PORT_IS_OPEN = -1013;
	// handle errors
	public const int FECOM_ERR_UNKNOWN_HND = -1020;
	public const int FECOM_ERR_HND_IS_NULL = -1021;
	public const int FECOM_ERR_HND_IS_NEGATIVE = -1022;

	public const int FECOM_ERR_NO_HND_FOUND = -1023;
	// communication errors
	public const int FECOM_ERR_TIMEOUT = -1030;
	public const int FECOM_ERR_NO_SENDPROTOCOL = -1031;
	public const int FECOM_ERR_RECEIVE_PROCESS = -1032;
	public const int FECOM_ERR_INIT_COMM_PROCESS = -1033;
	public const int FECOM_ERR_FLUSH_INPUT_BUFFER = -1034;
	public const int FECOM_ERR_FLUSH_OUTPUT_BUFFER = -1035;
	public const int FECOM_ERR_CHANGE_PORT_PARA = -1036;
	public const int FECOM_ERR_TRANSMIT_PROCESS = -1037;
	public const int FECOM_ERR_RECEIVE_NOISE_DATA = -1038;

	public const int FECOM_ERR_READ_PORT_PARA = -1039;
	// parameter errors
	public const int FECOM_ERR_UNKNOWN_PARAMETER = -1050;
	public const int FECOM_ERR_PARAMETER_OUT_OF_RANGE = -1051;
	public const int FECOM_ERR_ODD_PARAMETERSTRING = -1052;
	public const int FECOM_ERR_PORTNR_OUT_OF_RANGE = -1053;

	public const int FECOM_ERR_UNKNOWN_ERRORCODE = -1054;
	// receive buffer overflow

	public const int FECOM_ERR_OVL_RECBUF = -1070;
	// constants for dynamic load of library

	// miscellaneous functions
	public const int FECOM_GET_DLL_VERSION = 1000;
	public const int FECOM_GET_ERROR_TEXT = 1001;
	public const int FECOM_GET_LAST_ERROR = 1002;

	public const int FECOM_DETECT_PORT = 1003;
	// functions for event notification
	public const int FECOM_ADD_EVENT_HANDLER = 1010;

	public const int FECOM_DEL_EVENT_HANDLER = 1011;
	// port functions
	public const int FECOM_OPEN_PORT = 1020;
	public const int FECOM_CLOSE_PORT = 1021;
	public const int FECOM_GET_PORT_LIST = 1022;
	public const int FECOM_GET_PORT_PARA = 1023;
	public const int FECOM_SET_PORT_PARA = 1024;
	public const int FECOM_DO_PORT_CMD = 1025;

	public const int FECOM_GET_PORT_HND = 1026;
	// communication function
	public const int FECOM__TRANSCEIVE = 1030;
	public const int FECOM__TRANSMIT = 1031;

	public const int FECOM__RECEIVE = 1032;


	// constants for structure _FECOM_EVENT_INIT

	// defines for uiFlag in FECOM_EVENT_INIT
	public const int FECOM_THREAD_ID = 1;
	public const int FECOM_WND_HWND = 2;
	public const int FECOM_CALLBACK = 3;

	public const int FECOM_EVENT = 4;
	// defines for uiUse in FECOM_EVENT_INIT
	public const int FECOM_CTS_EVENT = 1;
	public const int FECOM_DCD_EVENT = 2;
	public const int FECOM_DSR_EVENT = 3;
	public const int FECOM_RTS_EVENT = 4;

	public const int FECOM_DTR_EVENT = 5;
		//' only for Windows CE (customized versions)
	public const int FECOM_PORT_CLOSED_EVENT = 20;
		//	' only for Windows CE (customized versions) 
	public const int FECOM_PORT_REOPEN_EVENT = 21;

	// #####################################################
	// FECOM structures
	// #####################################################
	// structure for transfering thread-IDs, message-handles or callbacks
	// structure for transfering callback functions
	public struct FECOM_EVENT_INIT
	{
		public int uiUse;
		public int uiMsg;
		public int uiFlag;
		public static DelgOnMessage Method;
		//Public Method As DelgComInfo
		//Public hwndWnd As Integer
	}

	public struct EventTest
	{
		public event SayHalloEventHandler SayHallo;
		public delegate void SayHalloEventHandler();
		public void DoEvent()
		{
			if (SayHallo != null) {
				SayHallo();
			}
		}
	}

	#endregion


	#region "FEISC constants"
	// #####################################################
	// FEISC constants
	// #####################################################

    // command bytes
    const int FEISC_DESTROY = 0x18;
    const int FEISC_HALT = 0x1A;
    const int FEISC_RESET_QUIET_BIT	= 0x1B;
    const int FEISC_EAS_REQUEST = 0x1C;
    const int FEISC_MAX_DATA_EXCHANGE = 0x1F;
    const int FEISC_READ_BUFFER = 0x21;
    const int FEISC_ADV_READ_BUFFER = 0x22;
    const int FEISC_READ_BUFFER_INFO = 0x31;
    const int FEISC_CLEAR_BUFFER = 0x32;
    const int FEISC_INIT_BUFFER	= 0x33;
    const int FEISC_FORCE_NOTIFY_TRIGGER = 0x34;
    const int FEISC_SOFTWARE_TRIGGER = 0x35;
    const int FEISC_GET_BAUDRATE = 0x52;
    const int FEISC_START_FLASH_LOADER = 0x55;
    const int FEISC_START_FLASH_LOADER_EX = 0x55;
    const int FEISC_CPU_RESET = 0x63;
    const int FEISC_SYSTEM_RESET = 0x64;
    const int FEISC_SOFT_VERSION = 0x65;
    const int FEISC_READER_INFO = 0x66;
    const int FEISC_CHANNEL_ANALYZE = 0x68;
    const int FEISC_RF_RESET = 0x69;
    const int FEISC_RF_ON_OFF = 0x6A;
    const int FEISC_CENTRALIZED_RF_SYNC = 0x6B;
    const int FEISC_SET_NOISE_LEVEL = 0x6C;
    const int FEISC_GET_NOISE_LEVEL = 0x6D;
    const int FEISC_READER_DIAGNOSE = 0x6E;
    const int FEISC_ANTENNA_TUNING = 0x6F;
    const int FEISC_SET_OUTPUT = 0x71;
    const int FEISC_SET_OUTPUT_EX = 0x72;
    const int FEISC_GET_INPUT = 0x74;
    const int FEISC_ADJ_ANTENNA = 0x75;
    const int FEISC_CHECK_ANTENNAS = 0x76;
    const int FEISC_GET_COUNTER = 0x77;
    const int FEISC_SET_COUNTER = 0x78;
    const int FEISC_WRITE_DISPLAY = 0x7A;
    const int FEISC_READ_CONF_BLOCK = 0x80;
    const int FEISC_WRITE_CONF_BLOCK = 0x81;
    const int FEISC_SAVE_CONF_BLOCK = 0x82;
    const int FEISC_RESET_CONF_BLOCK = 0x83;
    const int FEISC_SET_CONF_MEM_LOC = 0x84;
    const int FEISC_SET_SYSTEM_TIMER = 0x85;
    const int FEISC_GET_SYSTEM_TIMER = 0x86;
    const int FEISC_SET_SYSTEM_DATE = 0x87;
    const int FEISC_GET_SYSTEM_DATE = 0x88;
    const int FEISC_READ_CONFIGURATION = 0x8A;
    const int FEISC_WRITE_CONFIGURATION = 0x8B;
    const int FEISC_RESET_CONFIGURATION = 0x8C;
    const int FEISC_PIGGYBACK_COMMAND = 0x9F;
    const int FEISC_READER_LOGIN = 0xA0;
    const int FEISC_WRITE_MIFARE_KEYS = 0xA2;
    const int FEISC_WRITE_DES_AES_KEYS = 0xA3;
    const int FEISC_WRITE_READER_AUTHENT_KEY = 0xAD;
    const int FEISC_READER_AUTHENT = 0xAE;
    const int FEISC_CRYPTO_COMMAND = 0xAF;
    const int FEISC_ISO_COMMAND = 0xB0;
    const int FEISC_ISO_CUST_PROP_CMD = 0xB1;
    const int FEISC_ISO_SP14443_COMMAND = 0xB2;
    const int FEISC_EPC_COMMAND = 0xB3;
    const int FEISC_EPC_UHF_COMMAND = 0xB4;
    const int FEISC_C1G2_TRANSPARENT_CMD = 0xBB;
    const int FEISC_COMMAND_QUEUE = 0xBC;
    const int FEISC_ISO14443A_TRANSPARENT_CMD = 0xBD;
    const int FEISC_ISO14443B_TRANSPARENT_CMD = 0xBE;
    const int FEISC_ISO15693_TRANSPARENT_CMD = 0xBF;
    const int FEISC_SAM_COMMAND = 0xC0;
    const int FEISC_DESFIRE_COMMAND = 0xC1;
    const int FEISC_MIFARE_PLUS_COMMAND = 0xC2;
    const int FEISC_DESFIRE_COMMAND_EX = 0xC3;


    // FEISC Error codes

    // common errors
    const int FEISC_ERR_NEWREADER_FAILURE = -4000;
    const int FEISC_ERR_EMPTY_LIST = -4001;
    const int FEISC_ERR_POINTER_IS_NULL = -4002;
    const int FEISC_ERR_NO_MORE_MEM = -4003;
    const int FEISC_ERR_UNKNOWN_COMM_PORT = -4004;
    const int FEISC_ERR_UNSUPPORTED_FUNCTION = -4005;
    const int FEISC_ERR_NO_USB_SUPPORT = -4006;
    const int FEISC_ERR_OLD_FECOM = -4007;

    // query errors
    const int FEISC_ERR_NO_VALUE = -4010;

    // handle errors
    const int FEISC_ERR_UNKNOWN_HND = -4020;
    const int FEISC_ERR_HND_IS_NULL = -4021;
    const int FEISC_ERR_HND_IS_NEGATIVE = -4022;
    const int FEISC_ERR_NO_HND_FOUND = -4023;
    const int FEISC_ERR_PORTHND_IS_NEGATIVE = -4024;
    const int FEISC_ERR_HND_UNVALID = -4025;

    // communication errors
    const int FEISC_ERR_PROTLEN = -4030;
    const int FEISC_ERR_CHECKSUM = -4031;
    const int FEISC_ERR_BUSY_TIMEOUT = -4032;
    const int FEISC_ERR_UNKNOWN_STATUS = -4033;
    const int FEISC_ERR_NO_RECPROTOCOL = -4034;
    const int FEISC_ERR_CMD_BYTE = -4035;
    const int FEISC_ERR_TRANSCEIVE = -4036;
    const int FEISC_ERR_REC_BUS_ADR = -4037;
    const int FEISC_ERR_READER_POWER_DOWN = -4038;

    // parameter errors
    const int FEISC_ERR_UNKNOWN_PARAMETER = -4050;
    const int FEISC_ERR_PARAMETER_OUT_OF_RANGE = -4051;
    const int FEISC_ERR_ODD_PARAMETERSTRING = -4052;
    const int FEISC_ERR_UNKNOWN_ERRORCODE = -4053;
    const int FEISC_ERR_UNSUPPORTED_OPTION = -4054;
    const int FEISC_ERR_UNKNOWN_EPC_TYPE = -4055;
    const int FEISC_ERR_PARAMETER_LOCKED = -4056;

    // plug-in errors
    const int FEISC_ERR_NO_PLUGIN = -4060;
    const int FEISC_ERR_PLUGIN_PRESENT = -4061;
    const int FEISC_ERR_UNKNOWN_PLUGIN_ID = -4062;
    const int FEISC_ERR_PI_BUILD_DATA = -4063;
    const int FEISC_ERR_PI_BUILD_FRAME = -4064;
    const int FEISC_ERR_PI_SPLIT_FRAME = -4065;
    const int FEISC_ERR_PI_SPLIT_DATA = -4066;
    const int FEISC_ERR_PI_GET = -4067;
    const int FEISC_ERR_PI_LOAD = -4068;
    const int FEISC_ERR_PI_RELEASE = -4069;

    // communication data flow errors
    const int FEISC_ERR_BUFFER_OVERFLOW = -4070;

    // task errors
    const int FEISC_ERR_TASK_STILL_RUNNING = -4080;
    const int FEISC_ERR_TASK_NOT_STARTED = -4081;
    const int FEISC_ERR_TASK_TIMEOUT = -4082;
    const int FEISC_ERR_TASK_SOCKET_INIT = -4083;
    const int FEISC_ERR_TASK_BUSY = -4084;
    const int FEISC_ERR_THREAD_CANCEL_ERROR = -4085;

    // errors from cryptography module
    const int FEISC_ERR_CRYPT_LOAD_LIBRARY = -4090;
    const int FEISC_ERR_CRYPT_INIT = -4091;
    const int FEISC_ERR_CRYPT_AUTHENT_PROCESS = -4092;
    const int FEISC_ERR_CRYPT_ENCYPHER = -4093;
    const int FEISC_ERR_CRYPT_DECYPHER = -4094;



    // defines for uiFlag in FEISC_EVENT_INIT
    const int FEISC_THREAD_ID = 1;
    const int FEISC_WND_HWND = 2;
    const int FEISC_CALLBACK = 3;
    const int FEISC_EVENT = 4;
    const int FEISC_CALLBACK_2 = 5;
    const int FEISC_CALLBACK_4 = 6;

    // defines for uiUse in FEISC_EVENT_INIT
    const int FEISC_PRT_EVENT = 1;
    const int FEISC_SNDPRT_EVENT = 2;
    const int FEISC_RECPRT_EVENT = 3;
    const int FEISC_SCANNER_EVENT = 4;


    // defines for uiFlag in FEISC_TASK_INIT
    const int FEISC_TASKCB_1 = 1;
    const int FEISC_TASKCB_2 = 2;
    const int FEISC_TASKCB_3 = 3;
    const int FEISC_TASKCB_NET_1 = 10;

    // defines for task identifier
    const int FEISC_TASKID_FIRST_NEW_TAG = 1;
    const int FEISC_TASKID_EVERY_NEW_TAG = 2;
    const int FEISC_TASKID_NOTIFICATION = 3;
    const int FEISC_TASKID_SAM_COMMAND = 4;
    const int FEISC_TASKID_COMMAND_QUEUE = 5;
    const int FEISC_TASKID_MAX_EVENT = 6;
    const int FEISC_TASKID_PEOPLE_COUNTER_EVENT = 7;

    // defines for channel types in FEISC_TASK_INIT
    const int FEISC_TASK_CHANNEL_TYPE_AS_OPEN = 1;	// for open serial, usb, tcp channels
    const int FEISC_TASK_CHANNEL_TYPE_NEW_TCP = 5;	// for new tcp connection, established by host or reader

    // #####################################################
    // FEISC block constant
    // #####################################################

    // block 79: [0xB1] send data							 FF-MM-BN-BY-NB-BI-Nb
    // #define FEDM_ISC_TMP_0xB1_REQ_IDS_PASSWORD_LEVEL		"03 10 79 00 01 00 00" // for IDS [0xA0], [0xAB]
    // #define FEDM_ISC_TMP_0xB1_REQ_IDS_PASSWORD				"03 10 79 01 04 00 00" // for IDS [0xA0]
    // #define FEDM_ISC_TMP_0xB1_REQ_IDS_LOG_MODE				"03 10 79 05 04 00 00" // for IDS [0xA1]
    // #define FEDM_ISC_TMP_0xB1_REQ_IDS_LOG_LIMITS			"03 10 79 09 04 00 00" // for IDS [0xA2]
    const int FEDM_ISC_TMP_0xB1_REQ_IDS_EXT_CAL_DATA = 0xA4; //"03 10 79 13 04 00 00" // for IDS [0xA4]
    // #define FEDM_ISC_TMP_0xB1_REQ_IDS_INT_CAL_DATA			"03 10 79 17 04 00 00" // for IDS [0xA5]
    // #define FEDM_ISC_TMP_0xB1_REQ_IDS_START_TIME			"03 10 79 21 04 00 00" // for IDS [0xA7]
    // #define FEDM_ISC_TMP_0xB1_REQ_IDS_INIT_DATA				"03 10 79 25 04 00 00" // for IDS [0xAC]
    										
    // block 80: [0xB1] receive data						 FF-MM-BN-BY-NB-BI-Nb
    // #define FEDM_ISC_TMP_0xB1_RSP_IDS_START_TIME			"03 10 80 00 04 00 00" // for IDS [0xA3]
    // #define FEDM_ISC_TMP_0xB1_RSP_IDS_LOG_LIMITS			"03 10 80 04 04 00 00" // for IDS [0xA3]
    // #define FEDM_ISC_TMP_0xB1_RSP_IDS_LOG_MODE				"03 10 80 08 04 00 00" // for IDS [0xA3]
    // #define FEDM_ISC_TMP_0xB1_RSP_IDS_DELAY_TIME			"03 10 80 12 04 00 00" // for IDS [0xA3]
    // #define FEDM_ISC_TMP_0xB1_RSP_IDS_MEAS_STATE			"03 10 80 16 04 00 00" // for IDS [0xA8]
    // #define FEDM_ISC_TMP_0xB1_RSP_IDS_LIMITS_COUNTER		"03 10 80 20 04 00 00" // for IDS [0xA8]
    // #define FEDM_ISC_TMP_0xB1_RSP_IDS_INT_CAL_DATA			"03 10 80 24 04 00 00" // for IDS [0xA9]
    // #define FEDM_ISC_TMP_0xB1_RSP_IDS_EXT_CAL_DATA			"03 10 80 28 04 00 00" // for IDS [0xA9]
										
    // block 81: [0xB1] receive data						 FF-MM-BN-BY-NB-BI-Nb
    // #define FEDM_ISC_TMP_0xB1_RSP_IDS_BATTERY_LEVEL			"03 10 81 00 01 00 00" // for IDS [0xAA]
    // #define FEDM_ISC_TMP_0xB1_RSP_IDS_MEAS_TEMP				"03 10 81 01 02 00 00" // for IDS [0xAD]



    // #####################################################
    // FEISC structures
    // #####################################################

	public struct FEISC_EVENT_INIT
	{
		public int uiUse;
		public int uiMsg;
		public int uiFlag;
		public DelgOnMessage Method;
	}

	// structure for transfering anything for an asynchronous task
	public struct FEISC_TASK_INIT
	{
		public byte[] pAny;
		public byte ucBusAdr;
		public int uiChannelType;
		public int iConnectByHost;
		public char[] cIPAdr;
		public int iPortNr;
		public int uiTimeout;
		public int uiFlag;
		public class dummy1
		{
			object cbFct1;
			object cbFct2;
		}
		public class dummy2
		{
			int iNotifyWithAck;
		}
	}

	#endregion

	#region "ISC Reader constans"
	// size of memories in bytes

		// memory for parameters in readers EEPROM
	public const int FEDM_ISC_MAX_EEDATA_MEM = 1024;
		// memory for parameters in readers RAM
	public const int FEDM_ISC_MAX_RAMDATA_MEM = 1024;
		// memory for temporary data
	public const int FEDM_ISC_MAX_TMPDATA_MEM = 4096;
	// size of memories in bytes
		// memory for parameters in readers EEPROM
	public const int FEDM_ISC_MAX_RFC_EEDATA_MEM = 1024;
		// memory for parameters in readers RAM
	public const int FEDM_ISC_MAX_RFC_RAMDATA_MEM = 1024;
		// memory for parameters in readers EEPROM
	public const int FEDM_ISC_MAX_ACC_EEDATA_MEM = 8192;
		// memory for parameters in readers RAM
	public const int FEDM_ISC_MAX_ACC_RAMDATA_MEM = 8192;

	// table types 
	// (uiTableID in functions GetTableData, SetTableData, FindTableIndex,
	//	GetTableSize, ResetTable)
	public const int FEDM_ISC_BRM_TABLE = 1;
	public const int FEDM_ISC_ISO_TABLE = 2;

	public const int FEDM_ISC_CAD_TABLE = 3;

	// access flags for the tables BRM and ISO
	// (uiDataID in functions GetTableData, SetTableData, FindTableIndex, ResetTable)
		// BRM and ISO
	public const int FEDM_ISC_DATA_TRTYPE = 0x8000001;
		// BRM and ISO
	public const int FEDM_ISC_DATA_SNR = 0x8000002;
		// BRM and ISO
	public const int FEDM_ISC_DATA_SNR_LEN = 0x8000003;
		// only ISO
	public const int FEDM_ISC_DATA_EPC = 0x8000004;
		// only ISO
	public const int FEDM_ISC_DATA_EPC_HEADER = 0x8000005;
		// only ISO
	public const int FEDM_ISC_DATA_EPC_DOMAIN = 0x8000006;
		// only ISO
	public const int FEDM_ISC_DATA_EPC_OBJECT = 0x8000007;
		// only ISO
	public const int FEDM_ISC_DATA_EPC_SNR = 0x8000008;
		// only ISO
	public const int FEDM_ISC_DATA_EPC_TYPE = 0x800000a;
		// only ISO
	public const int FEDM_ISC_DATA_AFI = 0x800000b;
		// only ISO
	public const int FEDM_ISC_DATA_DSFID = 0x800000c;
		// only ISO
	public const int FEDM_ISC_DATA_MEM_SIZE = 0x800000d;
		// only ISO
	public const int FEDM_ISC_DATA_IC_REF = 0x800000e;
		// only ISO14443A
	public const int FEDM_ISC_DATA_TRINFO = 0x800000f;
		// only ISO14443A
	public const int FEDM_ISC_DATA_OPTINFO = 0x8000010;
		// only ISO14443-4
	public const int FEDM_ISC_DATA_FSCI = 0x8000011;
		// only ISO14443-4
	public const int FEDM_ISC_DATA_FWI = 0x8000012;
		// only ISO14443-4
	public const int FEDM_ISC_DATA_DSI = 0x8000013;
		// only ISO14443-4
	public const int FEDM_ISC_DATA_DRI = 0x8000014;
		// only ISO14443-4
	public const int FEDM_ISC_DATA_NAD = 0x8000015;
		// only ISO14443-4
	public const int FEDM_ISC_DATA_CID = 0x8000016;
		// only ISO14443B
	public const int FEDM_ISC_DATA_PROTOINFO = 0x8000017;
		// BRM and ISO
	public const int FEDM_ISC_DATA_EPC_CL1_GEN2_PC = 0x8000018;
		// BRM and ISO
	public const int FEDM_ISC_DATA_RxDB = 0x4000001;
		// only ISO
	public const int FEDM_ISC_DATA_TxDB = 0x4000002;
		// only ISO
	public const int FEDM_ISC_DATA_SEC_STATUS = 0x4000003;
		// BRM and ISO
	public const int FEDM_ISC_DATA_BLOCK_SIZE = 0x4000004;
		// only ISO
	public const int FEDM_ISC_DATA_RxCB = 0x2000001;
		// only ISO
	public const int FEDM_ISC_DATA_TxCB = 0x2000002;
		// only BRM
	public const int FEDM_ISC_DATA_TIMER = 0x1000001;
		// only BRM
	public const int FEDM_ISC_DATA_DB_ADR = 0x1000002;
		// only BRM
	public const int FEDM_ISC_DATA_DBN = 0x1000003;
		// only BRM
	public const int FEDM_ISC_DATA_ANT_NR = 0x1000004;
		// only BRM
	public const int FEDM_ISC_DATA_DATE = 0x1000005;
		// only BRM
	public const int FEDM_ISC_DATA_INPUT = 0x1000006;
		// only BRM
	public const int FEDM_ISC_DATA_STATE = 0x1000007;
		// only ISO
	public const uint FEDM_ISC_DATA_IS_BLOCK_SIZE_SET = 0x80000001;
		// only ISO
	public const uint FEDM_ISC_DATA_IS_SELECTED = 0x80000002;
		// BRM and ISO
	public const uint FEDM_ISC_DATA_IS_SNR = 0x80000003;
		// only ISO
	public const uint FEDM_ISC_DATA_IS_EPC = 0x80000004;
		// only ISO
	public const uint FEDM_ISC_DATA_IS_AFI = 0x80000005;
		// only ISO
	public const uint FEDM_ISC_DATA_IS_DSFID = 0x80000006;
		// only ISO14443A
	public const uint FEDM_ISC_DATA_IS_TRINFO = 0x80000007;
		// only ISO
	public const uint FEDM_ISC_DATA_IS_SYSINFO = 0x80000008;
		// only BRM
	public const uint FEDM_ISC_DATA_IS_DB = 0x80000009;
		// only BRM
	public const uint FEDM_ISC_DATA_IS_TIMER = 0x8000000a;
		// only BRM
	public const uint FEDM_ISC_DATA_IS_ANT_NR = 0x8000000b;
		// only BRM
	public const uint FEDM_ISC_DATA_IS_DATE = 0x8000000c;
		// only ISO14443-4
	public const uint FEDM_ISC_DATA_IS_ISO14443_4_INFO = 0x8000000d;
		// only BRM
	public const uint FEDM_ISC_DATA_IS_INPUT = 0x8000000e;
		// for initialize all with ResetTable
	public const uint FEDM_ISC_DATA_ALL = 0xffff0000;
		// only ISO
	public const int FEDM_ISC_DATA_MEM_SIZE_LEN = 0x8000019;
		// only ISO
	public const int FEDM_ISC_DATA_CHIP_ID = 0x8000020;
		// only ISO
	public const int FEDM_ISC_DATA_VALUE = 0x4000005;


	// only for EPCglobal Class1, Generation2 Transponder
	// the user memory bank can be accessed with FEDM_ISC_DATA_RxDB and FEDM_ISC_DATA_TxDB
		// only ISO
	public const int FEDM_ISC_DATA_RxDB_EPC_BANK = 0x4001001;
		// only ISO
	public const int FEDM_ISC_DATA_TxDB_EPC_BANK = 0x4001002;
		// only ISO
	public const int FEDM_ISC_DATA_RxDB_TID_BANK = 0x4001003;
		// only ISO
	public const int FEDM_ISC_DATA_TxDB_TID_BANK = 0x4001004;
		// only ISO
	public const int FEDM_ISC_DATA_RxDB_RES_BANK = 0x4001005;
		// only ISO
	public const int FEDM_ISC_DATA_TxDB_RES_BANK = 0x4001006;


	// access flags for the table CAD
	// (uiDataID in functions GetTableData, ResetTable)
		// average power value
	public const int FEDM_ISC_DATA_POWER_AVG = 0x1;
		// peak power value
	public const int FEDM_ISC_DATA_POWER_PEAK = 0x2;
		// history record: allocation of channels
	public const int FEDM_ISC_DATA_HISTORY_ALLOC = 0x4;
		// history record: transmit channel
	public const int FEDM_ISC_DATA_HISTORY_TX = 0x8;
		// history record: no. of receive channels
	public const int FEDM_ISC_DATA_HISTORY_RX = 0x10;
		// miscellaneous data
	public const int FEDM_ISC_DATA_MISC = 0x20;


	// ISO-Command mode flags
	public const int FEDM_ISC_ISO_MODE_NONADR = 0x0;
	public const int FEDM_ISC_ISO_MODE_ADR = 0x1;
	public const int FEDM_ISC_ISO_MODE_SEL = 0x2;
	public const int FEDM_ISC_ISO_MODE_SEC = 0x8;
		// only for jewel transponder
	public const int FEDM_ISC_ISO_MODE_WR_NE = 0x8;
	public const int FEDM_ISC_ISO_MODE_UID_LF = 0x10;
		// only for ISO14443-Transponder and [= &HB0][= &H25] Select
	public const int FEDM_ISC_ISO_MODE_CINF = 0x20;
	public const int FEDM_ISC_ISO_MODE_EXT_ADR = 0x20;

	public const int FEDM_ISC_ISO_MODE_MORE = 0x80;
	// ISO-Command bank identifier for:
	// [0xB0][0x23] Read Multiple Blocks
	// [0xB0][0x24] Write Multiple Blocks
	public const int FEDM_ISC_ISO_BANK_RESERVED = 0x0;
	public const int FEDM_ISC_ISO_BANK_EPC_MEMORY = 0x1;
	public const int FEDM_ISC_ISO_BANK_TID_MEMORY = 0x2;

	public const int FEDM_ISC_ISO_BANK_USER_MEMORY = 0x3;


	// Manufacturer Code for [= &HB1] ISO-15693 Host Commands
	public const int FEDM_ISC_ISO_MFR_PHILIPS = 0x4;
	public const int FEDM_ISC_ISO_MFR_TI = 0x7;

	public const int FEDM_ISC_ISO_MFR_NXP = 0x4;

    // LF classic reader types
    public const int FEDM_RW_TYPE_RW40_XX = 20; // ID RW40.30-U

	// HF reader types
	public const int FEDM_ISC_TYPE_ISCM01 = 30;
	public const int FEDM_ISC_TYPE_ISCM02 = 31;
    public const int FEDM_ISC_TYPE_ISCM02_M8 = 33;
	public const int FEDM_ISC_TYPE_ISCLR100 = 40;
	public const int FEDM_ISC_TYPE_ISCLR200 = 41;
	public const int FEDM_ISC_TYPE_ISCLR2000 = 42;
    public const int FEDM_ISC_TYPE_ISCLR2500_B = 43;
    public const int FEDM_ISC_TYPE_ISCLR2500_A = 44;
    public const int FEDM_ISC_TYPE_ISCLR1002 = 45;
	public const int FEDM_ISC_TYPE_ISCPRH101 = 60;
	public const int FEDM_ISC_TYPE_ISCPRH101_U = 61;
	public const int FEDM_ISC_TYPE_ISCPRHD102 = 62;
    public const int FEDM_ISC_TYPE_ISCPRH102 = 63;
	public const int FEDM_ISC_TYPE_ISCPRH100_U = 71;
	public const int FEDM_ISC_TYPE_ISCPRH100 = 72;
	public const int FEDM_ISC_TYPE_ISCMR100_U = 73;
	public const int FEDM_ISC_TYPE_ISCMR100 = 74;
	public const int FEDM_ISC_TYPE_ISCPR100 = 74;
	public const int FEDM_ISC_TYPE_ISCMR200 = 75;
	public const int FEDM_ISC_TYPE_ISCMR101 = 76;
    public const int FEDM_ISC_TYPE_ISCMR102 = 77;
	public const int FEDM_ISC_TYPE_ISCMR101_U = 78;

    // HF classsic-pro reader types
    public const int FEDM_ISC_TYPE_CPRM02 =	80;
    public const int FEDM_ISC_TYPE_CPR02 = 81;
    public const int FEDM_ISC_TYPE_CPR40_XX_U =	82;
    public const int FEDM_ISC_TYPE_CPR40_XX	= 83;
    public const int FEDM_ISC_TYPE_CPR50_XX	= 84;
    public const int FEDM_ISC_TYPE_CPR44_XX	= 85;
    public const int FEDM_ISC_TYPE_CPR30_XX = 86;
    public const int FEDM_ISC_TYPE_CPR52_XX	= 87;
    public const int FEDM_ISC_TYPE_CPR04_U = 88;
    public const int FEDM_ISC_TYPE_CPR46_XX	= 89;
    public const int FEDM_ISC_TYPE_CPR20_XX	= 110;



    // OBID myAXXESS reader types
    public const int FEDM_ISC_TYPE_MAX50_XX	= 100;
    public const int FEDM_ISC_TYPE_MAX_STANDARD	= 102;
    public const int FEDM_ISC_TYPE_MAX_COMFORT = 103;

	// UHF reader types
    public const int FEDM_ISC_TYPE_ISCMU02 = 50;
    public const int FEDM_ISC_TYPE_ISCMU95 = 51; // Neuer Reader für das Gerät MC95
    public const int FEDM_ISC_TYPE_ISCMRU102 = 54;
    public const int FEDM_ISC_TYPE_ISCMRU200 = 55;
    public const int FEDM_ISC_TYPE_ISCLRU1000 = 92;
    public const int FEDM_ISC_TYPE_ISCLRU2000 = 93;
    public const int FEDM_ISC_TYPE_ISCLRU3000 = 94;


    // OBID megalock C reader types
    public const int FEDM_ISC_TYPE_MLC_COM = 120;
    public const int FEDM_ISC_TYPE_MLC_EMPS = 121;

	// name of readers
	public const string FEDM_ISC_NAME_ANT_DAT =	"ID ANT.DAT";	// HF Dynamic Antenna Tuner
    public const string FEDM_ISC_NAME_ANT_UMUX =	"ID ANT.UMUX";	// UHF Antenna Multiplexer
    public const string FEDM_ISC_NAME_ANT_GPC =	"ID ANT.GPC";	// Gate People Counter
    public const string FEDM_ISC_NAME_ISCM01 = "ID ISC.M01";
    public const string FEDM_ISC_NAME_ISCM02 =	"ID ISC.M02";
    public const string FEDM_ISC_NAME_ISCMU95 = "ID ISC.MU95";
    public const string FEDM_ISC_NAME_ISCM02_M8 = "ID ISC.M02.M8";
    public const string FEDM_ISC_NAME_ISCLR100 = "ID ISC.LR100";
    public const string FEDM_ISC_NAME_ISCLR200	=	"ID ISC.LR200";
    public const string FEDM_ISC_NAME_ISCLR2000=		"ID ISC.LR2000";
    public const string FEDM_ISC_NAME_ISCLRM2000=	"ID ISC.LRM2000";
    public const string FEDM_ISC_NAME_ISCPRHD102_X = "ID ISCUM02/PRHD102";
    public const string FEDM_ISC_NAME_ISCLR2500_A=	"ID ISC.LR2500-A";
    public const string FEDM_ISC_NAME_ISCLR2500_B=	"ID ISC.LR2500-B";
    public const string FEDM_ISC_NAME_ISCLR1002=		"ID ISC.LR1002";
    public const string FEDM_ISC_NAME_ISCPRH101=		"ID ISC.PRH101";
    public const string FEDM_ISC_NAME_ISCPRH101_U=	"ID ISC.PRH101-U";
    public const string FEDM_ISC_NAME_ISCPRH102=		"ID ISC.PRH102";
    public const string FEDM_ISC_NAME_ISCPRHD102=	"ID ISC.PRHD102";
    public const string FEDM_ISC_NAME_ISCPRH100_U=	"ID ISC.PRH100-U";
    public const string FEDM_ISC_NAME_ISCPRH100=		"ID ISC.PRH100";
    public const string FEDM_ISC_NAME_ISCMR100_U=	"ID ISC.MR100-U";
    public const string FEDM_ISC_NAME_ISCMR100	=	"ID ISC.MR100";
    public const string FEDM_ISC_NAME_ISCPR100	=	"ID ISC.PR100";
    public const string FEDM_ISC_NAME_ISCMR200	=	"ID ISC.MR200";
    public const string FEDM_ISC_NAME_ISCMR101	=	"ID ISC.MR101";
    public const string FEDM_ISC_NAME_ISCMR101_U=	"ID ISC.MR101-U";
    public const string FEDM_ISC_NAME_ISCMR102	=	"ID ISC.MR102";

    // experimental reader, identified by reader class, class generation and reader type
    public const string FEDM_ISC_NAME_ISCMR10X	=	"OBID i-scan HF Mid-Range Type 1";

    public const string FEDM_ISC_NAME_CPRM02	=	"ID CPR.M02";
    public const string FEDM_ISC_NAME_CPR02	=		"ID CPR.02";
    public const string FEDM_ISC_NAME_CPR04_U	=	"ID CPR.04-USB";
    public const string FEDM_ISC_NAME_CPR20_XX	=	"ID CPR20.xx";
    public const string FEDM_ISC_NAME_CPR40_XX_U=	"ID CPR40.xx-U";
    public const string FEDM_ISC_NAME_CPR40_XX	=	"ID CPR40.xx";
    public const string FEDM_ISC_NAME_CPR50_XX	=	"ID CPR50.xx";
    public const string FEDM_ISC_NAME_CPR30_XX	=	"ID CPR30.xx";
    public const string FEDM_ISC_NAME_CPR52_XX	=	"ID CPR52.xx";
    public const string FEDM_ISC_NAME_CPR44_XX	=	"ID CPR44.xx";
    public const string FEDM_ISC_NAME_CPR46_XX	=	"ID CPR46.xx";

    public const string FEDM_ISC_NAME_MAX50_XX	=	"ID MAX50.xx";
    public const string FEDM_ISC_NAME_MAX_STANDARD=	"OBID myAXXESS standard";
    public const string FEDM_ISC_NAME_MAX_COMFORT	="OBID myAXXESS comfort";

    public const string FEDM_ISC_NAME_ISCMU02	=	"ID ISC.MU02";			// UHF
    public const string FEDM_ISC_NAME_ISCMRU102=		"ID ISC.MRU102";			// UHF
    public const string FEDM_ISC_NAME_ISCMRU200=		"ID ISC.MRU200";			// UHF
    public const string FEDM_ISC_NAME_ISCLRU1000=	"ID ISC.LRU1000";		// UHF
    public const string FEDM_ISC_NAME_ISCLRMU1000=	"ID ISC.LRMU1000";		// UHF
    public const string FEDM_ISC_NAME_ISCLRU2000=	"ID ISC.LRU2000";		// UHF
    public const string FEDM_ISC_NAME_ISCLRMU2000=	"ID ISC.LRMU2000";		// UHF
    public const string FEDM_ISC_NAME_ISCLRU3000=	"ID ISC.LRU3000";		// UHF
    public const string FEDM_ISC_NAME_ISCLRU3500=	"ID ISC.LRU3500";		// UHF

    public const string FEDM_ISC_NAME_MLC_COM	=	"ID MLC.COM";
    public const string FEDM_ISC_NAME_MLC_EMPS	=	"ID MLC.EMPS";

	// HF transponder types
	public const int FEDM_ISC_TR_TYPE_ICODE1 = 0x00;
	public const int FEDM_ISC_TR_TYPE_TAGIT = 0x01;
	public const int FEDM_ISC_TR_TYPE_ISO15693 = 0x03;
	public const int FEDM_ISC_TR_TYPE_ISO14443A = 0x04;
	public const int FEDM_ISC_TR_TYPE_ISO14443B = 0x05;
	public const int FEDM_ISC_TR_TYPE_EPC = 0x06;
	public const int FEDM_ISC_TR_TYPE_ICODE_UID = 0x07;
	public const int FEDM_ISC_TR_TYPE_JEWEL = 0x08;
    public const int FEDM_ISC_TR_TYPE_ISO18000_3M3 = 0x09;
    public const int FEDM_ISC_TR_TYPE_STM_SR176 = 0x0A;
    public const int FEDM_ISC_TR_TYPE_STM_SRIxx = 0x0B;
    public const int FEDM_ISC_TR_TYPE_MCRFxxx = 0x0C;
    public const int FEDM_ISC_TR_TYPE_KEYBOARD = 0x0E; // it's not a transponder, but handles so
    public const int FEDM_ISC_TR_TYPE_INNOVATRON	= 0x10;
    public const int FEDM_ISC_TR_TYPE_ASK_CTx = 0x11;
	public const int FEDM_ISC_TR_TYPE_0x64 = 0x64;

	// UHF transponder types
	public const int FEDM_ISC_TR_TYPE_ISO18000_6_A = 0x80;
	public const int FEDM_ISC_TR_TYPE_ISO18000_6_B = 0x81;
	public const int FEDM_ISC_TR_TYPE_EM4222 = 0x83;
	public const int FEDM_ISC_TR_TYPE_EPC_CLASS1_GEN2 = 0x84;
	public const int FEDM_ISC_TR_TYPE_EPC_CLASS0 = 0x88;
	public const int FEDM_ISC_TR_TYPE_EPC_CLASS1_GEN1 = 0x89;

    // HF transponder names
    public const string FEDM_ISC_TR_NAME_ICODE1 = "NXP I-Code1";
    public const string FEDM_ISC_TR_NAME_TAGIT	= "Texas Instruments Tag-it HF";
    public const string FEDM_ISC_TR_NAME_ISO15693	=	"ISO 15693";
    public const string FEDM_ISC_TR_NAME_ISO14443A	=	"ISO 14443-A";
    public const string FEDM_ISC_TR_NAME_ISO14443B	=	"ISO 14443-B";
    public const string FEDM_ISC_TR_NAME_EPC		=	"I-CODE EPC";
    public const string FEDM_ISC_TR_NAME_ICODE_UID	=	"I-CODE UID";
    public const string FEDM_ISC_TR_NAME_JEWEL		=	"Jewel";
    public const string FEDM_ISC_TR_NAME_ISO18000_3M3=	"ISO 18000-3M3";
    public const string FEDM_ISC_TR_NAME_STM_SR176	=	"STMicroelectronics SR176"; // ISO 14443-B
    public const string FEDM_ISC_TR_NAME_STM_SRIxx	=	"STMicroelectronics SRIxx"; // ISO 14443-B
    public const string FEDM_ISC_TR_NAME_MCRFxxx	=	"Microchip MCRFxxx";
    public const string FEDM_ISC_TR_NAME_0x64		=	"ISO 15693";
    public const string FEDM_ISC_TR_NAME_INNOVATRON=		"Innovatron (ISO 14443B')";
    public const string FEDM_ISC_TR_NAME_ASK_CTx	=	"ASK CTx";

    // UHF transponder names
    public const string FEDM_ISC_TR_NAME_ISO18000_6_A	="ISO 18000-6-A";
    public const string FEDM_ISC_TR_NAME_ISO18000_6_B	="ISO 18000-6-B";
    public const string FEDM_ISC_TR_NAME_EM4222		=	"EM4222";
    public const string FEDM_ISC_TR_NAME_EPC_CLASS0	=	"EPC Class 0";
    public const string FEDM_ISC_TR_NAME_EPC_CLASS1_GEN1="EPC Class 1 Gen 1";
    public const string FEDM_ISC_TR_NAME_EPC_CLASS1_GEN2="EPC Class 1 Gen 2";

	// EPC types (EPC = Electronic Product Code)
		// 96 bit EPC
	public const int FEDM_ISC_EPC_TYPE_1 = 0x0;
		// 64 bit EPC Type I
	public const int FEDM_ISC_EPC_TYPE_2 = 0x1;
		// 64 bit EPC Type II
	public const int FEDM_ISC_EPC_TYPE_3 = 0x2;
		// 64 bit EPC Type III
	public const int FEDM_ISC_EPC_TYPE_4 = 0x3;
	#endregion

	#region "FECOM DLL"


	// #####################################################
	// FECOM DLL 
	// #####################################################

	[DllImport(dllName_com, EntryPoint = "FECOM_GetDLLVersion", SetLastError = true)]
	private static extern void FECOM_GetDLLVersion(byte[] cVersion);
	[DllImport(dllName_com, EntryPoint = "FECOM_OpenPort", SetLastError = true)]
	private static extern int FECOM_OpenPort(byte[] cPortNr);
	[DllImport(dllName_com, EntryPoint = "FECOM_ClosePort", SetLastError = true)]
	private static extern int FECOM_ClosePort(int iPortHnd);
	[DllImport(dllName_com, EntryPoint = "FECOM_SetPortPara", SetLastError = true)]
	private static extern int FECOM_SetPortPara(int iPortHnd, byte[] cPara, byte[] cValue);
	[DllImport(dllName_com, EntryPoint = "FECOM_DoPortCmd", SetLastError = true)]
	private static extern int FECOM_DoPortCmd(int iPortHnd, byte[] cCmd, byte[] cValue);
	[DllImport(dllName_com, EntryPoint = "FECOM_AddEventHandler", SetLastError = true)]
	private static extern int FECOM_AddEventHandler(int iPortHnd, ref FECOM_EVENT_INIT pInit);
    [DllImport(dllName_com, EntryPoint = "FECOM_GetErrorText", SetLastError = true)]
    private static extern int FECOM_GetErrorText(int iErrorCode, ref string cErrorText);
    [DllImport(dllName_com, EntryPoint = "FECOM_GetLastError", SetLastError = true)]
    private static extern int FECOM_GetLastError(int iPortHnd, ref int iErrorCode, ref string cErrorText);
    [DllImport(dllName_com, EntryPoint = "FECOM_DetectPort", SetLastError = true)]
    private static extern int FECOM_DetectPort(int iPortNr);
    [DllImport(dllName_com, EntryPoint = "FECOM_DelEventHandler", SetLastError = true)]
    private static extern int FECOM_DelEventHandler(int iPortHnd, ref FECOM_EVENT_INIT pInit);
    [DllImport(dllName_com, EntryPoint = "FECOM_GetPortList", SetLastError = true)]
    private static extern int FECOM_GetPortList(int iNext);
    [DllImport(dllName_com, EntryPoint = "FECOM_GetPortPara", SetLastError = true)]
    private static extern int FECOM_GetPortPara(int iPortHnd, ref string cPara, ref string cValue);
    [DllImport(dllName_com, EntryPoint = "FECOM_GetPortHnd", SetLastError = true)]
    private static extern int FECOM_GetPortHnd(ref string cPortNr);
    [DllImport(dllName_com, EntryPoint = "FECOM_Transceive", SetLastError = true)]
    private static extern int FECOM_Transceive(int iPortHnd, ref byte cSendProt, int iSendLen, ref byte cRecProt, int iRecLen);
    [DllImport(dllName_com, EntryPoint = "FECOM_Transmit",SetLastError = true)]
    private static extern int FECOM_Transmit(int iPortHnd, ref byte cSendProt, int iSendLen);
    [DllImport(dllName_com, EntryPoint = "FECOM_Receive",SetLastError = true)]
    private static extern int FECOM_Receive(int iPortHnd, ref byte cRecProt, int iRecLen);
	//Declare Function FECOM_AddEventHandler Lib "fecomce.dll" Alias "FECOM_AddEventHandler" (ByVal iPortHnd As Integer, ByRef pInit As FECOM_EVENT_INIT) As Integer
	#endregion

	#region "FEISC DLL"

	// #####################################################
	// FEISC declarate
	// #####################################################

	[DllImport(dllName_isc, EntryPoint = "FEISC_SetReaderPara", SetLastError = true)]
	private static extern int FEISC_SetReaderPara(int iReaderHnd, byte[] cPara, byte[] cValue);
	[DllImport(dllName_isc, EntryPoint = "FEISC_GetDLLVersion", SetLastError = true)]
	private static extern void FEISC_GetDLLVersion(byte[] cVersion);
	[DllImport(dllName_isc, EntryPoint = "FEISC_NewReader", SetLastError = true)]
	private static extern int FEISC_NewReader(int iPortHnd);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x66_ReaderInfo", SetLastError = true)]
	private static extern int FEISC_0x66_ReaderInfo(int iReaderHnd, byte cBusAdr, byte cMode, byte[] cVersion, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x65_SoftVersion", SetLastError = true)]
	private static extern int FEISC_0x65_SoftVersion(int iReaderHnd, byte cBusAdr, byte[] cVersion, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_DeleteReader", SetLastError = true)]
	private static extern int FEISC_DeleteReader(int iReaderHnd);
    
    [DllImport(dllName_isc, EntryPoint = "FEISC_0x80_ReadConfBlock", SetLastError = true)]
    private static extern int FEISC_0x80_ReadConfBlock(int iReaderHnd, byte cBusAdr, byte cConfAdr,  byte[] cConfBlock, int iDataFormat);
	
    [DllImport(dllName_isc, EntryPoint = "FEISC_0x81_WriteConfBlock", SetLastError = true)]
	private static extern int FEISC_0x81_WriteConfBlock(int iReaderHnd, byte cBusAdr, byte cConfAdr, byte[] cConfBlock, int iDataFormat);

    [DllImport(dllName_isc, EntryPoint = "FEISC_0x83_ResetConfBlock", SetLastError = true)]
    private static extern int FEISC_0x83_ResetConfBlock(int iReaderHnd, byte cBusAdr, byte cConfAdr);
	
    [DllImport(dllName_isc, EntryPoint = "FEISC_0x82_SaveConfBlock", SetLastError = true)]
	private static extern int FEISC_0x82_SaveConfBlock(int iReaderHnd, byte cBusAdr, byte cConfAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x69_RFReset", SetLastError = true)]
	private static extern int FEISC_0x69_RFReset(int iReaderHnd, byte cBusAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0xB0_ISOCmd", SetLastError = true)]
	private static extern int FEISC_0xB0_ISOCmd(int iReaderHnd, byte cBusAdr, byte[] cReqData, int iReqLen, byte[] cRspData, ref int iRspLen, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0xB2_ISOCmd", SetLastError = true)]
	private static extern int FEISC_0xB2_ISOCmd(int iReaderHnd, byte cBusAdr, byte[] cReqData, int iReqLen, byte[] cRspData, ref int iRspLen, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_AddEventHandler", SetLastError = true)]
	private static extern int FEISC_AddEventHandler(int iReaderHnd, ref FEISC_EVENT_INIT pInit);
	[DllImport(dllName_isc, EntryPoint = "FEISC_GetErrorText", SetLastError = true)]
	private static extern int FEISC_GetErrorText(int iErrorCode, byte[] cErrorText);
	[DllImport(dllName_isc, EntryPoint = "FEISC_GetStatusText", SetLastError = true)]
	private static extern int FEISC_GetStatusText(byte ucStatus, ref char cStatusText);
	[DllImport(dllName_isc, EntryPoint = "FEISC_DelEventHandler", SetLastError = true)]
	private static extern int FEISC_DelEventHandler(int iReaderHnd, ref FEISC_EVENT_INIT pInit);
	[DllImport(dllName_isc, EntryPoint = "FEISC_SetCbPtr_ProtStr", SetLastError = true)]
	private static extern void FEISC_SetCbPtr_ProtStr(byte[] cbPtr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_GetReaderList", SetLastError = true)]
	private static extern int FEISC_GetReaderList(int iNext);
	[DllImport(dllName_isc, EntryPoint = "FEISC_GetReaderPara", SetLastError = true)]
	private static extern int FEISC_GetReaderPara(int iReaderHnd, ref char cPara, ref char cValue);
	[DllImport(dllName_isc, EntryPoint = "FEISC_InstallPlugIn", SetLastError = true)]
	private static extern int FEISC_InstallPlugIn(int iReaderHnd, int iPlugInID, byte[] pPlugIn);
	[DllImport(dllName_isc, EntryPoint = "FEISC_RemovePlugIn", SetLastError = true)]
	private static extern int FEISC_RemovePlugIn(int iReaderHnd, int iPlugInID);
	[DllImport(dllName_isc, EntryPoint = "FEISC_StartAsyncTask", SetLastError = true)]
	private static extern int FEISC_StartAsyncTask(int iReaderHnd, int iTaskID, ref FEISC_TASK_INIT pInit, byte[] pInput);
	[DllImport(dllName_isc, EntryPoint = "FEISC_CancelAsyncTask", SetLastError = true)]
	private static extern int FEISC_CancelAsyncTask(int iReaderHnd);
	[DllImport(dllName_isc, EntryPoint = "FEISC_TriggerAsyncTask", SetLastError = true)]
	private static extern int FEISC_TriggerAsyncTask(int iReaderHnd);
	[DllImport(dllName_isc, EntryPoint = "FEISC_BuildSendProtocol", SetLastError = true)]
	private static extern int FEISC_BuildSendProtocol(int iReaderHnd, byte cBusAdr, byte cCmdByte, ref byte cSendData, int iDataLen, ref byte cSendProt, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_BuildRecProtocol", SetLastError = true)]
	private static extern int FEISC_BuildRecProtocol(int iReaderHnd, byte cBusAdr, byte cCmdByte, byte cStatus, ref byte cRecData, int iDataLen, ref byte cRecProt, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_SplitSendProtocol", SetLastError = true)]
	private static extern int FEISC_SplitSendProtocol(int iReaderHnd, ref byte cSendProt, int iSendLen, ref byte cBusAdr, ref byte cCmdByte, ref byte cSendData, ref int iDataLen, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_SplitRecProtocol", SetLastError = true)]
	private static extern int FEISC_SplitRecProtocol(int iReaderHnd, ref byte cRecProt, int iRecLen, ref byte cBusAdr, ref byte cCmdByte, ref byte cRecData, ref int iDataLen, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_Conv2StdProtocol", SetLastError = true)]
	private static extern int FEISC_Conv2StdProtocol(ref byte cProt, int iLen);
	[DllImport(dllName_isc, EntryPoint = "FEISC_Conv2AdvProtocol", SetLastError = true)]
	private static extern int FEISC_Conv2AdvProtocol(ref byte cProt, int iLen);
	[DllImport(dllName_isc, EntryPoint = "FEISC_GetLastSendProt", SetLastError = true)]
	private static extern int FEISC_GetLastSendProt(int iReaderHnd, ref byte cSendProt, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_GetLastRecProt", SetLastError = true)]
	private static extern int FEISC_GetLastRecProt(int iReaderHnd, ref byte cRecProt, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_GetLastState", SetLastError = true)]
	private static extern int FEISC_GetLastState(int iReaderHnd, ref char cStateText);
	[DllImport(dllName_isc, EntryPoint = "FEISC_GetLastRecProtLen", SetLastError = true)]
	private static extern int FEISC_GetLastRecProtLen(int iReaderHnd);
	[DllImport(dllName_isc, EntryPoint = "FEISC_GetLastError", SetLastError = true)]
	private static extern int FEISC_GetLastError(int iReaderHnd, ref int iErrorCode, ref char cErrorText);
	[DllImport(dllName_isc, EntryPoint = "FEISC_SendTransparent", SetLastError = true)]
	private static extern int FEISC_SendTransparent(int iReaderHnd, ref byte cSendProt, int iSendLen, ref byte cRecProt, int iMaxRecLen, int iCheckSum, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_Transmit", SetLastError = true)]
	private static extern int FEISC_Transmit(int iReaderHnd, ref byte cSendProt, int iSendLen, int iCheckSum, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_Receive", SetLastError = true)]
	private static extern int FEISC_Receive(int iReaderHnd, ref byte cRecProt, int iMaxRecLen, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x18_Destroy", SetLastError = true)]
	private static extern int FEISC_0x18_Destroy(int iReaderHnd, byte cBusAdr, byte ucMode, ref byte ucEPC, ref byte ucPW);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x1A_Halt", SetLastError = true)]
	private static extern int FEISC_0x1A_Halt(int iReaderHnd, byte cBusAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x1B_ResetQuietBit", SetLastError = true)]
	private static extern int FEISC_0x1B_ResetQuietBit(int iReaderHnd, byte cBusAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x1C_EASRequest", SetLastError = true)]
	private static extern int FEISC_0x1C_EASRequest(int iReaderHnd, byte cBusAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x21_ReadBuffer", SetLastError = true)]
	private static extern int FEISC_0x21_ReadBuffer(int iReaderHnd, byte cBusAdr, byte cSets, ref byte cTrData, ref byte cRecSets, ref byte cRecDataSets, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x22_ReadBuffer", SetLastError = true)]
	private static extern int FEISC_0x22_ReadBuffer(int iReaderHnd, byte cBusAdr, int iSets, ref byte cTrData, ref int iRecSets, ref byte cRecDataSets, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x31_ReadDataBufferInfo", SetLastError = true)]
	private static extern int FEISC_0x31_ReadDataBufferInfo(int iReaderHnd, byte cBusAdr, ref byte cTabSize, ref byte cTabStart, ref byte cTabLen, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x32_ClearDataBuffer", SetLastError = true)]
	private static extern int FEISC_0x32_ClearDataBuffer(int iReaderHnd, byte cBusAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x33_InitBuffer", SetLastError = true)]
	private static extern int FEISC_0x33_InitBuffer(int iReaderHnd, byte cBusAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x34_ForceNotifyTrigger", SetLastError = true)]
	private static extern int FEISC_0x34_ForceNotifyTrigger(int iReaderHnd, byte cBusAdr, byte cMode);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x52_GetBaud", SetLastError = true)]
	private static extern int FEISC_0x52_GetBaud(int iReaderHnd, byte cBusAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x55_StartFlashLoader", SetLastError = true)]
	private static extern int FEISC_0x55_StartFlashLoader(int iReaderHnd);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x55_StartFlashLoaderEx", SetLastError = true)]
	private static extern int FEISC_0x55_StartFlashLoaderEx(int iReaderHnd, byte cBusAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x63_CPUReset", SetLastError = true)]
	private static extern int FEISC_0x63_CPUReset(int iReaderHnd, byte cBusAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x64_SystemReset", SetLastError = true)]
	private static extern int FEISC_0x64_SystemReset(int iReaderHnd, byte cBusAdr, byte cMode);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x68_ChannelAnalyze", SetLastError = true)]
	private static extern int FEISC_0x68_ChannelAnalyze(int iReaderHnd, byte cBusAdr, byte cMode, byte cRes1, byte cRes2, byte cRes3, int iRes4, ref int nData);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x6A_RFOnOff", SetLastError = true)]
	private static extern int FEISC_0x6A_RFOnOff(int iReaderHnd, byte cBusAdr, byte cRF);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x6B_InitNoiseThreshold", SetLastError = true)]
	private static extern int FEISC_0x6B_InitNoiseThreshold(int iReaderHnd, byte cBusAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x6B_CentralizedRFSync", SetLastError = true)]
	private static extern int FEISC_0x6B_CentralizedRFSync(int iReaderHnd, byte cBusAdr, byte cMode, byte cTxChannel, int iTxPeriod, byte cRes1, byte cRes2);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x6C_SetNoiseLevel", SetLastError = true)]
	private static extern int FEISC_0x6C_SetNoiseLevel(int iReaderHnd, byte cBusAdr, ref byte cLevel, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x6D_GetNoiseLevel", SetLastError = true)]
	private static extern int FEISC_0x6D_GetNoiseLevel(int iReaderHnd, byte cBusAdr, ref byte cLevel, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x6E_RdDiag", SetLastError = true)]
	private static extern int FEISC_0x6E_RdDiag(int iReaderHnd, byte cBusAdr, byte cMode, ref byte cData);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x6F_AntennaTuning", SetLastError = true)]
	private static extern int FEISC_0x6F_AntennaTuning(int iReaderHnd, byte cBusAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x71_SetOutput", SetLastError = true)]
	private static extern int FEISC_0x71_SetOutput(int iReaderHnd, byte cBusAdr, int iOS, int iOSF, int iOSTime, int iOutTime);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x72_SetOutput", SetLastError = true)]
	private static extern int FEISC_0x72_SetOutput(int iReaderHnd, byte cBusAdr, byte cMode, byte cOutN, ref byte pRecords);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x74_ReadInput", SetLastError = true)]
	private static extern int FEISC_0x74_ReadInput(int iReaderHnd, byte cBusAdr, ref byte cInput);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x75_AdjAntenna", SetLastError = true)]
	private static extern int FEISC_0x75_AdjAntenna(int iReaderHnd, byte cBusAdr, ref byte cValue, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x84_SetCFGMemLoc", SetLastError = true)]
	private static extern int FEISC_0x84_SetCFGMemLoc(int iReaderHnd, byte cBusAdr, byte cMemAdr);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x85_SetSysTimer", SetLastError = true)]
	private static extern int FEISC_0x85_SetSysTimer(int iReaderHnd, byte cBusAdr, ref byte cTime, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x86_GetSysTimer", SetLastError = true)]
	private static extern int FEISC_0x86_GetSysTimer(int iReaderHnd, byte cBusAdr, ref byte cTime, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x87_SetSystemDate", SetLastError = true)]
	private static extern int FEISC_0x87_SetSystemDate(int iReaderHnd, byte cBusAdr, byte cCentury, byte cYear, byte cMonth, byte cDay, byte cTimezone, byte cHour, byte cMinute, int iMilliSecond);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0x88_GetSystemDate", SetLastError = true)]
	private static extern int FEISC_0x88_GetSystemDate(int iReaderHnd, byte cBusAdr, ref byte cCentury, ref byte cYear, ref byte cMonth, ref byte cDay, ref byte cTimezone, ref byte cHour, ref byte cMinute, ref int iMilliSecond);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0xA0_RdLogin", SetLastError = true)]
	private static extern int FEISC_0xA0_RdLogin(int iReaderHnd, byte cBusAdr, ref byte cRd_PW, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0xA2_WriteMifareKeys", SetLastError = true)]
	private static extern int FEISC_0xA2_WriteMifareKeys(int iReaderHnd, byte cBusAdr, byte c_Type, byte cAdr, ref byte cKey, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0xB1_ISOCust", SetLastError = true)]
	private static extern int FEISC_0xB1_ISOCust(int iReaderHnd, byte cBusAdr, byte cMfr, ref byte cReqData, int iReqLen, ref byte cRspData, ref int iRspLen, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0xB3_EPCCmd", SetLastError = true)]
	private static extern int FEISC_0xB3_EPCCmd(int iReaderHnd, byte cBusAdr, ref byte cReqData, int iReqLen, ref byte cRspData, ref int iRspLen, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0xB4_EPC_UHF_Cmd", SetLastError = true)]
	private static extern int FEISC_0xB4_EPC_UHF_Cmd(int iReaderHnd, byte cBusAdr, byte cMfr, ref byte cReqData, int iReqLen, ref byte cRspData, ref int iRspLen, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0xBC_CmdQueue", SetLastError = true)]
	private static extern int FEISC_0xBC_CmdQueue(int iReaderHnd, int iMode, int iCmdNo, ref byte cCmdQueue, int iCmdQueueLen, ref FEISC_TASK_INIT pInit);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0xBD_ISOTranspCmd", SetLastError = true)]
	private static extern int FEISC_0xBD_ISOTranspCmd(int iReaderHnd, byte cBusAdr, int iMode, int iRspLength, ref byte cReqData, int iReqLen, ref byte cRspData, ref int iRspLen, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0xBE_ISOTranspCmd", SetLastError = true)]
	private static extern int FEISC_0xBE_ISOTranspCmd(int iReaderHnd, byte cBusAdr, int iMode, int iRspLength, ref byte cReqData, int iReqLen, ref byte cRspData, ref int iRspLen, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0xBF_ISOTranspCmd", SetLastError = true)]
	private static extern int FEISC_0xBF_ISOTranspCmd(int iReaderHnd, byte cBusAdr, int iMode, int iRspLength, ref byte cReqData, int iReqLen, ref byte cRspData, ref int iRspLen, int iDataFormat);
	[DllImport(dllName_isc, EntryPoint = "FEISC_0xC0_SAMCmd", SetLastError = true)]
	private static extern int FEISC_0xC0_SAMCmd(int iReaderHnd, int iSlot, ref byte cReqData, int iReqLen, ref FEISC_TASK_INIT pInit);


	//Declare Function FEISC_AddEventHandler Lib "feiscce.dll" Alias "FEISC_AddEventHandler" (ByVal iReaderHnd As Integer, ByRef pInit As FEISC_EVENT_INIT) As Integer
	#endregion

	#region "Reader Vaiablen"

    private const string dllName_com = "fecomce_mc9090.dll";
    //private const string dllName_isc = "feiscce_mc9090.dll";
    //private const string dllName_usb = "feusb.dll";  // not in use // its MC95... PC
	//private const string dllName_com = "fecom.dll"; // PC
    //private const string dllName_isc = "feisc.dll"; // PC
    private const string dllName_usb = "feusbce.dll"; // CE
    private const string dllName_isc = "feiscce.dll";  // CE
		//Data of last Read operation
	private byte[] Read_Array;
		//Data of last Read operation
	private byte[] Write_Array;
		// Confog array for ISO Scan or ISO Host Mode
	private byte[] Config_Array;
		//Data for the ReaderInfo
	private byte[] Info_Array;
	private byte[] My_ComArray;
	public string[] InfoReader;
	private byte[] Error_Array;
	private byte[] cReqData;
	private byte[] cRspData;
	private byte[] UID_Array;
	private byte[] UID_Array24;

	private byte[] Value_Array;
	private byte[] SelMifare_Array;
	private byte[] Key_Array;
	private byte[] ReadMi_Array;

	private byte[] WriteMi_Array;
	private int iReaderHnd;
	private int iPortHnd;
	private int ideviceID;
	private byte iDataType = 0;
	private int iRspLen;
	private int iReqLen;
	private byte cBusAdr = 255;

	private byte cMode = 0;
	private string sBaud = "38400";
	private string sFrame = "8E1";
	private string sTimeout = "1000";
	private int R_Result;
	public string UIDNr = "";
    public string EPC_UHFNr = "";
	public string BlockDataHex = "";
	public string BlockDataAScII = "";

    public bool IsUHF = false;
    public bool IsHF = false;
	public string TagTyp;
	public string COMVersion;
	public string ISCVersion;
	public string ReaderInfo;
	public string SoftwareInfo;
	private string Info_Text;
	public bool IsActive = false;
	public bool IsSascha = false;
	public FECOM_EVENT_INIT m_Init;
	public bool LastCTS;
    
	public string MifareKey;
    public int LastErrorID;
    public int RFPower;
    public int ISOMode;
    public bool IsUSB = false;
    public bool IsCom = false;

    private System.Windows.Forms.Timer RepeatTimer = new System.Windows.Forms.Timer();


    #endregion

    #region "delegate test"
    public delegate void DelgOnMessage(IntPtr wParam, IntPtr lParam);
	public delegate void DelgComInfo(IntPtr wParam, IntPtr lParam);
	#endregion

	#region "Events"
	// These events allow the program using this class to react to Comm Port
	//   events.
	//    Public Event GetReaderText(ByVal Source As RFIDFeig, ByVal Info_Text As String)

	public event GetReaderTextEventHandler GetReaderText;
	public delegate void GetReaderTextEventHandler(string Info_Text);

	public event GetBlockDataEventHandler GetBlockData;
	public delegate void GetBlockDataEventHandler(int BlockNr, string BlockData);


	#endregion

    #region "Property"
    public int RW_TimeInterval
    {
        get {return RepeatTimer.Interval;}
        set {RepeatTimer.Interval = value;} 
    }
        
    #endregion 

    #region "utility"


    private byte[] ASCIIToByteArray(string str)
    {
        byte[] Result;
        ASCIIEncoding enc = new System.Text.ASCIIEncoding();
        Result = enc.GetBytes(str);
        return Result;
    }

    public string ByteArrayToASCII(byte[] arr)
    {
        string Result = string.Empty;
        System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
        Result = enc.GetString(arr, 0, arr.Length);
        return Result;
    }


	public void ClearStringArray(ref string[] src)
	{
		for (int i = src.GetLowerBound(0); i <= src.GetUpperBound(0); i++) {
			src[i] = "";
		}
	}

	public void ClearCharArray(ref char[] src)
	{
		for (int i = src.GetLowerBound(0); i <= src.GetUpperBound(0); i++) {
			src[i] = '\0';
		}
	}

	public void ClearByteArray(ref byte[] src)
	{
		for (int i = src.GetLowerBound(0); i <= src.GetUpperBound(0); i++) {
			src[i] = 0;
		}
       // Array.Clear(src, 0, src.Length);  another solution
       // for (int i; i = 0; i <= src.length) { src[i] = 0; }
	}

    public string GetInverted(string Data)
    {
        string Result = string.Empty;
        for (int i = Data.Length - 1; i >= 0; i += -1)
        {
            Result = Result + Data[i];
        }
        return Result;
    }


    public string GetByteDataBufferToHex(byte[] Src, int BStart, int BStop)
    {
        string Result = string.Empty; 
        for (int i = BStart; i <= BStop; i++)
        {
            Result += Src[i].ToString("X2");
        }
        return Result;
    }

    public string GetByteDataBufferToHexInverted(byte[] Src, int BStart, int BStop)
    {
        string Result = string.Empty;
        for (int i = BStop; i >= BStart; i += -1)
        {
            Result += Src[i].ToString("X2");
        }
        return Result;
    }

    public string GetByteBufferToHex(byte[] Src, int DataLen)
    {
        string Result = string.Empty; 
        for (int i = 0; i <= DataLen - 1; i++)
        {
            Result += Src[i].ToString("X2");
        }
        return Result;
    }

    public string ASCIIToHex(string Input)
    {
        string Result = string.Empty; 
        System.Text.StringBuilder sb = new System.Text.StringBuilder();
        foreach (char c in Input)
        {
            sb.Append(Convert.ToInt32(c).ToString("X2"));
        }
        Result = sb.ToString();
        return Result;
    }

    public string HexToASCII(string Input)
    {
        string Result = string.Empty; 
        System.Text.StringBuilder sb = new System.Text.StringBuilder();
        for (int i = 0; i <= Input.Length - 2; i += 2)
        {
            sb.Append(Convert.ToChar(Convert.ToByte(Input.Substring(i, 2), 16))).ToString();
        }
        Result = sb.ToString();
        return Result;
    }

    public int HexToInt(string Input)
    {
        return Convert.ToInt32(Input, 16);
    }


    public byte[] HexToByteArray(string input)
    {
        byte[] result;
        int c = 0;
        result = new byte[input.Length / 2];
        for (int i =0 ;i <= input.Length-1; i+= 2)
        {
            result[c] = Convert.ToByte(input.Substring(i, 2).ToString(),16);  
            c +=1; 
        }
        return result;
    }

    public string ByteToHex(byte[] src)
    {
        string Result = string.Empty; 
        for (int i = src.GetLowerBound(0); i <= src.GetUpperBound(0); i++)
        {
            Result += src[i].ToString("X2");
        }
        return Result;
    }

    public byte[] GetASCIIByte(string src)
    {
        byte[] Result;
        ASCIIEncoding enc = new System.Text.ASCIIEncoding();
        Result = enc.GetBytes(src);
        return Result;
    }

    public byte[] GetASCIIByteFix(string src)
    {
        byte[] Result = new byte[256];
        for (int i = 0; i <= src.Length - 1; i++)
        {
            Result[i] = (byte)Convert.ToChar(src.Substring(i, 1));
        }
        return Result;
    }

    public char[] GetASCIIChar(string src)
    {
        char[] Result = new char[src.Length - 1];
        Result = src.ToCharArray();
        return Result;
    }

    #endregion

    #region "Reader function/void"

    public int IsUSBDevice()
	{
		return FEUSB_IsDevicePresent(iPortHnd);
	}

    public bool InitializeUSB()
    {
        bool functionReturnValue = false;
        FEUSB_SCANSEARCH pSearchOp1 = new FEUSB_SCANSEARCH();
        functionReturnValue = false;
        IsActive = false;
        iReaderHnd = -1;
        iPortHnd = -1;
        iReaderHnd = FEISC_NewReader(0);
        if (iReaderHnd > 0)
        {
            SetReaderText("iReaderHnd = OK");
            FEUSB_ClearScanList();
            iPortHnd = FEUSB_ScanAndOpen(FEUSB_SCAN_FIRST, ref pSearchOp1);
            if (iPortHnd > 0)
            {
                SetReaderText("iPortHnd = OK");
                R_Result = FEISC_SetReaderPara(iReaderHnd, GetASCIIByte("PortHnd"), GetASCIIByte(iPortHnd.ToString()));
                functionReturnValue = true;
            }
            else
            {
                GetFeigErrorTextUSB(iPortHnd);
            }
        }
        IsActive = functionReturnValue;
        IsUSB = IsActive; 
        if (!IsActive)
        {
            ReaderUnload();
        }
        return functionReturnValue;
    }


    /*	public bool InitUSB_Old_Test()
	{
		bool functionReturnValue = false;
		int Back = 0;
		//Dim oSearch As New FEUSB_SCANSEARCH_BYTE
		byte[] sDeviceID = new byte[16];
		string[] strDeviceID = new string[16];
		long nDeviceID = 0;
		string mystring = null;
		FEUSB_SCANSEARCH_BYTE pSearchOpt = new FEUSB_SCANSEARCH_BYTE();
		pSearchOpt.iMask = 0;
		 // ERROR: Not supported in C#: ReDimStatement

		 // ERROR: Not supported in C#: ReDimStatement

		 // ERROR: Not supported in C#: ReDimStatement

		 // ERROR: Not supported in C#: ReDimStatement


		functionReturnValue = false;

		FEUSB_ClearScanList();
		// clear the device scan list

		//Back = FEUSB_Scan(FEUSB_SCAN_ALL, pSearchOpt);
		// scan USB for devices

		// did we find some devices ?
		if (Back < 0) {
			// no
			//Interaction.MsgBox("No RFID devices detected");
			return false;
		}

		Back = FEUSB_GetScanListPara(0, "Device-ID", sDeviceID);
		// get the device-id of the first detected device
		Back = FEUSB_GetScanListSize();
		// get the device-id of the first detected device


		if (Back == 1) {

			// ideviceID = HexToInt(CStr(sDeviceID.Value)) ' convert hex id to decimal
			mystring = ByteToASCII(sDeviceID);

			nDeviceID = BitConverter.ToUInt32(sDeviceID, 0);
			iPortHnd = FEUSB_OpenDevice(nDeviceID);
			// open the channel to USB device

			// did we open the channel ?
			if (iPortHnd > 0) {
				// yes, create a reader object
				iReaderHnd = FEISC_NewReader(iPortHnd);
				functionReturnValue = true;
			} else {
				GetFeigErrorTextUSB(iPortHnd);
			}
		}
		return functionReturnValue;
	*}*/
    /*	public bool InitializeUSB_Old1_Test()
	{
		bool functionReturnValue = false;
		//UID_Array = New Byte(11) {}
		//Type(FEUSB_SCANSEARCH = record)
		//iMask:                          Integer;
		//cFamily,cProduct,cInterface:    array[0..24] of Char;
		//cDeviceID:                      array[0..8] of Char;
		//end;
		FEUSB_SCANSEARCH_BYTE pSearchOpt = new FEUSB_SCANSEARCH_BYTE();
		pSearchOpt.iMask = 0;
		 // ERROR: Not supported in C#: ReDimStatement

		 // ERROR: Not supported in C#: ReDimStatement

		 // ERROR: Not supported in C#: ReDimStatement

		 // ERROR: Not supported in C#: ReDimStatement


		FEUSB_SCANSEARCH pSearchOp1 = new FEUSB_SCANSEARCH();


		functionReturnValue = false;
		IsActive = false;
		iReaderHnd = -1;
		iPortHnd = -1;
		iReaderHnd = FEISC_NewReader(0);
		if (iReaderHnd > 0) {
			if (GetReaderText != null) {
				GetReaderText("iReaderHnd = OK");
			}
			FEUSB_ClearScanList();
			iPortHnd = FEUSB_ScanAndOpen(FEUSB_SCAN_FIRST, ref pSearchOp1);
			if (iPortHnd > 0) {
				if (GetReaderText != null) {
					GetReaderText("iPortHnd = OK");
				}
				R_Result = FEISC_SetReaderPara(iReaderHnd, GetASCIIByte("PortHnd"), GetASCIIByte(iPortHnd.ToString()));
				functionReturnValue = true;
			} else {
				GetFeigErrorTextUSB(iPortHnd);
			}
		}
		IsActive = functionReturnValue;
		if (!IsActive) {
			ReaderUnload();
		} else {
			SetOn();
		}
		return functionReturnValue;
	}*/
	/*public bool InitializeUSB_Old_Test()
	{
		bool functionReturnValue = false;
		//UID_Array = New Byte(11) {}
		//Type(FEUSB_SCANSEARCH = record)
		//iMask:                          Integer;
		//cFamily,cProduct,cInterface:    array[0..24] of Char;
		//cDeviceID:                      array[0..8] of Char;
		//end;
		//ReDim pSearchOpt.cFamilyName(0 To 24) ' = "                         " 'New Char(25) {}
		//ReDim pSearchOpt.cDeviceName(0 To 24) ' = "                         " 'New Char(25) {}
		//ReDim pSearchOpt.cInterface(0 To 24) '= "                         " 'New Char(25) {}
		//ReDim pSearchOpt.cDeviceID(0 To 7) '= "         " 'New Char(16) {}
		int isA = 0;
		string[] sDeviceID = new string[9];
		FEUSB_SCANSEARCH pSearchOpt = new FEUSB_SCANSEARCH();
		pSearchOpt.iMask = 0;
		pSearchOpt.cFamily = "";
		pSearchOpt.cProduct = "";
		pSearchOpt.cInterface = "";
		pSearchOpt.cDeviceID = "";
		functionReturnValue = false;
		IsActive = false;
		iReaderHnd = -1;
		iPortHnd = -1;
		iReaderHnd = FEISC_NewReader(0);
		if (iReaderHnd > 0) {
			if (GetReaderText != null) {
				GetReaderText("iReaderHnd = OK");
			}
			R_Result = FEISC_SetReaderPara(iReaderHnd, GetASCIIByte("FrameSupport"), GetASCIIByte("Standart"));
			FEUSB_ClearScanList();
            iPortHnd = FEUSB_ScanAndOpen(FEUSB_SCAN_FIRST, ref pSearchOpt);
			if (iPortHnd > 0) {
				if (GetReaderText != null) {
					GetReaderText("iPortHnd = OK");
				}
				R_Result = FEISC_SetReaderPara(iReaderHnd, GetASCIIByte("PortHnd"), GetASCIIByte(iPortHnd.ToString()));
				functionReturnValue = true;
			} else {
				GetFeigErrorTextUSB(iPortHnd);
			}
		}
		IsActive = InitializeUSB_ALT();
		if (!IsActive) {
			ReaderUnload();
		} else {
			SetOn();
		}
		return InitializeUSB_ALT();
		return functionReturnValue;
	}*/


	public int ReaderUnload()
	{
		int functionReturnValue = 0;
		functionReturnValue = -1;
        RepeatTimer.Enabled = false; 
        if (iPortHnd > 0)
        {
            if (IsCom)
            {
                try
                { functionReturnValue = FECOM_ClosePort(iPortHnd); }
                catch (Exception ex)
                { /*SetReaderText(ex.Message.ToString());*/}
            }
            if (IsUSB)
            {
                try
                { functionReturnValue = FEUSB_CloseDevice(iPortHnd); }
                catch (Exception ex)
                {/* SetReaderText(ex.Message.ToString());*/ }
            }
        }
		if (iReaderHnd > 0)
			functionReturnValue = FEISC_DeleteReader(iReaderHnd);
		IsActive = false;
		return functionReturnValue;
	}

	public bool GetSoftwareVersion()
	{
		SoftwareInfo = "";
		ClearByteArray(ref Read_Array);
		R_Result = FEISC_0x65_SoftVersion(iReaderHnd, cBusAdr, Read_Array, iDataType);
		GetErrorText(R_Result);
        SoftwareInfo = "Software Revision" + ControlChars.CrLf + "Hardware Type" + ControlChars.CrLf + "Reader Firmware" + ControlChars.CrLf + "Transponder Types";
		SoftwareInfo = ByteToHex(Read_Array);
		SoftwareInfo = SoftwareInfo.Substring(1, 16);
		return R_Result == 0;
	}

	public bool GetReaderInfo(byte index)
	{
		string tmp = "";
		int tmpint = 0;
		//SetOn()
		//Firmware Version of RF Communication Controller:
		//Software Revision:.....00-07-00
		//Hardware Type:.........= &HFE
		//Reader Type:...........93 (ID ISC.LRU2000)
		//Transponder Types:.....= &H00 = &H12
		//Max. Rx buffer size:...00280
		//Max. Tx buffer size:...00520
		//000700 FE 5D 001201180208000000
		ClearStringArray(ref InfoReader);
		R_Result = FEISC_0x66_ReaderInfo(iReaderHnd, cBusAdr, index, Read_Array, iDataType);
		GetErrorText(R_Result);
		tmp = ByteToHex(Read_Array);
		InfoReader[0] = "Software Revision:" + tmp.Substring(0, 6);
		InfoReader[1] = "Hardware Type:= 0x" + tmp.Substring(6, 2);
		tmpint = Convert.ToInt32("0x" + tmp.Substring(8, 2),16);
		InfoReader[2] = "Reader Type:" + tmpint.ToString() + " / " + GetReaderTyp(tmpint);
		//5D / 93
		InfoReader[3] = "Transponder Types:" + tmp.Substring(10, 4);
        IsUHF = ((InfoReader[2].IndexOf("MU02") >= 0) | (InfoReader[2].IndexOf("MU95") >= 0));
        IsHF = (InfoReader[2].IndexOf("M02*") >= 0);
		//0012
		return R_Result == 0;
	}

	private int CheckFeigVersion()
	{
		int functionReturnValue = 0;
		functionReturnValue = 0;
		COMVersion = GetCOMDLLVersion();
		ISCVersion = GetISCDLLVersion();
		ClearByteArray(ref Read_Array);
		functionReturnValue = 0;
        ReaderInfo = ByteArrayToASCII(Read_Array);
		return functionReturnValue;
	}


	public string GetCOMDLLVersion()
	{
		string functionReturnValue = "";
		functionReturnValue = "";
        SetReaderText("GetCOMDLLVersion");
		FECOM_GetDLLVersion(Read_Array);
        functionReturnValue = ByteArrayToASCII(Read_Array);
		return functionReturnValue;
	}

	public string GetISCDLLVersion()
	{
		string functionReturnValue = "";
		functionReturnValue = "";
        SetReaderText("GetISCDLLVersion");
		FEISC_GetDLLVersion(Read_Array);
        functionReturnValue = ByteArrayToASCII(Read_Array);
		return functionReturnValue;
	}


    public bool WriteEPC_UHF(byte BNr, byte Blaenge, string Data, int Index)
    {   bool Result = false;
		string tmpData = string.Empty;
        byte [] datablock;

        //11/13/12 11:10:40.948  >>  02 00 1C FF B0 24 30 01 00 01 07 02 30 00 44 44 44 44 44 44 44 44 44 44 44 44 EA 14 
        //11/13/12 11:10:41.308  <<  02 00 08 00 B0 00 90 DF   OK

        //11/13/12 11:15:05.815  >>  02 00 1C FF B0 24 30 01 00 01 07 02 30 00 33 33 33 33 33 33 33 33 33 33 33 33 82 68 
        //11/13/12 11:15:06.065  <<  02 00 08 00 B0 00 90 DF   OK


        //11/13/12 11:27:29.795  >>  02 00 1A FF B0 24 30 01 00 02 06 02 33 33 33 33 33 33 33 33 33 33 33 33 B2 96 
        //11/13/12 11:27:30.115  <<  02 00 08 00 B0 00 90 DF   OK


        //11/13/12 11:29:54.496  >>  02 00 27 FF B0 24 31 0C 33 33 33 33 33 33 33 33 33 33 33 33 01 00 02 06 02 33 33 33 33 33 33 33 33 33 33 33 33 B7 2B 
        //11/13/12 11:29:54.826  <<  02 00 08 00 B0 00 90 DF   OK


		int erg = 0;
		int i = 0;
		//SetOn()
		if (UIDNr.Length != 24)
			return false;
        erg = 20 + (Blaenge * 2);
        datablock = new byte[Blaenge * 2];

		Write_Array = new byte[erg];
        Write_Array[0] = 36; // Commando..0x24
        Write_Array[1] = 49; // Mode..0x31
        Write_Array[2] = 12; // length byte..0x0C
		Write_Array[3] = Convert.ToByte("0x" + UIDNr[0] + UIDNr[1],16);
        Write_Array[4] = Convert.ToByte("0x" + UIDNr[2] + UIDNr[3],16);
        Write_Array[5] = Convert.ToByte("0x" + UIDNr[4] + UIDNr[5],16);
        Write_Array[6] = Convert.ToByte("0x" + UIDNr[6] + UIDNr[7],16);
        Write_Array[7] = Convert.ToByte("0x" + UIDNr[8] + UIDNr[9],16);
        Write_Array[8] = Convert.ToByte("0x" + UIDNr[10] + UIDNr[11],16);
        Write_Array[9] = Convert.ToByte("0x" + UIDNr[12] + UIDNr[13],16);
        Write_Array[10] = Convert.ToByte("0x" + UIDNr[14] + UIDNr[15],16);
        Write_Array[11] = Convert.ToByte("0x" + UIDNr[16] + UIDNr[17],16);
        Write_Array[12] = Convert.ToByte("0x" + UIDNr[18] + UIDNr[19],16);
        Write_Array[13] = Convert.ToByte("0x" + UIDNr[20] + UIDNr[21],16);
        Write_Array[14] = Convert.ToByte("0x" + UIDNr[22] + UIDNr[23],16);
        Write_Array[15] = 1;  // b01: epc memory
        Write_Array[16] = 0;  // Access password 0: disabled
        Write_Array[17] = BNr;  // Address..2
        Write_Array[18] = Blaenge; // Number of Block...6
        Write_Array[19] = 2;    //   Block Size

        for (i = Data.Length; i <= Blaenge * 2; i++) { Data += "00"; } // up data with 00 to finish
        datablock = HexToByteArray(Data);
        for (i = 0; i <= datablock.Length - 1; i++) {Write_Array[20 + i] = datablock[i];} 
    	R_Result = FEISC_0xB0_ISOCmd(iReaderHnd, cBusAdr, Write_Array, Write_Array.Length, cRspData, ref iRspLen, iDataType);
        GetErrorText(R_Result);
		Result = R_Result == 0;
        return Result;
    }

    public bool WriteTagBlock(byte BNr, byte Blaenge, string Data, int Index)  // not final..check with HF-Reader
    {
        bool Result = false;
        byte[] datablock;
        int erg = 0;
        int i = 0;
        erg = 13 + (Blaenge * 4);
        datablock = new byte[Blaenge * 4];
        Write_Array = new byte[erg];

        Write_Array[0] = 36;
        Write_Array[1] = 1;
        Write_Array[2] = Convert.ToByte("0x" + UIDNr[0] + UIDNr[1], 16);
        Write_Array[3] = Convert.ToByte("0x" + UIDNr[2] + UIDNr[3], 16);
        Write_Array[4] = Convert.ToByte("0x" + UIDNr[4] + UIDNr[5], 16);
        Write_Array[5] = Convert.ToByte("0x" + UIDNr[6] + UIDNr[7], 16);
        Write_Array[6] = Convert.ToByte("0x" + UIDNr[8] + UIDNr[9], 16);
        Write_Array[7] = Convert.ToByte("0x" + UIDNr[10] + UIDNr[11], 16);
        Write_Array[8] = Convert.ToByte("0x" + UIDNr[12] + UIDNr[13], 16);
        Write_Array[9] = Convert.ToByte("0x" + UIDNr[14] + UIDNr[15], 16);
        Write_Array[10] = BNr;
        Write_Array[11] = Blaenge;
        Write_Array[12] = 4;
        for (i = Data.Length; i <= Blaenge * 4; i++) { Data += "00"; } // up data with 00 to finish
        datablock = HexToByteArray(Data);
        for (i = 0; i <= datablock.Length - 1; i++) { Write_Array[13 + i] = datablock[i]; }
        R_Result = FEISC_0xB0_ISOCmd(iReaderHnd, cBusAdr, Write_Array, Write_Array.Length, cRspData, ref iRspLen, iDataType);
        GetErrorText(R_Result);
        Result = R_Result == 0;
        return Result;
    }

	public bool WriteTagBlock24_UHF(byte BNr, byte Blaenge, string Data, int Index)
	{

        // Adress 3 No of Block 4 Block Size 2 UID (333333333333333333333333)

        //11/12/12 16:35:29.042  >>  02 00 23 FF B0 24 31 0C (33 33 33 33 33 33 33 33 33 33 33 33) 03 00 03 04 02 37 37 37 37 37 37 37 37 A4 A1 
        //11/12/12 16:35:29.158  <<  02 00 08 00 B0 00 90 DF   OK

        //(03 00 ....03 04 02...)...
        //37 37 37 37 37 37 37 37...data 
		bool Result = false;
        byte [] datablock;
		int erg = 0;
		int i = 0;
        if (UIDNr.Length != 24)
        {  // check for UID length
            SetReaderText("Error:UID length");
            return false;
        }
        if (Data.Length != (Blaenge * 2) * 2)
        {
            SetReaderText("Error:data length");
            return false;
        }

        erg = 20 + (Blaenge * 2);
        datablock = new byte[Blaenge * 2];

		Write_Array = new byte[erg];
        Write_Array[0] = 36; // Commando..0x24
        Write_Array[1] = 49; // Mode..0x31
        Write_Array[2] = 12; // length byte..0x0C
		Write_Array[3] = Convert.ToByte("0x" + UIDNr[0] + UIDNr[1],16);
        Write_Array[4] = Convert.ToByte("0x" + UIDNr[2] + UIDNr[3],16);
        Write_Array[5] = Convert.ToByte("0x" + UIDNr[4] + UIDNr[5],16);
        Write_Array[6] = Convert.ToByte("0x" + UIDNr[6] + UIDNr[7],16);
        Write_Array[7] = Convert.ToByte("0x" + UIDNr[8] + UIDNr[9],16);
        Write_Array[8] = Convert.ToByte("0x" + UIDNr[10] + UIDNr[11],16);
        Write_Array[9] = Convert.ToByte("0x" + UIDNr[12] + UIDNr[13],16);
        Write_Array[10] = Convert.ToByte("0x" + UIDNr[14] + UIDNr[15],16);
        Write_Array[11] = Convert.ToByte("0x" + UIDNr[16] + UIDNr[17],16);
        Write_Array[12] = Convert.ToByte("0x" + UIDNr[18] + UIDNr[19],16);
        Write_Array[13] = Convert.ToByte("0x" + UIDNr[20] + UIDNr[21],16);
        Write_Array[14] = Convert.ToByte("0x" + UIDNr[22] + UIDNr[23],16);
		Write_Array[15] = 3;  // b11: user memory
		Write_Array[16] = 0;  // Access password 0: disabled
		Write_Array[17] = BNr;  // Address
		Write_Array[18] = Blaenge; // Number of Block
		Write_Array[19] = 2;    //   Block Size

        for (i = Data.Length; i <= Blaenge * 2; i++) { Data += "00"; } // up data with 00 to finish
        datablock = HexToByteArray(Data);
        for (i = 0; i <= datablock.Length - 1; i++) {Write_Array[20 + i] = datablock[i];} 
    	R_Result = FEISC_0xB0_ISOCmd(iReaderHnd, cBusAdr, Write_Array, Write_Array.Length, cRspData, ref iRspLen, iDataType);
        if (R_Result == 0)
        {
            GetErrorText(R_Result);
        }
        else
        {

        }
		Result = R_Result == 0;
        GetErrorText(R_Result);
		return Result;
	}

	public bool ReadTagBlock24_UHF(byte BNr, byte BLaenge, int Index) // UHF Mode 
	{
		bool Result = false;
		int i = 0;
		int BStart = 3;
		int BStop = 4;

		//05000000000000000001E0C9
		//B0 23 11   0C (längenflag)   05 00 00 00 00 00 00 00 00 01 E0 C9     02 06 Adress+Blocklänge
		//09/07/09 15:49:39.500  >>  02 00 18 FF B0 23 11 0C 05 00 00 00 00 00 00 00 00 01 E0 C9 02 06 3C E4
		//09/07/09 15:49:39.593  <<  02 00 1C 00 B0 00 06 02 00 05 00 00 00 00 00 00 00 00 00 00 00 00 01 00 E0 C9 0B 8C   OK
		//09/08/09 11:22:02.000  >>  02 00 1A FF B0 23 31 0C 05 00 00 00 00 00 00 00 00 01 E0 C9  03  00  08 05  83 52
		//09/08/09 11:22:02.109  <<  02 00 19 00 B0 00 05 02 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 6C A3   OK


        // Adress 3 No of Block 4 Block Size 2 UID //  333333333333333333333333
        //11/12/12 16:11:12.466  >>  02 00 1A FF B0 23 31 0C 33 33 33 33 33 33 33 33 33 33 33 33 00 00 03 04 E0 6E   Bank - b00: reserved
        //11/12/12 16:31:59.067  <<  02 00 09 00 B0 95 03 6C CA   Reader: ISO-Error
        

        //11/12/12 16:11:17.442  >>  02 00 1A FF B0 23 31 0C 33 33 33 33 33 33 33 33 33 33 33 33 01 00 03 04 5B 72  Bank - b01: EPC memory
        //11/12/12 16:32:09.926  <<  02 00 16 00 B0 00 04 02 00 33 33 00 33 33 00 33 33 00 33 33 2C 28   OK
        

        //11/12/12 16:11:21.843  >>  02 00 1A FF B0 23 31 0C 33 33 33 33 33 33 33 33 33 33 33 33 02 00 03 04 96 57 Bank - b10: TID memory
        //11/12/12 16:32:25.149  <<  02 00 16 00 B0 00 04 02 00 F1 00 00 0A A8 00 27 33 00 14 01 B9 37   OK
        

        //11/12/12 16:11:25.179  >>  02 00 1A FF (B0)..Function (23)..commando (31)..Mode (0C)..length-byte(33 33 33 33 33 33 33 33 33 33 33 33)..UID, (03)..bank 00 (03)..Adress (04)..Number of Block 2D 4B Bank - b11: User Memory
        //11/12/12 16:32:36.205  <<  02 00 16 00 B0 00 04 02 00 (37 37 00 37 37 00 37 37 00 37 37) F5 C1   OK
        //(37 37 00 37 37 00 37 37 00 37 37) At adress 3....4 blocks * 2 Block Size = 8 Bytes = 16 Hex.... 
   
    	BlockDataHex = "";
		cRspData = new byte[512];
		R_Result = -1;
        if (UIDNr.Length != 24)
        {
            SetReaderText("Error:UID length");
            return false;
        }
		UID_Array24[0] = 35; // Commando..0x23
		UID_Array24[1] = 49; // Mode..0x31
		UID_Array24[2] = 12; // length byte..0x0C

		UID_Array24[3] = Convert.ToByte("0x" + UIDNr[0] + UIDNr[1],16);
        UID_Array24[4] = Convert.ToByte("0x" + UIDNr[2] + UIDNr[3],16);
        UID_Array24[5] = Convert.ToByte("0x" + UIDNr[4] + UIDNr[5],16);
        UID_Array24[6] = Convert.ToByte("0x" + UIDNr[6] + UIDNr[7],16);
        UID_Array24[7] = Convert.ToByte("0x" + UIDNr[8] + UIDNr[9],16);
        UID_Array24[8] = Convert.ToByte("0x" + UIDNr[10] + UIDNr[11],16);
        UID_Array24[9] = Convert.ToByte("0x" + UIDNr[12] + UIDNr[13],16);
        UID_Array24[10] = Convert.ToByte("0x" + UIDNr[14] + UIDNr[15],16);
        UID_Array24[11] = Convert.ToByte("0x" + UIDNr[16] + UIDNr[17],16);
        UID_Array24[12] = Convert.ToByte("0x" + UIDNr[18] + UIDNr[19],16);
        UID_Array24[13] = Convert.ToByte("0x" + UIDNr[20] + UIDNr[21],16);
        UID_Array24[14] = Convert.ToByte("0x" + UIDNr[22] + UIDNr[23],16);
        UID_Array24[15] = 3;  // Bank - b11: User Memory 
		UID_Array24[16] = 0;  // Access Password..0 = disabled 
		UID_Array24[17] = BNr; // Address-Block Nr.
		//    UIDBlock24[11] :=   '02; 2 Blöcke, 08 = 8 Blöcke
		UID_Array24[18] = BLaenge; // Length...No of Block .......always 2 blocks are read..standart..
		iRspLen = 0;
        R_Result = FEISC_0xB0_ISOCmd(iReaderHnd, cBusAdr, UID_Array24, UID_Array24.Length, cRspData, ref iRspLen, iDataType);
        if (R_Result == 0)
        {
            if (iRspLen > 0)
            {
                  for (i = 1; i <= cRspData[0]; i++)
                  {
                      if (Index == 0) BlockDataHex = BlockDataHex + GetByteDataBufferToHex(cRspData, BStart, BStop);
                      if (Index == 1) BlockDataHex = GetByteDataBufferToHex(cRspData, BStart, BStop) + BlockDataHex;
                      BStart = BStart + 3;
                      BStop = BStop + 3;
                  }
                  BlockDataAScII = HexToASCII(BlockDataHex);
            }
        }
        GetErrorText(R_Result);
		Result = R_Result == 0;
		return Result;
	}

	public bool ReadMifare(byte BNr, byte BLaenge, int Index)
	{
		bool Result = false;
		int i = 0;
		int BStart = 3;
		int BStop = 18;
		BlockDataHex = "";
		cRspData = new byte[512];
		R_Result = -1;

		//09 FF B0 23 02 00 04 1A C8
		ReadMi_Array[0] = 35;
		// hex 23 = DataBlock   ' hex 0A = ConfigBlock (geht nicht)
		ReadMi_Array[1] = 2;
		// Mode Addressed selected, ist immer 2
		ReadMi_Array[2] = BNr;
		ReadMi_Array[3] = BLaenge;
		iRspLen = cRspData.Length;
		iRspLen = 0;
        R_Result = FEISC_0xB0_ISOCmd(iReaderHnd, cBusAdr, ReadMi_Array, ReadMi_Array.Length, cRspData, ref iRspLen, iDataType);
		GetErrorText(R_Result);
		if (R_Result == 0) {
			if (iRspLen > 0) {
				if (Index == 0 | Index == 1) {
					for (i = 1; i <= cRspData[0]; i++) {
						if (Index == 0)
							BlockDataHex = BlockDataHex + GetByteDataBufferToHexInverted(cRspData, BStart, BStop);
						if (Index == 1)
							BlockDataHex = GetByteDataBufferToHexInverted(cRspData, BStart, BStop) + BlockDataHex;
						BStart = BStart + 5;
						BStop = BStop + 5;
					}
					BlockDataAScII = HexToASCII(BlockDataHex);
				}
			}
		}
		Result = R_Result == 0;
		return Result;
	}


	public bool WriteMifare(byte BNr, byte BLaenge, string Data, int Index)
	{
		bool Result = false;
		int i = 0;
		int erg = 0;
		BlockDataHex = "";
		cRspData = new byte[512];
		R_Result = -1;

		//1A FF B0 24 02  01 01  10  00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 69 27 
		//2A FF B0 24 02  02 02  10  00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 14 77 

		//09 FF B0 23 02 00 04 1A C8
		erg = 5 + (BLaenge * 16) - 1;
		WriteMi_Array = new byte[erg + 1];


		WriteMi_Array[0] = 36;
		// hex 24 = DataBlock 
		WriteMi_Array[1] = 2;
		// Mode Addressed selected, ist immer 2
		WriteMi_Array[2] = BNr;
		WriteMi_Array[3] = BLaenge;
		WriteMi_Array[4] = 16;
		//Block-Size, ist fix



		for (i = Data.Length; i <= BLaenge * 16 - 1; i++) {
			Data += " ";
		}

		Data = GetInverted(Data);

		for (i = 5; i <= erg; i++) {
			WriteMi_Array[i] = (byte)Convert.ToChar(Data[i - 5]);
		}

		iRspLen = cRspData.Length;
		iRspLen = 0;
        R_Result = FEISC_0xB0_ISOCmd(iReaderHnd, cBusAdr, WriteMi_Array, WriteMi_Array.Length, cRspData, ref iRspLen, iDataType);
		GetErrorText(R_Result);

		Result = R_Result == 0;
		return Result;
	}



    public bool ReadTagBlock(byte BNr, byte BLaenge, int Index) // not final..check with HF-Reader
	{
		bool Result = false;
		int i = 0;
		int BStart = 3;
		int BStop = 6;
		BlockDataHex = "";
		cRspData = new byte[512];
		R_Result = -1;
		if (UIDNr.Length != 16)
			return false ;
		UID_Array[0] = 35;
		UID_Array[1]= 1;
		UID_Array[2] = Convert.ToByte("0x" + UIDNr[0] + UIDNr[1],16);
        UID_Array[3] = Convert.ToByte("0x" + UIDNr[2] + UIDNr[3],16);
        UID_Array[4] = Convert.ToByte("0x" + UIDNr[4] + UIDNr[5],16);
        UID_Array[5] = Convert.ToByte("0x" + UIDNr[6] + UIDNr[7],16);
        UID_Array[6] = Convert.ToByte("0x" + UIDNr[8] + UIDNr[9],16);
        UID_Array[7] = Convert.ToByte("0x" + UIDNr[10] + UIDNr[11],16);
        UID_Array[8] = Convert.ToByte("0x" + UIDNr[12] + UIDNr[13],16);
        UID_Array[9] = Convert.ToByte("0x" + UIDNr[14] + UIDNr[15],16);

		UID_Array[10] = BNr;
		//    UIDBlock[11] :=   '02; 2 Blöcke, 08 = 8 Blöcke
		UID_Array[11] = BLaenge;
		iRspLen = 0;
        R_Result = FEISC_0xB0_ISOCmd(iReaderHnd, cBusAdr, UID_Array, UID_Array.Length, cRspData, ref iRspLen, iDataType);
        if (R_Result == 0)
        {
            if (iRspLen > 0)
            {
                 for (i = 1; i <= cRspData[0]; i++)
                 {
                     if (Index == 0)
                         BlockDataHex = BlockDataHex + GetByteDataBufferToHex(cRspData, BStart, BStop);
                     if (Index == 1)
                         BlockDataHex = GetByteDataBufferToHex(cRspData, BStart, BStop) + BlockDataHex;
                     BStart = BStart + 5;
                     BStop = BStop + 5;
                    }
                 BlockDataAScII = HexToASCII(BlockDataHex);
            }
        }
        GetErrorText(R_Result); 
        Result = R_Result == 0;
		return Result;
	}

	public bool SelectMifare()
	{
		bool functionReturnValue = false;
		string tmpE = "";
		int i = 0;
		functionReturnValue = false;
		//R_Result = FEISC_0x69_RFReset(iReaderHnd, cBusAdr)

		//cReqData(0) = 1  '01 00
		//cReqData(1) = 0
		//iReqLen = cReqData.Length
		//iRspLen = cRspData.Length
		tmpE = UIDNr;
		for (i = UIDNr.Length; i <= 16 - 1; i++) {
			tmpE = "0" + tmpE;
		}

		SelMifare_Array[0] = 37;
		SelMifare_Array[1] = 1;

		SelMifare_Array[2] = Convert.ToByte("0x" + tmpE[0] + tmpE[1],16);
        SelMifare_Array[3] = Convert.ToByte("0x" + tmpE[2] + tmpE[3],16);
        SelMifare_Array[4] = Convert.ToByte("0x" + tmpE[4] + tmpE[5],16);
        SelMifare_Array[5] = Convert.ToByte("0x" + tmpE[6] + tmpE[7],16);
        SelMifare_Array[6] = Convert.ToByte("0x" + tmpE[8] + tmpE[9],16);
        SelMifare_Array[7] = Convert.ToByte("0x" + tmpE[10] + tmpE[11],16);
        SelMifare_Array[8] = Convert.ToByte("0x" + tmpE[12] + tmpE[13],16);
        SelMifare_Array[9] = Convert.ToByte("0x" + tmpE[14] + tmpE[15],16);

		iRspLen = cRspData.Length;

		// B0 25 01 00 00 00 00 A6 8A B4 B6 26 2D 
		// iReaderHnd As Integer, ByVal cBusAdr As Byte, ByRef cReqData As Byte, ByVal iReqLen As Integer, ByRef cRspData As Byte, ByRef iRspLen As Integer, ByVal iDataFormat As Integer) As Integer
        R_Result = FEISC_0xB0_ISOCmd(iReaderHnd, cBusAdr, SelMifare_Array, SelMifare_Array.Length, cRspData, ref iRspLen, iDataType);
		GetErrorText(R_Result);
		functionReturnValue = R_Result == 0;
		return functionReturnValue;
	}

	public bool SetKeyMifare(byte Block)
	{
		bool functionReturnValue = false;
		functionReturnValue = false;
		//R_Result = FEISC_0x69_RFReset(iReaderHnd, cBusAdr)

		//cReqData(0) = 1  '01 00
		//cReqData(1) = 0
		//iReqLen = cReqData.Length
		//iRspLen = cRspData.Length
		Key_Array[0] = 176;
		Key_Array[1] = 10;

		Key_Array[2] = Block;
		// DB Adresse
		Key_Array[3] = 0;
		// Key A = 0, Key B = 1

		if (MifareKey.Length == 12) {
			Key_Array[4] = Convert.ToByte("0x" + MifareKey[0] + MifareKey[1],16);
            Key_Array[5] = Convert.ToByte("0x" + MifareKey[0] + MifareKey[1],16);
            Key_Array[6] = Convert.ToByte("0x" + MifareKey[0] + MifareKey[1],16);
            Key_Array[7] = Convert.ToByte("0x" + MifareKey[0] + MifareKey[1],16);
            Key_Array[8] = Convert.ToByte("0x" + MifareKey[0] + MifareKey[1],16);
            Key_Array[9] = Convert.ToByte("0x" + MifareKey[0] + MifareKey[1],16);
		} else {
			Key_Array[4] = 0;
			Key_Array[5] = 0;
			Key_Array[6] = 0;
			Key_Array[7] = 0;
			Key_Array[8] = 0;
			Key_Array[9] = 0;
		}

		iReqLen = Key_Array.Length;
		iRspLen = cRspData.Length;


		//FEISC_0xB2_ISOCmd(ByVal
		// B0 0A 00 00 FF FF FF FF FF FF 
		// iReaderHnd As Integer, ByVal cBusAdr As Byte, ByRef cReqData As Byte, ByVal iReqLen As Integer, ByRef cRspData As Byte, ByRef iRspLen As Integer, ByVal iDataFormat As Integer) As Integer
		//00 00 FF FF FF FF FF FF 
        R_Result = FEISC_0xB2_ISOCmd(iReaderHnd, cBusAdr, Key_Array, Key_Array.Length, cRspData, ref iRspLen, iDataType);
		GetErrorText(R_Result);
		functionReturnValue = R_Result == 0;
		return functionReturnValue;
	}



    /*public bool GetEPC()
    {
        bool Result = false;
        R_Result = FEISC_0x69_RFReset(iReaderHnd, cBusAdr);
        EPC_UHFNr = "";

    }*/

    public bool GetUID_Repeat()
    {
        bool functionReturnValue = false;
        RepeatTimer.Enabled = true; 
        do
        {
            System.Windows.Forms.Application.DoEvents();
            functionReturnValue = GetUID();
            if (functionReturnValue) { RepeatTimer.Enabled = false; }
        } while (RepeatTimer.Enabled);
        return functionReturnValue;

    }

    public bool GetUID()
	{
		bool functionReturnValue = false;
        string UIDHexNr = "";
		R_Result = FEISC_0x69_RFReset(iReaderHnd, cBusAdr);
		UIDNr = "";
		cReqData[0] = 1;
		//01 00
		cReqData[1] = 0;
		iReqLen = cReqData.Length;
		iRspLen = cRspData.Length;
        R_Result = FEISC_0xB0_ISOCmd(iReaderHnd, cBusAdr, cReqData, cReqData.Length, cRspData, ref iRspLen, iDataType);
		GetErrorText(R_Result);
		if (R_Result == 0) {
			if (iRspLen > 0) {
				UIDHexNr = GetByteBufferToHex(cRspData, iRspLen);
				UIDNr = CheckTAGData(UIDHexNr);
			}
		}
		functionReturnValue = R_Result == 0;
		return functionReturnValue;
	}

	private string CheckTAGData(string Data)
	{
    	int Tags = 0;
        string tmp = "";
		string uTagNr = "";
		TagTyp = "";
        if (Data.Length < 22)
            return uTagNr;
		Tags = Convert.ToInt32(Data.Substring(0, 2));
		Data = Data.Substring(2, Data.Length - 2);
		if (Tags == 1) {
			tmp = Data.Substring(0, 2);
			TagTyp = GetTransponderTyp(Convert.ToInt32(tmp));
            // E00401000F199C69..HF
			if (tmp == "03") {
                uTagNr = Data.Substring(4, Data.Length - 4);
			} else 
            if (tmp == "00") {
                uTagNr = Data.Substring(4, Data.Length - 4); 
            }else                   
            {
				uTagNr = Data.Substring(10, Data.Length - 10);
			}
		}
		return uTagNr;
	}

    public int SetRF_ON_OFF(byte index)
    {
        R_Result = -1;
        R_Result = FEISC_0x6A_RFOnOff(iReaderHnd, 255, index);
        GetErrorText(R_Result);
        return R_Result;
    }

    public int SetResetConfig(byte ConfAdr)
    {
        ConfAdr = Convert.ToByte(0xC0);
        R_Result = -1;
        R_Result = FEISC_0x83_ResetConfBlock(iReaderHnd, 255, ConfAdr);
        GetErrorText(R_Result);
        return R_Result;
    }


    public int SetConfig_RFPower()
    {
        const int Read_Write_RF_Power = 0xA4;
        R_Result = -1;
        ClearByteArray(ref Config_Array);
        R_Result = FEISC_0x80_ReadConfBlock(iReaderHnd, 255, Read_Write_RF_Power, Config_Array, iDataType);
        if (R_Result == 0){
            Config_Array[2] = Convert.ToByte(RFPower);
            R_Result = FEISC_0x81_WriteConfBlock(iReaderHnd, 255, Read_Write_RF_Power, Config_Array, iDataType);
        }
        GetErrorText(R_Result);
        return R_Result;
    }

    public int GetConfig_RFPower() 
    {
        const int Read_Write_RF_Power = 0xA4;
        R_Result = -1;
        RFPower = -1;
        ClearByteArray(ref Config_Array);
        R_Result = FEISC_0x80_ReadConfBlock(iReaderHnd, 255, Read_Write_RF_Power, Config_Array, iDataType);
        RFPower = Config_Array[2];
        GetErrorText(R_Result);
        return R_Result;
    }


    public int GetISOMode()
    {
        const int ISO_Check = 0x81;
        R_Result = -1;
        ISOMode = -1;
        ClearByteArray(ref Config_Array);
        R_Result = FEISC_0x80_ReadConfBlock(iReaderHnd, 255, ISO_Check, Config_Array, iDataType);
        GetErrorText(R_Result);
        if (R_Result == 0)
        {
            ISOMode = Config_Array[13]; 
        }
        return R_Result;
    }




	public int SetISOMode(byte index)
	{
        const int ISO_Check = 0x81;
        ClearByteArray(ref Config_Array);
        R_Result = FEISC_0x80_ReadConfBlock(iReaderHnd, 255, ISO_Check, Config_Array, iDataType);
        if (R_Result == 0)
        {            //Config_Array[0] = 0;
            //Config_Array[1] = 0;
            //Config_Array[2] = 8;
            //Config_Array[3] = 1;
            //Config_Array[4] = 0;
            //Config_Array[5] = 0;
            //Config_Array[6] = 0;
            //Config_Array[7] = 30;
            // 1E
            //Config_Array[8] = 0;
            //Config_Array[9] = 0;
            //Config_Array[10] = 0;
            //Config_Array[11] = 0;
            //Config_Array[12] = 0;
            Config_Array[13] = index;
            //  0 = ISOHost 1 = ISOScan
            R_Result = FEISC_0x81_WriteConfBlock(iReaderHnd, 255, ISO_Check, Config_Array, iDataType);
            GetErrorText(R_Result);
            if (R_Result == 0)
            {
                R_Result = FEISC_0x63_CPUReset(iReaderHnd, 255);
                //R_Result = FEISC_0x82_SaveConfBlock(iReaderHnd, 255, 1);
                GetErrorText(R_Result);
            }
        }
		return R_Result;
	}


	/*public override void New()
	{
		Read_Array = new byte[256];
		Write_Array = new byte[256];
		My_ComArray = new byte[4];
		Config_Array = new byte[14];
		cReqData = new byte[2];
		cRspData = new byte[512];
		Info_Array = new byte[512];
		UID_Array = new byte[12];
		UID_Array24 = new byte[19];
		Key_Array = new byte[10];
		SelMifare_Array = new byte[10];
		Value_Array = new byte[17];
		InfoReader = new string[6];
		ReadMi_Array = new byte[4];
		WriteMi_Array = new byte[22];
		Error_Array = new byte[256];
	}*)

	/*protected override void Finalize()
	{
		ReaderUnload();
		base.Finalize();
	}*/


	public string GetTransponderTyp(int iStatus)
	{
		string result = string.Empty;
		// Wurde umgebaut..weil die werte in Hex sind
		int htmp = 0;
		try {
			htmp = Convert.ToInt32("0x" + iStatus.ToString(),16);
		} catch (Exception ex) {
			htmp = -1;
		}
		switch (htmp) {
            // HF transponder types
			case FEDM_ISC_TR_TYPE_ICODE1:
				result = FEDM_ISC_TR_NAME_ICODE1;
				break;
			case FEDM_ISC_TR_TYPE_TAGIT:
				result = FEDM_ISC_TR_NAME_TAGIT;
				break;
			case FEDM_ISC_TR_TYPE_ISO15693:
				result = FEDM_ISC_TR_NAME_ISO15693;
				break;
			case FEDM_ISC_TR_TYPE_ISO14443A:
				result = FEDM_ISC_TR_NAME_ISO14443A;
				break;
			case FEDM_ISC_TR_TYPE_ISO14443B:
				result = FEDM_ISC_TR_NAME_ISO14443B;
				break;
			case FEDM_ISC_TR_TYPE_EPC:
				result = FEDM_ISC_TR_NAME_EPC;
				break;
			case FEDM_ISC_TR_TYPE_ICODE_UID:
				result = FEDM_ISC_TR_NAME_ICODE_UID;
				break;
			case FEDM_ISC_TR_TYPE_JEWEL:
				result = FEDM_ISC_TR_NAME_JEWEL;
				break;
			case FEDM_ISC_TR_TYPE_0x64:
				result = FEDM_ISC_TR_NAME_0x64;
				break;
            case FEDM_ISC_TR_TYPE_ISO18000_3M3:
                result = "";
                break;
            case FEDM_ISC_TR_TYPE_MCRFxxx:
                result = "";
                break;
            case FEDM_ISC_TR_TYPE_KEYBOARD: // it's not a transponder, but handles so
                result = "";
                break;
            case FEDM_ISC_TR_TYPE_INNOVATRON:
                result = "";
                break;
            case FEDM_ISC_TR_TYPE_ASK_CTx:
                result = "";
                break;
            // UHF transponder types
			case FEDM_ISC_TR_TYPE_ISO18000_6_A:
				result = FEDM_ISC_TR_NAME_ISO18000_6_A;
				break;
			case FEDM_ISC_TR_TYPE_ISO18000_6_B:
				result = FEDM_ISC_TR_NAME_ISO18000_6_B;
				break;
			case FEDM_ISC_TR_TYPE_EM4222:
				result = FEDM_ISC_TR_NAME_EM4222;
				break;
			case FEDM_ISC_TR_TYPE_EPC_CLASS1_GEN2:
				result = FEDM_ISC_TR_NAME_EPC_CLASS1_GEN2;
				break;
			case FEDM_ISC_TR_TYPE_EPC_CLASS0:
				result = FEDM_ISC_TR_NAME_EPC_CLASS0;
				break;
			case FEDM_ISC_TR_TYPE_EPC_CLASS1_GEN1:
				result = FEDM_ISC_TR_NAME_EPC_CLASS1_GEN1;
				break;
			case FEDM_ISC_TR_TYPE_STM_SR176:
				result = FEDM_ISC_TR_NAME_STM_SR176;
				break;
			case FEDM_ISC_TR_TYPE_STM_SRIxx:
				result = FEDM_ISC_TR_NAME_STM_SRIxx;
				break;
			default:
				result = "unbekannter Typ";
				break;
        }
		return result;
	}

	public string GetReaderTyp(int istatus)
	{
		string Result = "";
		switch (istatus) {

            case FEDM_RW_TYPE_RW40_XX:
                Result = "FEDM_RW_NAME_RW40_XX";
                break; 
	// HF reader types
            case FEDM_ISC_TYPE_ISCM01 :
                Result = FEDM_ISC_NAME_ISCM01;
                break;
	        case FEDM_ISC_TYPE_ISCM02 :
                Result = FEDM_ISC_NAME_ISCM02;                      
                break;
            case FEDM_ISC_TYPE_ISCMU95:
                Result = FEDM_ISC_NAME_ISCMU95;
                break;
            case FEDM_ISC_TYPE_ISCM02_M8 :
                Result = FEDM_ISC_NAME_ISCM02_M8;
                break;
            case FEDM_ISC_TYPE_ISCLR100 :
                Result = FEDM_ISC_NAME_ISCLR100;
                break;
	        case FEDM_ISC_TYPE_ISCLR200 :
                Result = FEDM_ISC_NAME_ISCLR200;
                break;
            case FEDM_ISC_TYPE_ISCLR2000 :
                Result = FEDM_ISC_NAME_ISCLR2000;
                break;
            case FEDM_ISC_TYPE_ISCLR2500_B :
                Result = FEDM_ISC_NAME_ISCLR2500_B;
                break;
            case FEDM_ISC_TYPE_ISCLR2500_A :
                Result = FEDM_ISC_NAME_ISCLR2500_A;
                break;
            case FEDM_ISC_TYPE_ISCLR1002:
                Result = FEDM_ISC_NAME_ISCLR1002;
                break;
            case FEDM_ISC_TYPE_ISCPRH101 :
                Result = FEDM_ISC_NAME_ISCPRH101;
                break;
            case FEDM_ISC_TYPE_ISCPRH101_U :
                Result = FEDM_ISC_NAME_ISCPRH101_U;
                break;
            case FEDM_ISC_TYPE_ISCPRHD102 :
                Result = FEDM_ISC_NAME_ISCPRHD102;
                break;
            case FEDM_ISC_TYPE_ISCPRH102 :
                Result = FEDM_ISC_NAME_ISCPRH102;
                break;
            case FEDM_ISC_TYPE_ISCPRH100_U :
                Result = FEDM_ISC_NAME_ISCPRH100_U;
                break;
            case FEDM_ISC_TYPE_ISCPRH100 :
                Result = FEDM_ISC_NAME_ISCPRH100;
                break;
            case FEDM_ISC_TYPE_ISCMR100_U :
                Result = FEDM_ISC_NAME_ISCMR100_U;
                break;
            case FEDM_ISC_TYPE_ISCMR100 :
                Result = FEDM_ISC_NAME_ISCMR100;
                break;
            case FEDM_ISC_TYPE_ISCMR200 :
                Result = FEDM_ISC_NAME_ISCMR200;
                break;
            case FEDM_ISC_TYPE_ISCMR101 :
                Result = FEDM_ISC_NAME_ISCMR101;
                break;
            case FEDM_ISC_TYPE_ISCMR102 :
                Result = FEDM_ISC_NAME_ISCMR102;
                break;
            case FEDM_ISC_TYPE_ISCMR101_U :
                Result = FEDM_ISC_NAME_ISCMR101_U;
                break;
    // HF classsic-pro reader types
            case FEDM_ISC_TYPE_CPRM02 :
                Result = FEDM_ISC_NAME_CPRM02;
                break;
            case FEDM_ISC_TYPE_CPR02 :
                Result = FEDM_ISC_NAME_CPR02;
                break;
            case FEDM_ISC_TYPE_CPR40_XX_U :
                Result = FEDM_ISC_NAME_CPR40_XX_U;
                break;
            case FEDM_ISC_TYPE_CPR40_XX	:
                Result = FEDM_ISC_NAME_CPR40_XX;
                break;
            case FEDM_ISC_TYPE_CPR50_XX	:
                Result = FEDM_ISC_NAME_CPR50_XX;
                break;
            case FEDM_ISC_TYPE_CPR44_XX	:
                Result = FEDM_ISC_NAME_CPR44_XX;
                break;
            case FEDM_ISC_TYPE_CPR30_XX :
                Result = FEDM_ISC_NAME_CPR44_XX;
                break;
            case FEDM_ISC_TYPE_CPR52_XX	:
                Result = FEDM_ISC_NAME_CPR52_XX;
                break;
            case FEDM_ISC_TYPE_CPR04_U :
                Result = FEDM_ISC_NAME_CPR04_U;
                break;
            case FEDM_ISC_TYPE_CPR46_XX	:
                Result = FEDM_ISC_NAME_CPR46_XX;
                break;
            case FEDM_ISC_TYPE_CPR20_XX	:
                Result = FEDM_ISC_NAME_CPR20_XX;
                break;
    // OBID myAXXESS reader types
            case FEDM_ISC_TYPE_MAX50_XX	:
                Result = FEDM_ISC_NAME_MAX50_XX;
                break;
            case FEDM_ISC_TYPE_MAX_STANDARD	:
                Result = FEDM_ISC_NAME_MAX_STANDARD;
                break;
            case FEDM_ISC_TYPE_MAX_COMFORT :
                Result = FEDM_ISC_NAME_MAX_COMFORT;
                break;
	// UHF reader types
            case FEDM_ISC_TYPE_ISCMU02 :
                Result = FEDM_ISC_NAME_ISCMU02;
                break;
            case FEDM_ISC_TYPE_ISCMRU102 :
                Result = FEDM_ISC_NAME_ISCMRU102;
                break;
            case FEDM_ISC_TYPE_ISCMRU200 :
                Result = FEDM_ISC_NAME_ISCMRU200;
                break;
            case FEDM_ISC_TYPE_ISCLRU1000 :
                Result = FEDM_ISC_NAME_ISCLRU1000;
                break;
            case FEDM_ISC_TYPE_ISCLRU2000 :
                Result = FEDM_ISC_NAME_ISCLRU2000;
                break;
            case FEDM_ISC_TYPE_ISCLRU3000 :
                Result = FEDM_ISC_NAME_ISCLRU3000;
                break;
    // OBID megalock C reader types
            case FEDM_ISC_TYPE_MLC_COM :
                Result = FEDM_ISC_NAME_MLC_COM;
                break;
            case FEDM_ISC_TYPE_MLC_EMPS :
                Result = FEDM_ISC_NAME_MLC_EMPS;
                break;
			default:
				Result = "unbekannter Typ";
				break;
		}
		return Result;
	}

    private void SetReaderText(string Info_Text)
    {
        if (GetReaderText != null) { GetReaderText(Info_Text); }
    }

	public string GetFeigErrorTextUSB(int iStatus)
	{
		string Result = "";
		byte[] MyError = new byte[256];
		//Dim MyError As New ArrayList
		//Dim MyError As New FixedLengthString(512)
        ClearByteArray(ref Error_Array);
		FEUSB_GetErrorText(iStatus, MyError);
        Result = ByteArrayToASCII(MyError);
		Info_Text = iStatus.ToString() + "/" + Result;
        SetReaderText(Info_Text);
		return Result;
	}

	public string GetFeigErrorTextISC(int iStatus)
	{
		string Result = "";
		byte[] MyError = new byte[256];
		//Dim MyError As New ArrayList
		//Dim MyError As New FixedLengthString(512)
        ClearByteArray(ref Error_Array);
		FEISC_GetErrorText(iStatus, MyError);
        Result = ByteArrayToASCII(MyError);
		Info_Text = iStatus.ToString() + "/" + Result;
        SetReaderText(Info_Text);
    	return Result;
	}

    public string GetErrorText_Ex(int istatus)
    {
        string result = string.Empty;

        return result;
    }

	public string GetErrorText(int iStatus)
	{
        LastErrorID = iStatus;
		string Result = "";
		switch (iStatus) {
			case 0x0:
				Result = "OK";
				break;
			//   Statusmeldung vom Reader
			case 0x1:
				Result = "Reader: No Transponder selected";
				break;
			case 0x2:
				Result = "Transponder: CRC Error";
				break;
			case 0x3:
				Result = "Transponder: Write Error";
				break;
			case 0x4:
				Result = "Transponder: Adress Error";
				break;
			case 0x5:
				Result = "Reader: Wrong Transponder Type";
				break;
			case 0x6:
				Result = "Transponder: Read Error";
				break;
			case 0x7:
				Result = "Reader: Select Error";
				break;
			case 0xa:
				Result = "Reader: Transponder Changed";
				break;
			case 0xb:
				Result = "Transponder: Collision";
				break;
			case 0xc:
				Result = "Transponder: Weak Collision";
				break;
			case 0xd:
				Result = "Transponder: Halt Error";
				break;
			case 0xe:
				Result = "Reader: No New Selected Transponders";
				break;
			case 0xf:
				Result = "Reader: Busy";
				break;
			case 0x10:
				Result = "Reader: EEPROM Error";
				break;
			case 0x11:
				Result = "Reader: Parameter Range Error";
				break;
			case 0x80:
				Result = "Reader: Unknown Command";
				break;
			case 0x81:
				Result = "Reader: Protocol Length Error";
				break;
			case 0x82:
				Result = "Reader: Command not available";
				break;
			case 0x83:
				Result = "Reader: RF Communication Error";
				break;
			case 0x90:
				Result = "Reader: Data Buffer Overrun";
				break;
			case 0x92:
				Result = "Reader: No Valid Data";
				break;
			case 0x93:
				Result = "Reader: Data Buffer Overflow";
				break;
			case 0x94:
				Result = "Reader: Poll Buffer Overflow";

				break;
			//   Fehlertexte für FECOM
			case -1000:
				Result = "FECOM: Fehler beim Erstellen eines neuen Port-Objekts";
				break;
			case -1001:
				Result = "FECOM: Port-Handleliste ist leer";
				break;
			case -1002:
				Result = "FECOM: ein Zeiger ist Null, also ungültig";
				break;
			case -1003:
				Result = "FECOM: Speichermangel";
				break;
			case -1004:
				Result = "FECOM: nicht unterstützte Hardware.";
				break;
			case -1005:
				Result = "FECOM: Rückgabe FECOM_DetectPort, Port nicht vorhanden";
				break;
			case -1006:
				Result = "FECOM: nicht unterstützte Funktion oder Option";

				break;
			case -1010:
				Result = "FECOM: Fehler beim Öffnen des COM-Ports";
				break;
			case -1011:
				Result = "FECOM: Timeout beim Öffnen des COM-Ports";
				break;
			case -1012:
				Result = "FECOM: Fehlerhafter Parameter cPortNr in FECOM_OpenPort";
				break;
			case -1013:
				Result = "FECOM: der Port ist bereits geöffnet";

				break;
			case -1020:
				Result = "FECOM: Unbekannter Port-Handle";
				break;
			case -1021:
				Result = "FECOM: Der übergebene Port-Handle ist null";
				break;
			case -1022:
				Result = "FECOM: Der übergebene Port-Handle ist negativ";
				break;
			case -1023:
				Result = "FECOM: Der übergebene Port-Handle ist unbekannt";

				break;
			case -1030:
				Result = "FECOM: Timeout beim Lesen vom Port";
				break;
			case -1031:
				Result = "FECOM: Kein Sendeprotokoll übergeben";
				break;
			case -1032:
				Result = "FECOM: Fehler im Empfangsprozess";
				break;
			case -1033:
				Result = "FECOM: Fehler beim Initialisieren des Ports";
				break;
			case -1034:
				Result = "FECOM: Fehler beim Löschen des Empfangspuffers";
				break;
			case -1035:
				Result = "FECOM: Fehler beim Löschen des Sendepuffers";
				break;
			case -1036:
				Result = "FECOM: Fehler beim Ändern eines Port-Parameters";
				break;
			case -1037:
				Result = "FECOM: Fehler im Sendeprozess";
				break;
			case -1038:
				Result = "FECOM: Checksummen- und/oder Paritätsfehler";
				break;
			case -1039:
				Result = "FECOM_ERR_READ_PORT_PARA";

				break;
			case -1050:
				Result = "FECOM: Übergabeparameter ist nicht bekannt";
				break;
			case -1051:
				Result = "FECOM: Übergabeparameter zu groß oder zu klein";
				break;
			case -1052:
				Result = "FECOM: Ungerader Parameterstring übergeben";
				break;
			case -1053:
				Result = "FECOM: COM-Port-Nummer außerhalb des gültigen Wertebereichs";
				break;
			case -1054:
				Result = "FECOM: unbekannter Fehlercode";

				break;
			case -1070:
				Result = "FECOM: Überlauf im Empfangspuffer";

				break;
			//   Fehlertexte für FEISC
			case -4000:
				Result = "FEISC: Fehler beim Erzeugen eines neuen Leser-Objekts";
				break;
			case -4001:
				Result = "FEISC: Leere Leser-Liste";
				break;
			case -4002:
				Result = "FEISC: Zeiger auf Übergabeparameter ist NULL";
				break;
			case -4003:
				Result = "FEISC: Kein Systemspeicher mehr";
				break;
			case -4004:
				Result = "FEISC: unbekannter COM-Port";
				break;
			case -4005:
				Result = "FEISC: nicht unterstützte Funktion";
				break;
			case -4006:
				Result = "FEISC: keine USB-Unterstützung (z.B. unter NT4)";
				break;
			case -4007:
				Result = "FEISC: alte FECOM.DLL detektiert";
				break;
			// query errors
			case -4010:
				Result = "FEISC: Kein Datenwert vorhanden";

				break;
			case -4020:
				Result = "FEISC: Reader-Handle ist unkekannt";
				break;
			case -4021:
				Result = "FEISC: Der übergebene Leser-Handlle ist null";
				break;
			case -4022:
				Result = "FEISC: Der übergebene Leser-Handle ist negativ";
				break;
			case -4023:
				Result = "FEISC: Reader-Handle in Reader-Handleliste";
				break;
			case -4024:
				Result = "FEISC: Der übergebene Port-Handle ist negativ";
				break;
			case -4025:
				Result = "FEISC: das erste Byte (MSB) im Port-Handle ist ungültig";

				break;
			case -4030:
				Result = "FEISC: Protokoll-Längenfehler";
				break;
			case -4031:
				Result = "FEISC: Protokoll-Checksummenfehler";
				break;
			case -4032:
				Result = "FEISC: Busy-Timeout";
				break;
			case -4050:
				Result = "FEISC: Unbekannter Übergabeparameter";
				break;
			case -4051:
				Result = "FEISC: Übergabeparameter außerhalb des gültigen Wertebereichs";
				break;
			case -4052:
				Result = "FEISC: Ungerader Parameterstring übergeben";
				break;
			case -4053:
				Result = "FEISC: Parameterstring ist zu lang";
				break;

			//Public Const FEISC_ERR_NO_PLUGIN = -4060
			//Public Const FEISC_ERR_PLUGIN_PRESENT = -4061
			//Public Const FEISC_ERR_UNKNOWN_PLUGIN_ID = -4062
			//Public Const FEISC_ERR_PI_BUILD_DATA = -4063
			//Public Const FEISC_ERR_PI_BUILD_FRAME = -4064
			//Public Const FEISC_ERR_PI_SPLIT_FRAME = -4065
			//Public Const FEISC_ERR_PI_SPLIT_DATA = -4066



			//   Fehlertexte für LUCA
			case -10001:
				Result = "COM-Port: unknown usual parameter";
				break;
			case -10002:
				Result = "COM-Port: unknown port handle";
				break;
			case -10003:
				Result = "COM-Port: to many ports open";
				break;
			case -10004:
				Result = "COM-Port: no more memory";

				break;
			case -10010:
				Result = "COM-Port: error in link identifier";
				break;
			case -10011:
				Result = "COM-Port: syntax error in parameter";
				break;
			case -10012:
				Result = "COM-Port: syntx error in value assignment";
				break;
			case -10013:
				Result = "COM-Port: syntx error in adress assignment";
				break;
			case -10014:
				Result = "COM-Port: parameterstring is to long";
				break;
			case -10016:
				Result = "COM-Port: input/output error";
				break;
			case -10017:
				Result = "COM-Port: timeout";
				break;
			case -10018:
				Result = "COM-Port: buffer is to small for next block";
				break;
			case -10020:
				Result = "COM-Port: value is out of range";
				break;
			case -10021:
				Result = "COM-Port: unknown parameter";
				break;
			case -10022:
				Result = "COM-Port: parameter is write protected";

				break;
			case -10099:
				Result = "COM-Port: not implemented";

				break;
			case -20001:
				Result = "COM-Port: abort";
				break;
			case -20002:
				Result = "COM-Port: modul not found";
				break;
			case -20003:
				Result = "COM-Port: unknown driver modul";
				break;
			case -20005:
				Result = "COM-Port: unknown protocol modul";
				break;
			case -20006:
				Result = "COM-Port: modul is still pushed";
				break;
			case -20007:
				Result = "COM-Port: missing adress";
				break;
			case -20008:
				Result = "COM-Port: missing protocol";

				break;
			case -21000:
				Result = "COM-Port: line is not ready";
				break;
			case -21001:
				Result = "COM-Port: timeout while connecting";
				break;
			case -21002:
				Result = "COM-Port: wrong network adress";
				break;
			case -21003:
				Result = "COM-Port: dial locked";
				break;
			case -21004:
				Result = "COM-Port: no dialtone";
				break;
			case -21005:
				Result = "COM-Port: <listen fehlgeschlagen>";
				break;
			case -21006:
				Result = "COM-Port: abnormal disconnect";

				break;
			case -22000:
				Result = "COM-Port: protocol error";

				break;
			case -23001:
				Result = "COM-Port: access denied";
				break;
			case -23002:
				Result = "COM-Port: authorization error";

				break;
			default:
				Result = "unbekannter Fehlercode";
				break;
		}
		Info_Text = iStatus.ToString() + "/" + Result;
        SetReaderText(Info_Text);
		return Result;
	}

    private void AddEvent()
    {
        this.RepeatTimer.Tick += new System.EventHandler(this.RepeatTimer_Tick);
    }

    private void RepeatTimer_Tick(object sender, EventArgs e)
    {
        RepeatTimer.Enabled = false; 
    }


    public void New_Init()
    {
        Read_Array = new byte[256];
        Write_Array = new byte[256];
        My_ComArray = new byte[4];
        Config_Array = new byte[14];
        cReqData = new byte[2];
        cRspData = new byte[512];
        Info_Array = new byte[512];
        UID_Array = new byte[12];
        UID_Array24 = new byte[19];
        Key_Array = new byte[10];
        SelMifare_Array = new byte[10];
        Value_Array = new byte[17];
        InfoReader = new string[6];
        ReadMi_Array = new byte[4];
        WriteMi_Array = new byte[22];
        Error_Array = new byte[256];
        AddEvent();
    }

	#endregion

}

