﻿//	Registry class
//	--
//	Read values from the registry.

using System.Runtime.InteropServices;
using System;
using System.Text;

// Read values from registry.
public class Registry
{
    // base registry keys
    //public sealed class RootKey


    public enum CarOptions
    {
        SunRoof = 0x01,
        Spoiler = 0x02,
        FogLights = 0x04,
        TintedWindows = 0x08,
    }

    enum Days : byte {Sat=1, Sun, Mon, Tue, Wed, Thu, Fri};

    public enum RootKey : uint
    {
        ClassesRoot = 0x80000000,
        CurrentUser = 0x80000001,
        LocalMachine = 0x80000002,
        Users = 0x80000003,
    }

    public class WinApi
    {
        #region " Registry API imports "

        [DllImport("coredll.dll")]
        public static extern int RegOpenKeyEx(uint hKey,string lpSubKey,uint ulOptions, uint samDesired, ref uint phkResult);

        [DllImport("coredll.dll")]
        public static extern int RegCloseKey(uint hKey);

        [DllImport("coredll.dll", SetLastError = true)]
        public static extern int RegQueryValueEx(uint hkey, string lpValueName, uint lpReserved, ref uint lpType, byte[] lpData, ref uint lpcbData);


        #endregion
    }

    // static class
    private Registry()
    {
    }


    // Read string value from registry.
    public static string GetString(RootKey rootKey, string keyName, string valueName)
    {
        // result that is returned
        string result = "";

        try
        {
            // read specified registry location and convert to string
            byte[] data = GetValue(rootKey, keyName, valueName);
            if ((data != null))
            {
                result = UnicodeEncoding.Unicode.GetString(data, 0, data.Length);
            }
        }
        catch
        {
            // return empty string if error occurs
        }

        return result;
    }


    // Read int value from registry.
    public static int GetInt(RootKey rootKey, string keyName, string valueName)
    {
        // result that is returned
        int result = 0;

        try
        {
            // read specified registry location and convert to int
            byte[] data = GetValue(rootKey, keyName, valueName);
            if ((data != null))
            {
                result = System.BitConverter.ToInt32(data, 0);
            }
        }
        catch
        {
            // return 0 if error occurs
        }

        return result;
    }


    // Read specified registry location. Returns data as a byte array
    // and caller methods can convert to different types.
    private static byte[] GetValue(RootKey rootKey, string keyName, string valueName)
    {
        // data that is returned
        //StringBuilder data = new StringBuilder();
        byte[] data = null;

        // handle to reg key
         uint hKey = 0;
        //IntPtr hKey = IntPtr.Zero;  

        try
        {
            // open registry key
            if (WinApi.RegOpenKeyEx((uint)rootKey, keyName, 0, 0, ref hKey) == 0)
            {
                // get the size of the data
                uint dataType = 0;
                uint dataSize = 0;
                WinApi.RegQueryValueEx((uint)hKey, valueName, 0, ref dataType, null, ref dataSize);

                // allocate room for data and read value
                if (dataSize != 0)
                {
                    data = new byte[dataSize];  
                    WinApi.RegQueryValueEx((uint)hKey, valueName, 0, ref dataType, data, ref dataSize);
                }
            }
        }
        finally
        {
            if (hKey != 0)
            {
                WinApi.RegCloseKey(hKey);
            }
        }

        return data;
    }

}
