﻿using System;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Threading;
using System.IO;



public class Sound
{
    private byte[] m_soundBytes;

    private string m_fileName;
    [DllImport("CoreDll.dll", EntryPoint = "PlaySound", CharSet = CharSet.Auto, SetLastError = true)]
    public static extern int WCE_PlaySound(string szSound, IntPtr hMod, int flags);

    [DllImport("CoreDll.dll", EntryPoint = "PlaySound", CharSet = CharSet.Auto, SetLastError = true)]
    public static extern int WCE_PlaySoundBytes(byte[] szSound, IntPtr hMod, int flags);


    //[DllImport("coredll.dll", EntryPoint = "MessageBeep", SetLastError = true)]
    //((private static extern int MessageBeep(int wType);



    private enum Flags
    {
        SND_SYNC = 0x0,
        // play synchronously (default) 
        SND_ASYNC = 0x1,
        // play asynchronously 
        SND_NODEFAULT = 0x2,
        // silence (!default) if sound not found 
        SND_MEMORY = 0x4,
        // pszSound points to a memory file 
        SND_LOOP = 0x8,
        // loop the sound until next sndPlaySound 
        SND_NOSTOP = 0x10,
        // don't stop any currently playing sound 
        SND_NOWAIT = 0x2000,
        // don't wait if the driver is busy 
        SND_ALIAS = 0x10000,
        // name is a registry alias 
        SND_ALIAS_ID = 0x110000,
        // alias is a predefined ID 
        SND_FILENAME = 0x20000,
        // name is file name 
        SND_RESOURCE = 0x40004
        // name is resource name or atom 
    }


    // Construct the Sound object to play sound data from the specified file.
    public Sound(string fileName)
    {
        m_fileName = fileName;
    }


    // Construct the Sound object to play sound data from the specified stream.
    public Sound(Stream stream)
    {
        // read the data from the stream
        m_soundBytes = new byte[stream.Length + 1];
        stream.Read(m_soundBytes, 0, Convert.ToInt32(stream.Length));
    }
    //New


    // Play the sound
    public void Play()
    {
        // If a file name has been registered, call WCE_PlaySound, 
        // otherwise call WCE_PlaySoundBytes.
        if ((m_fileName != null))
        {
            WCE_PlaySound(m_fileName, IntPtr.Zero, Convert.ToInt32(Flags.SND_ASYNC | Flags.SND_FILENAME));
        }
        else
        {
            WCE_PlaySoundBytes(m_soundBytes, IntPtr.Zero, Convert.ToInt32(Flags.SND_ASYNC | Flags.SND_MEMORY));
        }
    }
}
