﻿//*************************************************}
//                                                 }
//       Program: RFID Demo                        }
//       Programmed with Visual Studio 2008        }
//       Tectus Transponder Technology GmbH        }
//       Sascha Haveloh, s.haveloh@tec-tus.de      }
//       V1.7                                      } 
//                                                 }
//*************************************************}
// Tectus Transponder Technology GmbH
// 04.08.2011
// aktuelle Firmware Versionen
// - btrw-hdx.v1.40.frm fuer alles TIRIS, UNIQUE32/64, ZOODIAC
// - btrw-rw.v1.40.frm fuer Q5, HITAG-S(1), HITAG-II, EM4305
// - btrw-ti.v1.40.frm fuer TITAN, UNIQUE32/64, ZOODIAC
// - bt-psk.v1.72  04.08.2011 neue firmwareversion (kontrolle auf bt-psk)
// Only UHF-Reader..15.11.2012... TagType = 3 for UHF
// Version V1.4 LF_UHF 04.12.2012
// HF not work in this projekt....**///
// 22.01.2013...repeat..read write UID...
// Version V1.4.1 
// 24.01.2013...HF Reader 
// 31.01.2013..frmHF...txtData.txt...HEX to ASCCI
// 22.02.2013..check Reader_Typ..
// Comport Check...Port is ascending...GetFTDComport..(check for virtual Comport from FTDI)


using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlServerCe;
using System.Runtime;
using System.Runtime.InteropServices;
using System.Threading;
using System.IO;
using Symbol;
using Symbol.ResourceCoordination;
using BT_LF;

namespace RFID_Demo
{

    public partial class frmMain : Form
    {

        public frmMain()
        {
            InitializeComponent();
        }

        public sealed class ControlChars
        {
            public const char Back = '\b';
            public const char Cr = '\r';
            public const string CrLf = "\r\n";
            public const char FormFeed = '\f';
            public const char Lf = '\n';
            public const string NewLine = "\r\n";
            public const char NullChar = '\0';
            public const char Quote = '"';
            public const char Tab = '\t';
            public const char VerticalTab = '\v';
        }

        const string UpdateSql = "UPDATE [tblDaten] SET [VarField1] = @p1, [VarField2] = @p2, [VarField3] = @p3, [VarField4] = @p4, [VarField5] = @p5 "; //' WHERE ([VarField1] = @p1)";
        const string InsertSql = "INSERT INTO [tblDaten] ([VarField1], [VarField2], [VarField3], [VarField4], [VarField5]) VALUES (@p1, @p2, @p3, @p4, @p5)";

        public System.Data.SqlServerCe.SqlCeConnection conn_bar = null;
        public System.Data.SqlServerCe.SqlCeCommand cmd_bar = null;
        public System.Data.SqlServerCe.SqlCeDataAdapter rs_bar = null;
        public System.Data.DataSet ds_bar = null;
        
        public int Reader_Typ;
        public int HF_Transponder_Typ;

        public DataGridTableStyle myGridStyle;
        public string InBartecDB;
        public bool IsDBActive;
        public bool IsReaderActive;
        public string Root;
        public int LFComport;

        public string DBName = "\\TIDBartec.sdf";
        public int TagType = 0;
        public string TagName = "0 - EM 41xx (UNIQUE)";
        public string RFIDData;
        public frmLF frmLFDlg;
        public frmHF frmHFDlg;
        public frmUHF frmUHFDlg;
        public frmRFIDData frmRFIDDataDlg;
        public string errortext;

        public RFIDFeigUHF RFID_UHF = new RFIDFeigUHF();  // FEIG UHF Class
        public RFIDTectusHF RFID_HF = new RFIDTectusHF();  // TECTUS HF Class 
        public BT_LF.BT_LF MyBTLF = new BT_LF.BT_LF(); // LF DLL
        public TIDSendKey TIDSend = new TIDSendKey();  // SendKey...

        [DllImport("coredll.dll", EntryPoint = "MessageBeep", SetLastError = true)]
        private static extern int MessageBeep(int wType);

        public Symbol.ResourceCoordination.Trigger KeyTrigger = null;

        public void PlaySound()
        {
            Sound sound = new Sound(Root + "\\Laser.wav");
            if (SoundItem.Checked) 
            {
                sound.Play();
            }
            else
            {
                MessageBeep(Convert.ToInt32("0x40",16));
            }
        }

        private void ReadIni()
        {
            int tmpInt = 0;
            Reader_Typ = Ini.Settings.GetInt("Reader_Typ");
            InBartecDB = Ini.Settings.GetString("BartecDB");
            LFComport = Ini.Settings.GetInt("LFComport");
            TagName = Ini.Settings.GetString("TagName");
            TagType = Ini.Settings.GetInt("TagType");
            MyBTLF.TitanPassword = Ini.Settings.GetString("TitanPassword");
            if (string.IsNullOrEmpty(MyBTLF.TitanPassword))
            {
                MyBTLF.TitanPassword = "00000000";
                Ini.Settings.SetValue("TitanPassword", MyBTLF.TitanPassword);
            }

            MyBTLF.Hitag2Password = Ini.Settings.GetString("Hitag2Password");
            if (string.IsNullOrEmpty(MyBTLF.Hitag2Password))
            {
                MyBTLF.Hitag2Password = "4D494B52";
                Ini.Settings.SetValue("Hitag2Password", MyBTLF.Hitag2Password);
            }

            tmpInt = Ini.Settings.GetInt("ReadTimeOut");
            if (tmpInt <= 0)
            {
                tmpInt = 2000;
                MyBTLF.ReadTimeOut = tmpInt;
                Ini.Settings.SetValue("ReadTimeOut", MyBTLF.ReadTimeOut);
            }
            else
            {
                MyBTLF.ReadTimeOut = tmpInt;
            }

            tmpInt = Ini.Settings.GetInt("WriteTimeOut");
            if (tmpInt <= 0)
            {
                tmpInt = 2000;
                MyBTLF.WriteTimeOut = tmpInt;
                Ini.Settings.SetValue("WriteTimeOut", MyBTLF.WriteTimeOut);
            }


            RFID_UHF.MifareKey = Ini.Settings.GetString("MifareKey");
            if (string.IsNullOrEmpty(RFID_UHF.MifareKey))
            {
                RFID_UHF.MifareKey = "000000000000";
                Ini.Settings.SetValue("MifareKey", RFID_UHF.MifareKey);
            }

            HF_Transponder_Typ = Ini.Settings.GetInt("HF_Transponder_Typ");
            if (HF_Transponder_Typ < 0)
            {
                HF_Transponder_Typ = 0;
            }


            if (string.IsNullOrEmpty(TagName))
            {
                TagName = "1 - HITAG S";
                Ini.Settings.SetValue("TagName", TagName);
            }

            if (TagType < 0)
            {
                TagType = 1;
                Ini.Settings.SetValue("TagType", TagType);
            }

            if (LFComport <= 0)
            {
                LFComport = 1;
                Ini.Settings.SetValue("LFComport", LFComport);
            }

            if (string.IsNullOrEmpty(InBartecDB))
            {
                Ini.Settings.SetValue("BartecDB", Root + DBName);
                InBartecDB = Root + DBName;
                Ini.Settings.SetValue("BartecDB", InBartecDB);
            }
            SoundItem.Checked = Ini.Settings.GetBool("Sound");

            RFID_UHF.RW_TimeInterval = Ini.Settings.GetInt("RW_TimeInterval");
            if (RFID_UHF.RW_TimeInterval < 2000) 
            {
                RFID_UHF.RW_TimeInterval = 2000;
                Ini.Settings.SetValue("RW_TimeInterval", RFID_UHF.RW_TimeInterval);
            }


        }

        public void WriteIni()
        {
            Ini.Settings.SetValue("Reader_Typ", Reader_Typ);
            Ini.Settings.SetValue("HF_Transponder_Typ", HF_Transponder_Typ);
            Ini.Settings.SetValue("BartecDB", InBartecDB);
            Ini.Settings.SetValue("LFComport", LFComport);
            Ini.Settings.SetValue("TagName", TagName);
            Ini.Settings.SetValue("TagType", TagType);
            Ini.Settings.SetValue("TitanPassword", MyBTLF.TitanPassword);
            Ini.Settings.SetValue("Hitag2Password", MyBTLF.Hitag2Password);
            Ini.Settings.SetValue("MifareKey", RFID_UHF.MifareKey);
            Ini.Settings.SetValue("Sound", SoundItem.Checked);
            Ini.Settings.SetValue("WriteTimeOut", MyBTLF.WriteTimeOut);
            Ini.Settings.SetValue("ReadTimeOut", MyBTLF.ReadTimeOut);
            Ini.Settings.SetValue("RW_Repeat", RFID_UHF.RW_TimeInterval);
        }
        
        private bool InitDB()
        {
            bool Result = false;
            if (File.Exists(InBartecDB))
            {
                lbDBInfo.Text = "Database: " + DBName;
                Result = true;
            }
            else
            {
                lbDBInfo.Text = "Database not found";
            }
            lbDBInfo.Refresh();
            return Result;
        }

        public bool OpenConnection()
        {
            bool functionReturnValue = false;
            if (File.Exists(InBartecDB))
            {
                conn_bar = new SqlCeConnection();
                conn_bar.ConnectionString = "Data Source = " + InBartecDB + "; Password = 0147";
                conn_bar.Open();
                rs_bar = new SqlCeDataAdapter("", conn_bar);
                cmd_bar = new SqlCeCommand("", conn_bar);
                ds_bar = new DataSet();
                functionReturnValue = true;
            }
            else
            {
                functionReturnValue = false;
                MessageBox.Show("DB NOT FOUND: " + InBartecDB);
            }
            return functionReturnValue;
        }

        public void CloseConnection()
        {
            if ((rs_bar != null))
            {
                rs_bar.Dispose();
                rs_bar = null;
            }

            if ((cmd_bar != null))
            {
                cmd_bar.Dispose();
                cmd_bar = null;
            }

            if ((ds_bar != null))
            {
                ds_bar.Dispose();
                ds_bar = null;
            }

            if ((conn_bar != null))
            {
                if (conn_bar.State == ConnectionState.Open)
                {
                    conn_bar.Close();
                }
                conn_bar = null;
            }
        }






        private void InitTimer_Tick(object sender, EventArgs e)
        {
            InitTimer.Enabled = false;
            try
            {
                CheckReaderOnBoard(); 
            }
                finally
            {
                InitItem.Enabled = true;
            }
           
       }

        private void InitTrigger()
        {
            KeyTrigger = new Symbol.ResourceCoordination.Trigger(new Symbol.ResourceCoordination.TriggerDevice(Symbol.ResourceCoordination.TriggerID.ALL_TRIGGERS, new System.Collections.ArrayList()));
            KeyTrigger.Stage2Notify += new Trigger.TriggerEventHandler(trigger_Stage2Notify);
        }


        private void trigger_Stage2Notify(object sender, Symbol.ResourceCoordination.TriggerEventArgs e)
        {
            // TRIGGER1 // left   yellow scan_button
            // TRIGGER2 // middle yellow scan_button
            // TRIGGER3 // right  yellow scan_button

            // if (e.NewState.ToString() != "STAGE2") { return; } 

            if (e.NewState.ToString() == "STAGE2")
            {
                if (e.TriggerID.ToString() == "TRIGGER1" | e.TriggerID.ToString() == "TRIGGER2" | e.TriggerID.ToString() == "TRIGGER3")
                {

                    if (Reader_Typ == 1)
                    {
                        if (frmLFDlg != null)
                        {
                            if (!frmLFDlg.IsKeyAction)
                            {
                                frmLFDlg.ReadTransponder();
                                return;
                            }
                        }

                    }
                    if (Reader_Typ == 2)
                    {
                        if (frmHFDlg != null)
                        {
                            if (!frmHFDlg.IsKeyAction)
                            {
                                frmHFDlg.ReadTransponder();
                                return;
                            }
                        }

                    }
                    if (Reader_Typ == 3)
                    {
                        if (frmUHFDlg != null)
                        {
                            if (!frmUHFDlg.IsKeyAction)
                            {
                                frmUHFDlg.ReadTransponder();
                                return;
                            }
                        }

                    }
                }
            }
        }


        private void AddEvents()
        {
            RFID_UHF.GetReaderText += new RFIDFeigUHF.GetReaderTextEventHandler(RFID_GetReaderText);
            RFID_HF.HFData +=new RFIDTectusHF.GetHFDataEventHandler(RFID_HF_HFData);
            MyBTLF.TLMData +=new BT_LF.BT_LF.GetTLMDataEventHandler(MyBTLF_TLMData);
        }

        private void ConfigNotification()
        {
            StringBuilder HTMLString = new StringBuilder();
            HTMLString.Append("<font color=\"#0000FF\"><b>RFID Demo V1.6 - automatic</b></font>");
            HTMLString.Append("<br><br>");
            HTMLString.Append("<input type=button name='btShow' value='Show Program'>");
            HTMLString.Append("<input type=button name='btExit' value='Exit Program'>");
            Tray1.Text = HTMLString.ToString();
        }

        private void RFID_GetReaderText(string Info_Text)
        {
            errortext = Info_Text;
        }

        private void RFID_HF_HFData(object Sender, string Direction, string Data)
        {
            if ((frmHFDlg != null))
            {
                if (frmHFDlg.EventItem.Checked)
                {
                    frmHFDlg.InfoAdd(Direction + " " + Data);
                }
            }
        }

        private void MyBTLF_TLMData(object Sender, string Direction, string Data)
        {
            if ((frmLFDlg != null))
            {
                if (frmLFDlg.EventItem.Checked)
                {
                    frmLFDlg.InfoAdd(Direction + " " + Data);
                }
            }
        }


        private void frmMain_Load(object sender, EventArgs e)
        {
            Root = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase);
            //Root = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetModules(0).FullyQualifiedName);
            ReadIni();
            lbDBInfo.Text = "Search database...";
            lbDBInfo.Refresh();
            IsDBActive = InitDB();
            lbReaderInfo.Text = "Search Reader...";
            lbReaderInfo.Refresh();
            InitTrigger();
            AddEvents();
            ConfigNotification(); 
            InitTimer.Enabled = true;
        
        }

        public void SendUIDNr(string TAG)
        {
            if (!this.Visible)
            {
                this.TIDSend.SendCEKeys(TAG, Convert.ToString(ControlChars.Cr), "");
            }
        }

        public Char CheckHexDigits(Char value)
        {
            Char result = Convert.ToChar(value.ToString().ToUpper());
            if ("1234567890ABCDEFGH\b".IndexOf(result) < 0) { result = '\0'; }
            return result;
        }

        private void ExitItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void btReaderLF_Click(object sender, EventArgs e)
        {
            string MyLFVersion = string.Empty;
            if (Reader_Typ == 0)
            {
                //MsgBox("No Rader found" & vbCrLf & _
                //      "Please initialize first a Reader")
                return;
            }
            if (Reader_Typ == 1)
            {
                frmLFDlg = new frmLF(this);
                if (MyBTLF.Active)
                {
                    MyLFVersion = MyBTLF.GetVersion();
                    frmLFDlg.lbReaderInfo.Text = MyLFVersion;
                    frmLFDlg.lbReaderInfo.Refresh(); 
                    frmLFDlg.IsFirst = true;
                    frmLFDlg.TagView.SelectedIndex = frmLFDlg.GetTagTypeIndex(TagType);
                    frmLFDlg.IsFirst = false;
                    frmLFDlg.SetActiveButton();
                }
                frmLFDlg.ShowDialog();
                if ((frmLFDlg != null))
                {
                    frmLFDlg.Dispose();
                    frmLFDlg = null;
                }
            }
            if (Reader_Typ == 2)
            {
                frmHFDlg = new frmHF(this);
                if (RFID_HF.Active)
                {
                    frmHFDlg.lbReaderInfo.Text = RFID_HF.GetVersion();
                    frmHFDlg.TagView.SelectedIndex = 0;
                    frmHFDlg.lbReaderInfo.Refresh();
                }
                frmHFDlg.ShowDialog();
                if ((frmHFDlg != null))
                {
                    frmHFDlg.Dispose();
                    frmHFDlg = null;
                }
            }
            if (Reader_Typ == 3)
            {
                frmUHFDlg = new frmUHF(this);
                if (RFID_UHF.IsActive)
                {
                    if (RFID_UHF.GetReaderInfo(0))
                    {
                        frmUHFDlg.lbReaderInfo.Text = RFID_UHF.InfoReader[2];
                        frmUHFDlg.TagView.SelectedIndex = 0;
                        frmUHFDlg.lbReaderInfo.Refresh();
                    }
                    else
                    {
                        frmUHFDlg.lbReaderInfo.Text = "No Reader";
                        frmUHFDlg.lbReaderInfo.Refresh();
                    }
                }
                frmUHFDlg.ShowDialog();
                if ((frmUHFDlg != null))
                {
                    frmUHFDlg.Dispose();
                    frmUHFDlg = null;
                }
            }

        }

        private void CloseReaderAll()
        {
            try
            {
                MyBTLF.Active = false;
                RFID_UHF.ReaderUnload();
            }
            catch (Exception ex)
            {
            }
        }

        public void CloseReader(int Index)
        {
            try
            {
                if (Index == 0)
                {
                    if (Reader_Typ == 1)
                    {
                        MyBTLF.Active = false;
                    }
                    if (Reader_Typ == 2)
                    {
                        RFID_HF.Active = false; 
                    }
                    if (Reader_Typ == 3)
                    {
                        RFID_UHF.ReaderUnload();
                    }
                }
                if (Index == 1)
                {
                    MyBTLF.Active = false;
                    RFID_UHF.ReaderUnload();
                    RFID_HF.Active = false; 
                }
            }
            catch (Exception ex)
            {
            }
        }

        private void CloseAll()
        {
            try
            {
                Tray1.Visible = false;
                Tray1.Dispose(); 
                CloseReader(1);
                InitTimer.Enabled = false;
                WriteIni();
                CloseConnection();
            }
            catch (Exception ex)
            {
                //MsgBox(ex.Message)
            }
        }

        private void frmMain_Closed(object sender, EventArgs e)
        {
            CloseAll(); 
        }

        private void frmMain_Closing(object sender, CancelEventArgs e)
        {
            e.Cancel = MessageBox.Show("Exit program ?", this.Text, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) == DialogResult.No;     
        }


        private bool CheckReaderHF()
        {
            bool result = false;
            IsReaderActive = false;
            CloseReader(1);
            lbReaderTyp.Text = "HF-Reader";
            lbReaderTyp.Refresh();
            IsReaderActive = InitReaderHF();
            if (IsReaderActive)
            {
                lbReaderInfo.Text = RFID_HF.GetVersion();
                lbReaderInfo.Text = RFID_HF.GetVersion();
                lbReaderInfo.Refresh();
                if (String.IsNullOrEmpty(lbReaderInfo.Text))
                {
                    lbReaderInfo.Text = "No Reader found";
                    lbReaderInfo.Refresh();
                    lbReaderTyp.Text = "Reader-Typ:";
                    lbReaderTyp.Refresh();
                    CloseReader(0); 
                }
                else
                {
                    btReader.Text = "HF Reader Config.";
                    btReader.Refresh();
                    btData.Text = "HF Read/Write Data";
                    btData.Refresh();
                    result = true; 
                }
            }
            else
            {
                lbReaderInfo.Text = "No Reader found";
                lbReaderInfo.Refresh();
                lbReaderTyp.Text = "Reader-Typ:";
                lbReaderTyp.Refresh(); 
            }
            return result;
        }

        private bool CheckReaderLF()
        {
            bool result = false;
            CloseReader(1);
            IsReaderActive = false;
            lbReaderTyp.Text = "LF-Reader";
            lbReaderTyp.Refresh();
            IsReaderActive = InitReaderLF();
            if (IsReaderActive)
            {
                lbReaderInfo.Text = MyBTLF.GetVersion().Trim();
                lbReaderInfo.Refresh();
                if (String.IsNullOrEmpty(lbReaderInfo.Text))
                {
                    lbReaderInfo.Text = "No Reader found";
                    lbReaderInfo.Refresh();
                    lbReaderTyp.Text = "Reader-Typ:";
                    lbReaderTyp.Refresh();
                    CloseReader(0); 
                }
                else
                {
                    btReader.Text = "LF Reader Config.";
                    btReader.Refresh();
                    btData.Text = "LF Read/Write Data";
                    btData.Refresh();
                    result = true;
                }
            }
            else
            {
                lbReaderInfo.Text = "No Reader found";
                lbReaderInfo.Refresh();
                lbReaderTyp.Text = "Reader-Typ:";
                lbReaderTyp.Refresh();
            }
            return result;
        }

        private bool CheckReaderUHF()
        {
            bool result = false;
            CloseReader(1);
            lbReaderTyp.Text = "UHF-Reader";
            lbReaderTyp.Refresh();
            IsReaderActive = InitReaderUHF_USB();
            if (IsReaderActive)
            {
                if (RFID_UHF.GetReaderInfo(0) & RFID_UHF.IsUHF)
                {
                    lbReaderInfo.Text = RFID_UHF.InfoReader[2];
                    lbReaderInfo.Refresh();
                    btReader.Text = "UHF Reader Config.";
                    btReader.Refresh();
                    btData.Text = "UHF Read/Write Data";
                    btData.Refresh();
                    result = true;
                }
                else
                {
                    lbReaderInfo.Text = "No Reader found";
                    lbReaderInfo.Refresh();
                    lbReaderTyp.Text = "Reader-Typ:";
                    lbReaderTyp.Refresh();
                }
            }
            else
            {
                lbReaderInfo.Text = "No Reader found";
                lbReaderInfo.Refresh();
                lbReaderTyp.Text = "Reader-Typ:";
                lbReaderTyp.Refresh();
            }
            return result;
        }

        public void CheckReaderOnBoard()
        {
            lbReaderInfo.Text = "Search Reader...";
            lbReaderInfo.Refresh();
            btReader.Text = "Reader Config.";
            btReader.Refresh();
            btData.Text = "Read/Write Data";
            btData.Refresh(); 
            if (Reader_Typ == 1) // LF Reader
            {
                if (CheckReaderLF()){return; }
                else { Reader_Typ = 0; CheckReaderOnBoard(); } 
            }
            if (Reader_Typ == 2) // HF Reader
            {
                if (CheckReaderHF()) { return; }
                else { Reader_Typ = 0; CheckReaderOnBoard(); } 
            }
            if (Reader_Typ == 3) // UHF Reader
            {
               if (CheckReaderUHF()) { return; }
               else { Reader_Typ = 0; CheckReaderOnBoard(); } 
            }
            if (Reader_Typ == 0)
            {
                LFComport = Convert.ToInt32(GetFTDIComport());  // if Error check for FTDI Ports
                MyBTLF.Comport = "COM" + LFComport.ToString();
                RFID_HF.Comport = "COM" + LFComport.ToString();
                Reader_Typ = 1;
                if (CheckReaderLF()) { return; }
                Reader_Typ = 2;
                if (CheckReaderHF()) { return; }
                Reader_Typ = 3;
                if (CheckReaderUHF()) { return; }
                Reader_Typ = 0;
            }

        }

        private string GetFTDIComport()
        {
            //"Drivers\\USB\\ClientDrivers\\FTDI_DEVICE\0"
            string result = string.Empty;
            string tmpint = string.Empty;
            string tmpName = string.Empty;
            string tmpKey = string.Empty;
            int p1 = -1;
            int p2 = -1;
            for (int i = 0; i <= 255; i++)
            {
                tmpint = i.ToString("D2");
                tmpName = Registry.GetString(Registry.RootKey.LocalMachine, "Drivers\\Active\\" + tmpint, "Name");
                tmpKey = Registry.GetString(Registry.RootKey.LocalMachine, "Drivers\\Active\\" + tmpint, "Key");
                p1 = tmpName.IndexOf("COM");
                p2 = tmpName.IndexOf(":");
                if ((p1 >= 0) | (p2 >= 0))
                {
                    if (tmpKey.IndexOf("FTDI_DEVICE") >= 0)
                    {
                        result = tmpName.Substring(p1 + 3, p2 - p1 - 3);
                    }
                }
            }
            if (string.IsNullOrEmpty(result)) { result = "1"; }
            return result;
        }


        private void CheckReaderOnBoard_alt(int index)
        {
            //Reader_Typ = 3;
            if (index == 0)
            {
                Reader_Typ = 1;
                CheckReaderOnBoard_alt(1);
                if (Reader_Typ == 0)
                {
                    Reader_Typ = 2;
                    CloseReader(0);
                    CheckReaderOnBoard_alt(1);
                }
                else
                {
                    return;
                }

                if (Reader_Typ == 0)
                {
                    Reader_Typ = 3;
                    CloseReader(0);
                    CheckReaderOnBoard_alt(1);
                }
                else
                {
                    return;
                }
            }
            if (index > 0)
            {
                CloseReader(1);
                lbReaderInfo.Text = "Search Reader...";
                lbReaderInfo.Refresh();
                btReader.Text = "Reader Config.";
                btReader.Refresh();
                btData.Text = "Read/Write Data";
                btData.Refresh();
                IsReaderActive = false;
                if (Reader_Typ == 1)
                {
                    lbReaderTyp.Text = "LF-Reader";
                    lbReaderTyp.Refresh();
                    IsReaderActive = InitReaderLF();
                    if (IsReaderActive)
                    {
                        System.Threading.Thread.Sleep(100);
                        System.Windows.Forms.Application.DoEvents();
                        lbReaderInfo.Text = MyBTLF.GetVersion().Trim();
                        lbReaderInfo.Refresh(); 
                        if (lbReaderInfo.Text == "")
                        {
                            lbReaderInfo.Text = "No Reader found";
                            lbReaderInfo.Refresh();
                            lbReaderTyp.Text = "Reader-Typ:";
                            lbReaderTyp.Refresh();                           
                            CloseReader(0);
                            Reader_Typ = 0;
                            return;
                        }
                        btReader.Text = "LF Reader Config.";
                        btReader.Refresh();
                        btData.Text = "LF Read/Write Data";
                        btData.Refresh();
                    }
                    else
                    {
                        lbReaderInfo.Text = "No Reader found";
                        lbReaderInfo.Refresh();
                        lbReaderTyp.Text = "Reader-Typ:";
                        lbReaderTyp.Refresh(); 
                        Reader_Typ = 0;
                    }
                }
                if (Reader_Typ == 2)
                {
                    lbReaderTyp.Text = "HF-Reader";
                    lbReaderTyp.Refresh(); 
                    IsReaderActive = InitReaderHF();
                    if (IsReaderActive)
                    {

                        System.Threading.Thread.Sleep(100);
                        System.Windows.Forms.Application.DoEvents();
                        lbReaderInfo.Text = RFID_HF.GetVersion();
                        lbReaderInfo.Refresh();
                        if (lbReaderInfo.Text == "")
                        {
                            lbReaderInfo.Text = "No Reader found";
                            lbReaderInfo.Refresh();
                            lbReaderTyp.Text = "Reader-Typ:";
                            lbReaderTyp.Refresh();
                            CloseReader(0);
                            Reader_Typ = 0;
                            return;
                        }
                        btReader.Text = "HF Reader Config.";
                        btReader.Refresh();
                        btData.Text = "HF Read/Write Data";
                        btData.Refresh();
                    }
                    else
                    {
                        lbReaderInfo.Text = "No Reader found";
                        lbReaderInfo.Refresh();
                        lbReaderTyp.Text = "Reader-Typ:";
                        lbReaderTyp.Refresh(); 
                        Reader_Typ = 0;
                    }
                }
                if (Reader_Typ == 3)
                {
                    lbReaderTyp.Text = "UHF-Reader";
                    lbReaderTyp.Refresh(); 
                    IsReaderActive = InitReaderUHF_USB();
                    if (IsReaderActive)
                    {
                        if (RFID_UHF.GetReaderInfo(0) & RFID_UHF.IsUHF)
                        {
                            lbReaderInfo.Text = RFID_UHF.InfoReader[2];
                            lbReaderInfo.Refresh();
                            btReader.Text = "UHF Reader Config.";
                            btReader.Refresh();
                            btData.Text = "UHF Read/Write Data";
                            btData.Refresh();
                        }
                        else
                        {
                            lbReaderInfo.Text = "No Reader found";
                            lbReaderInfo.Refresh();
                            lbReaderTyp.Text = "Reader-Typ:";
                            lbReaderTyp.Refresh(); 
                            Reader_Typ = 0;
                            CheckVersionUHF();
                        }
                        RFID_UHF.GetUID();
                    }
                    else
                    {
                        lbReaderInfo.Text = "No Reader found";
                        lbReaderInfo.Refresh();
                        lbReaderTyp.Text = "Reader-Typ:";
                        lbReaderTyp.Refresh(); 
                        Reader_Typ = 0;
                    }
                }
            }
        }

        private void CheckVersionUHF()
        {
            if (IsReaderActive)
            {
                if (RFID_UHF.GetReaderInfo(0) & RFID_UHF.IsUHF)
                {
                    lbReaderInfo.Text = RFID_UHF.InfoReader[2];
                    lbReaderInfo.Refresh();
                    btReader.Text = "UHF Reader Config.";
                    btReader.Refresh();
                    btData.Text = "UHF Read/Write Data";
                    btData.Refresh();
                    Reader_Typ = 3;
                }
                else
                {
                    lbReaderInfo.Text = "No Reader found";
                    lbReaderInfo.Refresh();
                    lbReaderTyp.Text = "Reader-Typ:";
                    lbReaderTyp.Refresh(); 
                }
            }
        }

        private bool InitReaderLF()
        {
            bool Result = false;
            try
            {
                MyBTLF.BaudRate = 9600; 
                MyBTLF.Comport = "COM" + LFComport.ToString() ;
                MyBTLF.Active = true;
                Result = MyBTLF.Active;
            }
            catch (Exception ex)
            {
                lbReaderInfo.Text = ex.Message;
                lbReaderInfo.Refresh(); 
            }
            return Result;
        }

        public bool InitReaderHF()
        {
            bool Result = false;
            try
            {
                RFID_HF.BaudRate = 19200;
                RFID_HF.Comport = "COM" + LFComport.ToString();
                RFID_HF.Active = true;
                Result = RFID_HF.Active; 
            }
            catch (Exception ex)
            {
                lbReaderInfo.Text = ex.Message;
                lbReaderInfo.Refresh();
            }
            return Result;
        }

        public bool InitReaderUHF_USB()
        {
            bool Result = false;
            try
            {
                RFID_UHF.ReaderUnload();
                RFID_UHF.InitializeUSB();
                Result = RFID_UHF.IsActive;
            }
            catch (Exception ex)
            {
                lbReaderInfo.Text = ex.Message;
                lbReaderInfo.Refresh(); 
            }
            return Result;
        }

        private void CloseAllForms()
        {

            if ((frmLFDlg != null))
            {
                frmLFDlg.Close();
                frmLFDlg.Dispose();
                frmLFDlg = null;
            }
            if ((frmRFIDDataDlg != null))
            {
                if ((frmRFIDDataDlg.frmDataDlg != null))
                {
                    frmRFIDDataDlg.frmDataDlg.Close();
                    frmRFIDDataDlg.frmDataDlg.Dispose();
                    frmRFIDDataDlg.frmDataDlg = null;
                }
                frmRFIDDataDlg.Close();
                frmRFIDDataDlg.Dispose();
                frmRFIDDataDlg = null;
            }
            if ((frmHFDlg != null))
            {
                frmHFDlg.Close();
                frmHFDlg.Dispose();
                frmHFDlg = null;
            }
            if ((frmUHFDlg != null))
            {
                frmUHFDlg.Close();
                frmUHFDlg.Dispose();
                frmUHFDlg = null;
            }
        }

        private void btData_Click(object sender, EventArgs e)
        {
            if (Reader_Typ == 0)
            {
                //MsgBox("No Rader found" & vbCrLf & _
                //      "Please initialize first a Reader")
                return;
            }
            frmRFIDDataDlg = new frmRFIDData(this);
            if (Reader_Typ == 1)
            {
                frmRFIDDataDlg.lbInfo1.Text = TagName;
                frmRFIDDataDlg.TagInfo = TagName;
                if (MyBTLF.Active)
                {
                    frmRFIDDataDlg.lbReaderInfo.Text = MyBTLF.GetVersion().Trim();
                    frmRFIDDataDlg.lbReaderInfo.Refresh(); 
                }
            }
            if (Reader_Typ == 2)
            {
                if (HF_Transponder_Typ == 0)
                {
                    frmRFIDDataDlg.lbInfo1.Text = "ISO 15693 - RW";
                    frmRFIDDataDlg.TagInfo = "ISO 15693 - RW";
                }
                if (HF_Transponder_Typ == 1)
                {
                    frmRFIDDataDlg.lbInfo1.Text = "ISO 14443 (Mifare) - RW";
                    frmRFIDDataDlg.TagInfo = "ISO 14443 (Mifare) - RW";
                }
                if (RFID_HF.Active)
                {
                    frmRFIDDataDlg.lbReaderInfo.Text = RFID_HF.GetVersion(); 
                    frmRFIDDataDlg.lbReaderInfo.Refresh(); 
                }
            }
            if (Reader_Typ == 3)
            {
                frmRFIDDataDlg.lbInfo1.Text = "EPC Gen 2";
                frmRFIDDataDlg.TagInfo = "EPC Gen 2";
                if (RFID_UHF.IsActive)
                {
                    if (RFID_UHF.GetReaderInfo(0))
                    {
                        frmRFIDDataDlg.lbReaderInfo.Text = RFID_UHF.InfoReader[2];
                        frmRFIDDataDlg.lbReaderInfo.Refresh(); 
                    }
                }
            }
            frmRFIDDataDlg.ShowDialog();
            if ((frmRFIDDataDlg != null))
            {
                frmRFIDDataDlg.Dispose();
                frmRFIDDataDlg = null;
            }
        }

        public void SC(int Index)
        {
            if (Index == 0) {Cursor.Current = Cursors.Default;}
            if (Index == 1) {Cursor.Current = Cursors.WaitCursor;}
        }

        public int GetRecordCount()
        {
            int functionReturnValue = 0;
            System.Data.SqlServerCe.SqlCeCommand cmd = null;
            System.Data.SqlServerCe.SqlCeDataReader rs = null;
            functionReturnValue = 0;
            cmd = new System.Data.SqlServerCe.SqlCeCommand();
            cmd.Connection = conn_bar;
            cmd.CommandText = "SELECT COUNT(VarField1) FROM tblDaten";
            rs = cmd.ExecuteReader();
            rs.Read();
            functionReturnValue = rs.GetInt32(0);
            rs.Close();
            rs.Dispose();
            rs = null;
            cmd.Dispose();
            cmd = null;
            return functionReturnValue;
        }

        private void InfoItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show(this.Text + ControlChars.CrLf + "12.06.2013"); 
        }

        private void HideItem_Click(object sender, EventArgs e)
        {
            this.Hide();
            this.Tray1.InitialDuration = 0;
            this.Tray1.Visible = true;
        }

        private void Tray1_ResponseSubmitted(object sender, Microsoft.WindowsCE.Forms.ResponseSubmittedEventArgs e)
        {
            if (e.Response == "btExit")
            {
                Close();
                return;
            }
            if (e.Response == "btShow")
            {
                //this.Tray1.Visible = false; 
                if (Reader_Typ == 3)
                {
                    if (frmUHFDlg != null)
                    {
                        this.Show();
                        frmUHFDlg.Show();
                        return; 
                    }
                }
                if (Reader_Typ == 1)
                {
                   if (frmLFDlg != null)
                   {
                       this.Show();
                       frmLFDlg.Show();
                       return; 
                   }
                }
                if (Reader_Typ == 2)
                {
                    if (frmHFDlg != null)
                    {
                        this.Show();
                        frmHFDlg.Show();
                        return; 
                    }
                }
            }
        }

        private void SoundItem_Click(object sender, EventArgs e)
        {
            SoundItem.Checked = !SoundItem.Checked;   
        }

        private void InitItem_Click(object sender, EventArgs e)
        {
            PlaySound();
            Reader_Typ = 0;
            CheckReaderOnBoard();
        }

        private void LFItem_Click(object sender, EventArgs e)
        {
            PlaySound();
            Reader_Typ = 1;
            CheckReaderOnBoard();
        }

        private void HFItem_Click(object sender, EventArgs e)
        {
            PlaySound();
            Reader_Typ = 2;
            CheckReaderOnBoard();
        }

        private void UHFItem_Click(object sender, EventArgs e)
        {
            PlaySound();
            Reader_Typ = 3;
            CheckReaderOnBoard();
        }

        private void ChangeRFIDItem_Click(object sender, EventArgs e)
        {
            CloseReader(1);
            MessageBox.Show("Please change the Reader." + ControlChars.Cr + "Then they press OK.",this.Text, MessageBoxButtons.OK,MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
            PlaySound();
            Reader_Typ = 0;
            CheckReaderOnBoard();
        }
    }
}